@@ -20,6 +20,7 @@ echo ""
 ./abi-extract.pl -N zh-TW 
 ./abi-extract.pl -N cy-GB 
 ./abi-extract.pl -N fi-FI 
+./abi-extract.pl -N hr-HR 
 ./abi-extract.pl -N lt-LT 
 ./abi-extract.pl -N pt-PT 
 ./abi-extract.pl -N da-DK 
@@ -33,3 +34,4 @@ echo ""
 ./abi-extract.pl -N ast-ES
 ./abi-extract.pl -N ku
 ./abi-extract.pl -N ms-MY
+./abi-extract.pl -N ne-NP
