@@ -29,7 +29,9 @@
 #include "xap_Types.h"
 #include "ev_QNXMenu.h"
 #include "xap_QNXApp.h"
-#include "xap_QNXFrame.h"
+#include "xap_QNXFrameImpl.h"
+#include "ap_QNXFrameImpl.h"
+#include "xap_Frame.h"
 #include "ev_QNXKeyboard.h"
 #include "ev_Menu_Layouts.h"
 #include "ev_Menu_Actions.h"
@@ -51,7 +53,7 @@
 */
 
 static const char ** _ev_GetLabelName(XAP_QNXApp * pQNXApp,
-									  XAP_QNXFrame * pQNXFrame,
+									  XAP_Frame * pFrame,
 									  const EV_Menu_Action * pAction,
 									  const EV_Menu_Label * pLabel)
 {
@@ -64,7 +66,7 @@ static const char ** _ev_GetLabelName(XAP_QNXApp * pQNXApp,
 	const char * szLabelName;
 	
 	if (pAction->hasDynamicLabel())
-		szLabelName = pAction->getDynamicLabel(pQNXFrame,pLabel);
+		szLabelName = pAction->getDynamicLabel(pLabel);
 	else
 		szLabelName = pLabel->getMenuLabel();
 
@@ -84,7 +86,7 @@ static const char ** _ev_GetLabelName(XAP_QNXApp * pQNXApp,
 			EV_EditMethod * pEM = pEMC->findEditMethodByName(szMethodName);
 			UT_ASSERT(pEM);						// make sure it's bound to something
 
-			const EV_EditEventMapper * pEEM = pQNXFrame->getEditEventMapper();
+			const EV_EditEventMapper * pEEM = pQNXApp->getEditEventMapper();
 			UT_ASSERT(pEEM);
 
 			const char * string = pEEM->getShortcutFor(pEM);
@@ -119,12 +121,12 @@ static const char ** _ev_GetLabelName(XAP_QNXApp * pQNXApp,
 
 /*****************************************************************/
 
-EV_QNXMenu::EV_QNXMenu(XAP_QNXApp * pQNXApp, XAP_QNXFrame * pQNXFrame,
+EV_QNXMenu::EV_QNXMenu(XAP_QNXApp * pQNXApp, XAP_Frame * pFrame,
 					   const char * szMenuLayoutName,
 					   const char * szMenuLabelSetName)
 	: EV_Menu(pQNXApp, pQNXApp->getEditMethodContainer(), szMenuLayoutName, szMenuLabelSetName),
 	  m_pQNXApp(pQNXApp),
-	  m_pQNXFrame(pQNXFrame)
+	  m_pFrame(pFrame)
 {
 }
 
@@ -133,9 +135,9 @@ EV_QNXMenu::~EV_QNXMenu()
 	m_vecMenuWidgets.clear();
 }
 
-XAP_QNXFrame * EV_QNXMenu::getFrame()
+XAP_Frame * EV_QNXMenu::getFrame()
 {
-	return m_pQNXFrame;
+	return m_pFrame;
 }
 
 bool EV_QNXMenu::menuEvent(XAP_Menu_Id id)
@@ -160,27 +162,12 @@ bool EV_QNXMenu::menuEvent(XAP_Menu_Id id)
 	EV_EditMethod * pEM = pEMC->findEditMethodByName(szMethodName);
 	UT_ASSERT(pEM);						// make sure it's bound to something
 
-	//Right away switch the focus back to the document, do this before
-	//we invoke the menu method since the menu method could tell us to 
-	//self destruct (ie a close selection) on the frame.
-	m_pQNXFrame->setDocumentFocus();
-
 	UT_String script_name(pAction->getScriptName());
-	invokeMenuMethod(m_pQNXFrame->getCurrentView(), pEM, script_name);
+	invokeMenuMethod(m_pFrame->getCurrentView(), pEM, script_name);
 
 	return true;
 }
 
-static const char * _ev_FakeName(const char * sz, UT_uint32 k)
-{
-	// construct a temporary string
-
-	static char buf[128];
-	UT_ASSERT(strlen(sz)<120);
-	sprintf(buf,"%s%d",sz,k);
-	return buf;
-}
-
 static char _ev_convert(char * bufResult,
 			const char * szString)
 {
@@ -212,7 +199,7 @@ static int s_menu_select(PtWidget_t *widget, void *data, PtCallbackInfo_t *info)
 		struct _cb_menu *mcb = (struct _cb_menu *)data;
 		UT_ASSERT(mcb && mcb->qnxmenu);
 
-		XAP_QNXFrame * pFrame = mcb->qnxmenu->getFrame();
+		XAP_Frame * pFrame = mcb->qnxmenu->getFrame();
 		UT_ASSERT(pFrame);
 
 		EV_Menu_Label * pLabel = mcb->qnxmenu->getLabelSet()->getLabel(mcb->id);
@@ -232,14 +219,14 @@ static int s_menu_select(PtWidget_t *widget, void *data, PtCallbackInfo_t *info)
 	
 static int s_menu_deselect(PtWidget_t *widget, void *data, PtCallbackInfo_t *info)
 {
-		struct _cb_menu *mcb = (struct _cb_menu *)data;
+		EV_QNXMenu *pQNXMenu = (EV_QNXMenu *)data;
 
-		UT_ASSERT(mcb && mcb->qnxmenu);
-
-		XAP_QNXFrame * pFrame = mcb->qnxmenu->getFrame();
+		XAP_Frame * pFrame = pQNXMenu->getFrame();
 		UT_ASSERT(pFrame);
 
 		pFrame->setStatusMessage(NULL);
+		static_cast<AP_QNXFrameImpl *>(pFrame->getFrameImpl())->setDocumentFocus();
+
 		return Pt_CONTINUE;
 }
 
@@ -340,7 +327,7 @@ bool EV_QNXMenu::synthesizeMenu(PtWidget_t * wMenuRoot)
 		{
 		case EV_MLF_Normal:
 		{
-			const char ** data = _ev_GetLabelName(m_pQNXApp, m_pQNXFrame, pAction, pLabel);
+			const char ** data = _ev_GetLabelName(m_pQNXApp, m_pFrame, pAction, pLabel);
 			szLabelName = data[0];
 			szMnemonicName = data[1];
 
@@ -361,7 +348,7 @@ bool EV_QNXMenu::synthesizeMenu(PtWidget_t * wMenuRoot)
 					PtSetArg(&args[n++], Pt_ARG_ACCEL_TEXT, szMnemonicName, 0); 
 				}
 
-				if (pAction->isCheckable()) {
+				if (pAction->isCheckable() || pAction->isRadio()) {
 					PtSetArg(&args[n++], Pt_ARG_FLAGS,
 							Pt_MENU_BUTTON | Pt_AUTOHIGHLIGHT | Pt_SET, 
 							Pt_MENU_BUTTON | Pt_AUTOHIGHLIGHT | Pt_SET);
@@ -378,7 +365,6 @@ bool EV_QNXMenu::synthesizeMenu(PtWidget_t * wMenuRoot)
 				mcb->qnxmenu = this;
 				PtAddCallback(wbutton, Pt_CB_ACTIVATE, s_menu_activate, mcb);
 				PtAddCallback(wbutton, Pt_CB_ARM, s_menu_select, mcb);
-				PtAddCallback(wbutton, Pt_CB_DISARM, s_menu_deselect, mcb);
 
 				if (szMnemonicName && *szMnemonicName && get_hotkey_key(szMnemonicName) != '\0') {
 					PtAddHotkeyHandler(PtGetParent(wMenuRoot, PtWindow),
@@ -398,7 +384,7 @@ bool EV_QNXMenu::synthesizeMenu(PtWidget_t * wMenuRoot)
 		}
 		case EV_MLF_BeginSubMenu:
 		{
-			const char ** data = _ev_GetLabelName(m_pQNXApp, m_pQNXFrame, pAction, pLabel);
+			const char ** data = _ev_GetLabelName(m_pQNXApp, m_pFrame, pAction, pLabel);
 			szLabelName = data[0];
 
 			PtWidget_t * wParent, *wbutton, *wmenu;
@@ -413,24 +399,24 @@ bool EV_QNXMenu::synthesizeMenu(PtWidget_t * wMenuRoot)
 
 				// create the item widget
 				int n = 0;
-				PtSetArg(&args[n], Pt_ARG_TEXT_STRING, buf, 0); n++;
-				PtSetArg(&args[n], Pt_ARG_ACCEL_KEY, accel, 0); n++;
-				//PtSetArg(&args[n], Pt_ARG_ACCEL_KEY, accel, 0); n++;
-				if (wParent != wMenuRoot) {
-					PtSetArg(&args[n], Pt_ARG_BUTTON_TYPE, Pt_MENU_RIGHT, Pt_MENU_RIGHT); n++;
+				PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, buf, 0); 
+				PtSetArg(&args[n++], Pt_ARG_ACCEL_KEY, accel, 0);
+				if ((PtGetParent(wParent,PtMenuBar) == NULL) || wParent != wMenuRoot) {
+					PtSetArg(&args[n++], Pt_ARG_BUTTON_TYPE, Pt_MENU_RIGHT, Pt_MENU_RIGHT);
 				}
 				wbutton = PtCreateWidget(PtMenuButton, wParent, n, args); 
 
 				n = 0;
-				if (wParent != wMenuRoot) {
+				if ((PtGetParent(wParent,PtMenuBar) == NULL) || wParent != wMenuRoot) {
 					PtSetArg(&args[n], Pt_ARG_MENU_FLAGS, 
 							Pt_MENU_CHILD|Pt_MENU_AUTO, 
 							Pt_MENU_CHILD|Pt_MENU_AUTO); n++;
 				}
 				wmenu =  PtCreateWidget(PtMenu, wbutton, n, args); 
-				if (wParent == wMenuRoot) {
+				if ((PtGetParent(wParent,PtMenuBar) && wParent == wMenuRoot)) {
 					PtAddCallback(wbutton, Pt_CB_ARM, s_menu_init, this);
 					PtAddCallback(wbutton, Pt_CB_ARM, s_menu_appear, wmenu);
+					PtAddCallback(wmenu,Pt_CB_UNREALIZED,s_menu_deselect,this);
 					PtAddHotkeyHandler(PtGetParent(wMenuRoot, PtWindow), 
 										tolower(accel[0]), Pk_KM_Alt, 
 										0, wmenu, s_menu_appear); 
@@ -451,7 +437,11 @@ bool EV_QNXMenu::synthesizeMenu(PtWidget_t * wMenuRoot)
 			PtWidget_t * w;
 			bResult = stack.pop((void **)&w);
 			UT_ASSERT(bResult);
-
+			
+			if(w == NULL) {
+			stack.push(wMenuRoot);
+			w= wMenuRoot;
+			}
 			//Add the same widget that was the start of the menu
 			m_vecMenuWidgets.addItem(w);
 			break;
@@ -467,10 +457,8 @@ bool EV_QNXMenu::synthesizeMenu(PtWidget_t * wMenuRoot)
 			m_vecMenuWidgets.addItem(w);
 			break;
 		}
-
 		case EV_MLF_BeginPopupMenu:
 		case EV_MLF_EndPopupMenu:
-			//Reserve slot for later
 			m_vecMenuWidgets.addItem(NULL);	
 			break;
 			
@@ -489,7 +477,7 @@ static void set_menu_enabled(PtWidget_t *w, int enabled, int checked) {
 
 /* Photon funniness ... why can't ghosting work? */
 	flags =  (enabled == true) ? Pt_SELECTABLE : (Pt_BLOCKED | Pt_GHOST)  ;
-	flags |= (checked == true) ? Pt_SET : 0;
+	flags |= (checked == true) ? Pt_SET : Pt_FALSE;
 	PtSetResource(w, Pt_ARG_FLAGS, flags, Pt_BLOCKED | Pt_GHOST | Pt_SET | Pt_SELECTABLE);
 }
 
@@ -549,7 +537,7 @@ bool EV_QNXMenu::_refreshMenu(AV_View * pView, void * wMenuRoot)
 				UT_ASSERT((k < m_vecMenuWidgets.getItemCount() - 1));
 
 				// Get the dynamic label
-				const char ** data = _ev_GetLabelName(m_pQNXApp, m_pQNXFrame, pAction, pLabel);
+				const char ** data = _ev_GetLabelName(m_pQNXApp, m_pFrame, pAction, pLabel);
 				const char * szLabelName = data[0];
 				
 				// Get the item we are pointing at
@@ -607,11 +595,6 @@ bool EV_QNXMenu::_refreshMenu(AV_View * pView, void * wMenuRoot)
 						n = 0;
 						PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, labelbuf, 0); 
 						PtSetArg(&args[n++], Pt_ARG_ACCEL_KEY, accel, 0); 
-						/*
-						if (szMnemonicName && *szMnemonicName) {
-							PtSetArg(&args[n++], Pt_ARG_ACCEL_TEXT, szMnemonicName, 0); 
-						}
-						*/
 
 						if (pAction->isCheckable()) {
 							PtSetArg(&args[n++], Pt_ARG_FLAGS,
@@ -633,16 +616,6 @@ bool EV_QNXMenu::_refreshMenu(AV_View * pView, void * wMenuRoot)
 						mcb->qnxmenu = this;
 						PtAddCallback(item, Pt_CB_ACTIVATE, s_menu_activate, mcb);
 						PtAddCallback(item, Pt_CB_ARM, s_menu_select, mcb);
-						PtAddCallback(item, Pt_CB_DISARM, s_menu_deselect, mcb);
-
-						/*
-						if (szMnemonicName && *szMnemonicName && get_hotkey_key(szMnemonicName) != '\0') {
-							PtAddHotkeyHandler(PtGetParent(wMenuRoot, PtWindow),
-									   get_hotkey_key(szMnemonicName), 
-									   get_hotkey_code(szMnemonicName),
-										0, mcb, s_menu_activate); 
-						}
-						*/
 
 						//Now that we have an item, add it to the vector
 						m_vecMenuWidgets.setNthItem(k, item, NULL);
@@ -720,10 +693,10 @@ bool EV_QNXMenu::_doAddMenuItem(UT_uint32 layout_pos)
 /*****************************************************************/
 
 EV_QNXMenuBar::EV_QNXMenuBar(XAP_QNXApp * pQNXApp,
-							   XAP_QNXFrame * pQNXFrame,
+							   XAP_Frame * pFrame,
 							   const char * szMenuLayoutName,
 							   const char * szMenuLabelSetName)
-	: EV_QNXMenu(pQNXApp,pQNXFrame,szMenuLayoutName,szMenuLabelSetName)
+	: EV_QNXMenu(pQNXApp,pFrame,szMenuLayoutName,szMenuLabelSetName)
 {
 	m_wMenuBar = NULL;
 }
@@ -739,7 +712,12 @@ EV_QNXMenuBar::~EV_QNXMenuBar(void)
 
 bool EV_QNXMenuBar::synthesizeMenuBar(void)
 {
-	m_wMenuBar = PtCreateWidget(PtMenuBar, m_pQNXFrame->getTBGroupWidget(), 0, NULL);
+	 PtArg_t args[2];
+	 int		 n=0;
+	 XAP_QNXFrameImpl *pQNXFrameImpl = static_cast<XAP_QNXFrameImpl *>(m_pFrame->getFrameImpl()); 
+
+	PtSetArg(&args[n++],Pt_ARG_FLAGS,Pt_FALSE,Pt_GETS_FOCUS);
+	m_wMenuBar = PtCreateWidget(PtMenuBar, pQNXFrameImpl->getTBGroupWidget(), n, args);
 	synthesizeMenu(m_wMenuBar);
 	return true;
 }
@@ -758,10 +736,10 @@ bool EV_QNXMenuBar::refreshMenu(AV_View * pView)
 /*****************************************************************/
 
 EV_QNXMenuPopup::EV_QNXMenuPopup(XAP_QNXApp * pQNXApp,
-								   XAP_QNXFrame * pQNXFrame,
+								   XAP_Frame * pFrame,
 								   const char * szMenuLayoutName,
 								   const char * szMenuLabelSetName)
-	: EV_QNXMenu(pQNXApp,pQNXFrame,szMenuLayoutName,szMenuLabelSetName)
+	: EV_QNXMenu(pQNXApp,pFrame,szMenuLayoutName,szMenuLabelSetName)
 {
 	m_wMenuPopup = NULL;
 }
@@ -781,23 +759,25 @@ PtWidget_t * EV_QNXMenuPopup::getMenuHandle(void) const
 }
 
 static int popup_realized(PtWidget_t *w, void *data, PtCallbackInfo_t *info) {
-	XAP_QNXFrame *pQNXFrame = (XAP_QNXFrame *)data; 
-
+	XAP_Frame *pFrame = (XAP_Frame *)data; 
+	 PtWidget_t *top =(static_cast<XAP_QNXFrameImpl *>(pFrame->getFrameImpl())->getTopLevelWindow()); 
 	PtArg_t arg;
 	PtSetArg(&arg, Pt_ARG_FLAGS, Pt_BLOCKED, Pt_BLOCKED);
-	PtSetResources(pQNXFrame->getTopLevelWindow(), 1, &arg);
+	PtSetResources(top, 1, &arg);
 
 	return Pt_CONTINUE;
 }
 
 static int popup_unrealized(PtWidget_t *w, void *data, PtCallbackInfo_t *info) {
-	XAP_QNXFrame *pQNXFrame = (XAP_QNXFrame *)data; 
-
+	XAP_Frame *pFrame = (XAP_Frame *)data; 
+	 PtWidget_t *top =(static_cast<XAP_QNXFrameImpl *>(pFrame->getFrameImpl())->getTopLevelWindow()); 
 	PtArg_t arg;
-	PtSetArg(&arg, Pt_ARG_FLAGS, 0, Pt_BLOCKED);
-	PtSetResources(pQNXFrame->getTopLevelWindow(), 1, &arg);
+	PtSetArg(&arg, Pt_ARG_FLAGS,Pt_FALSE, Pt_BLOCKED);
+	PtSetResources(top, 1, &arg);
 
-	pQNXFrame->setPopupDone(1);
+	pFrame->setStatusMessage(NULL);
+	static_cast<AP_QNXFrameImpl *>(pFrame->getFrameImpl())->setDocumentFocus();
+	static_cast<XAP_QNXFrameImpl *>(pFrame->getFrameImpl())->setPopupDone(1);
 
 	return Pt_CONTINUE;
 }
@@ -807,12 +787,12 @@ bool EV_QNXMenuPopup::synthesizeMenuPopup()
 {
     PtArg_t args[10];
 	int 	n = 0;
-
+	 PtWidget_t *top =(static_cast<XAP_QNXFrameImpl *>(m_pFrame->getFrameImpl())->getTopLevelWindow()); 
 	m_wMenuPopup = PtCreateWidget(PtMenu, 
-								  m_pQNXFrame->getTopLevelWindow(),
+								  top,
 								  n, args);
-	PtAddCallback(m_wMenuPopup, Pt_CB_REALIZED, popup_realized, m_pQNXFrame);
-	PtAddCallback(m_wMenuPopup, Pt_CB_UNREALIZED, popup_unrealized, m_pQNXFrame);
+	PtAddCallback(m_wMenuPopup, Pt_CB_REALIZED, popup_realized, m_pFrame);
+	PtAddCallback(m_wMenuPopup, Pt_CB_UNREALIZED, popup_unrealized, m_pFrame);
 
 	synthesizeMenu(m_wMenuPopup);
 
