@@ -26,7 +26,10 @@
 #include "ev_QNXToolbar.h"
 #include "xap_Types.h"
 #include "xap_QNXApp.h"
-#include "xap_QNXFrame.h"
+#include "xap_QNXFrameImpl.h"
+#include "xap_Frame.h"
+#include "ap_QNXFrameImpl.h"
+#include "ut_iconv.h"
 #include "ev_Toolbar_Actions.h"
 #include "ev_Toolbar_Layouts.h"
 #include "ev_Toolbar_Labels.h"
@@ -37,12 +40,20 @@
 #include "xav_View.h"
 #include "ut_xml.h"
 #include "xap_Prefs.h"
+#include "xap_QNXFontPreview.h"
 
+#include "ap_Toolbar_Id.h"
 
 /*****************************************************************/
+PtWidget_t * popupBalloon(PtWidget_t *window,PtWidget_t *widget,int position,char *text,char *font,PgColor_t fill_color,PgColor_t text_color)
+{
+char *tooltip;
+PtGetResource(widget,Pt_ARG_USER_DATA,&tooltip,0);
+return (PtInflateBalloon(window,widget,position,tooltip,"TextFont09",fill_color,text_color));
+}
 /*****************************************************************/
 
-EV_QNXToolbar::EV_QNXToolbar(XAP_QNXApp * pQNXApp, XAP_QNXFrame * pQNXFrame,
+EV_QNXToolbar::EV_QNXToolbar(XAP_QNXApp * pQNXApp, XAP_Frame * pFrame,
 							   const char * szToolbarLayoutName,
 							   const char * szToolbarLabelSetName)
 	: EV_Toolbar(pQNXApp->getEditMethodContainer(),
@@ -50,9 +61,10 @@ EV_QNXToolbar::EV_QNXToolbar(XAP_QNXApp * pQNXApp, XAP_QNXFrame * pQNXFrame,
 				 szToolbarLabelSetName)
 {
 	m_pQNXApp = pQNXApp;
-	m_pQNXFrame = pQNXFrame;
+	m_pFrame = pFrame;
 	m_pViewListener = 0;
 	m_lid = 0;							// view listener id
+	m_pFontPreview = 0;
 }
 
 EV_QNXToolbar::~EV_QNXToolbar(void)
@@ -62,7 +74,7 @@ EV_QNXToolbar::~EV_QNXToolbar(void)
 }
 
 bool EV_QNXToolbar::toolbarEvent(XAP_Toolbar_Id id, 
-				 					char * pData,
+				 					UT_UCS4String pData,
 				 					UT_uint32 dataLength)
 
 {
@@ -76,10 +88,11 @@ bool EV_QNXToolbar::toolbarEvent(XAP_Toolbar_Id id,
 	const EV_Toolbar_Action * pAction = pToolbarActionSet->getAction(id);
 	UT_ASSERT(pAction);
 
-	AV_View * pView = m_pQNXFrame->getCurrentView();
+	AV_View * pView = m_pFrame->getCurrentView();
 
 	//Right away switch the focus
-	m_pQNXFrame->setDocumentFocus();
+	static_cast<AP_QNXFrameImpl *>(m_pFrame->getFrameImpl())->setDocumentFocus();
+
 
 	// make sure we ignore presses on "down" group buttons
 	if (pAction->getItemType() == EV_TBIT_GroupButton) 
@@ -108,7 +121,7 @@ bool EV_QNXToolbar::toolbarEvent(XAP_Toolbar_Id id,
 	EV_EditMethod * pEM = pEMC->findEditMethodByName(szMethodName);
 	UT_ASSERT(pEM);						// make sure it's bound to something
 
-	invokeToolbarMethod(pView,pEM,(UT_UCSChar*)pData,dataLength);
+	invokeToolbarMethod(pView,pEM,pData.ucs4_str(),dataLength);
 	return true;
 }
 
@@ -120,52 +133,72 @@ struct _cb_data {
 
 static int s_combo_select(PtWidget_t *w, void *data, PtCallbackInfo_t *info) {
 
-	if (info->reason_subtype != Pt_LIST_SELECTION_FINAL) {
-		return 0;
-	}
-
 	PtListCallback_t *cb = (PtListCallback_t *)info->cbdata;
-
 	struct _cb_data *cb_data = (struct _cb_data *)data;
-	cb_data->tb->toolbarEvent(cb_data->id, cb->item, (UT_uint32)cb->item_len);
 
+	if(info->reason_subtype == Pt_LIST_SELECTION_BROWSE) {
+		if(cb_data->id == AP_TOOLBAR_ID_FMT_FONT)
+		{
+			if(cb_data->tb->m_pFontPreview == NULL)
+			{
+				short x,y;
+				PhArea_t* area; 
+
+				PtGetAbsPosition(w,&x,&y);
+				PtGetResource(w,Pt_ARG_AREA,&area,0);
+		
+				x += area->size.w;
+				y += area->size.h;
+
+				XAP_Frame *pFrame = static_cast<XAP_Frame *>(cb_data->tb->getFrame());
+				cb_data->tb->m_pFontPreview = new XAP_QNXFontPreview(pFrame,x,y);
+			}
+			cb_data->tb->m_pFontPreview->setFontFamily(cb->item);
+			cb_data->tb->m_pFontPreview->draw();
+		}
+	}
+	
+	if (info->reason_subtype == Pt_LIST_SELECTION_FINAL) {
+		cb_data->tb->toolbarEvent(cb_data->id, cb->item, (UT_uint32)cb->item_len);
+	}
 	return Pt_CONTINUE;
 }
 
+static int s_combo_list_close(PtWidget_t *w,void *data,PtCallbackInfo_t *info)
+{
+	struct _cb_data *cb_data = (struct _cb_data *)data;
+
+	if((cb_data->id == AP_TOOLBAR_ID_FMT_FONT) && cb_data->tb->m_pFontPreview != NULL)
+	{
+		DELETEP(cb_data->tb->m_pFontPreview);
+	}
+return Pt_CONTINUE;
+}
 static int s_button_activate(PtWidget_t *w, void *data, PtCallbackInfo_t *info) {
 
 	struct _cb_data *cb_data = (struct _cb_data *)data;
-	cb_data->tb->toolbarEvent(cb_data->id, 0, 0);
+	cb_data->tb->toolbarEvent(cb_data->id, (const char *)NULL, 0);
 
 	return Pt_CONTINUE;
 }
 
 static int s_colour_activate(PtWidget_t *w, void *data, PtCallbackInfo_t *info) {
-	int					ret;
-	char				*title;
-	char				oklabel[3];
-	char				cancellabel[8];
-	PtColorSelectInfo_t cinfo;
 
 	struct _cb_data *cb_data = (struct _cb_data *)data;
+	XAP_Toolbar_Id id = cb_data->id;
+	XAP_QNXApp * pQNXApp = cb_data->tb->getApp();
+	const EV_EditMethodContainer * pEMC = pQNXApp->getEditMethodContainer();
+	UT_ASSERT(pEMC);
+	EV_EditMethod * pEM = NULL;
 
-	memset(&cinfo, 0, sizeof(cinfo));
-	cinfo.flags = Pt_COLORSELECT_MODAL;
-	strcpy(oklabel, "OK");
-	cinfo.accept_text = oklabel;
-	strcpy(cancellabel, "Cancel");
-	cinfo.dismiss_text = cancellabel;
+	AV_View * pView = cb_data->tb->getFrame()->getCurrentView();
 
-	title = NULL;
-	PtGetResource(w, Pt_ARG_TEXT_STRING, &title, 0);
+	if(id ==  AP_TOOLBAR_ID_COLOR_FORE)
+		pEM = pEMC->findEditMethodByName("dlgColorPickerFore");
+	else
+    pEM = pEMC->findEditMethodByName("dlgColorPickerBack");
+	cb_data->tb->invokeToolbarMethod(pView,pEM,NULL,0);
 
-	ret = PtColorSelect(w, (title) ? title : "Select a colour", &cinfo);
-	if(ret & Pt_COLORSELECT_ACCEPT) {
-		char clrstr[9];
-		sprintf(clrstr, "%02x%02x%02x", PgRedValue(cinfo.rgb), PgGreenValue(cinfo.rgb), PgBlueValue(cinfo.rgb));
-		//printf("Colour is [%s] len %d \n", clrstr, strlen(clrstr));
-		cb_data->tb->toolbarEvent(cb_data->id, clrstr, strlen(clrstr));
-	} 
 	return Pt_CONTINUE;
 }
 
@@ -191,30 +224,29 @@ bool EV_QNXToolbar::synthesize(void)
 	//PtWidget_t * wTLW = m_pQNXFrame->getTopLevelWindow();
 	//PtWidget_t * wVBox = m_pQNXFrame->getVBoxWidget();
 
-#if 0
 	const XML_Char * szValue = NULL;
 	m_pQNXApp->getPrefsValue(XAP_PREF_KEY_ToolbarAppearance,&szValue);
 	UT_ASSERT((szValue) && (*szValue));
-	GtkToolbarStyle style = GTK_TOOLBAR_ICONS;
+
+	char style = Pt_IMAGE;
+
 	if (UT_XML_stricmp(szValue,"icon")==0)
 		style = Pt_IMAGE;
 	else if (UT_XML_stricmp(szValue,"text")==0)
 		style = Pt_Z_STRING;
 	else if (UT_XML_stricmp(szValue,"both")==0)
 		style = Pt_TEXT_IMAGE;
-#endif
 	
-
-	m_wToolbarGroup = m_pQNXFrame->getTBGroupWidget();
+	XAP_QNXFrameImpl * pQNXFrameImpl = static_cast<XAP_QNXFrameImpl *>(m_pFrame->getFrameImpl()); 
+	m_wToolbarGroup = pQNXFrameImpl->getTBGroupWidget();
 
 	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TOOLBAR_FLAGS, 0, Pt_TOOLBAR_FOLLOW_FOCUS);
 	PtSetArg(&args[n++], Pt_ARG_TOOLBAR_LAYOUT_FLAGS, 
 						Pt_TOOLBAR_FROM_LINE_START, 
 						Pt_TOOLBAR_FROM_LINE_START);
 	PtSetArg(&args[n++], Pt_ARG_TOOLBAR_FLAGS, 
 						Pt_TOOLBAR_ITEM_SEPARATORS, 
-						Pt_TOOLBAR_ITEM_SEPARATORS );
+						Pt_TOOLBAR_ITEM_SEPARATORS|Pt_TOOLBAR_FOLLOW_FOCUS );
 	m_wToolbar = PtCreateWidget(PtToolbar, m_wToolbarGroup, n, args);
 	UT_ASSERT(m_wToolbar);
 
@@ -263,28 +295,23 @@ bool EV_QNXToolbar::synthesize(void)
 					}
 
 					n = 0;
-					//This would add a ? help topic thingy
-					//PtSetArg(&args[n++], Pt_ARG_HELP_TOPIC, szToolTip, 0);
-					//PtSetArg(&args[n++], Pt_ARG_EFLAGS, Pt_INTERNAL_HELP, Pt_INTERNAL_HELP);
-					//This will add the balloon help
 					PtSetArg(&args[n++], Pt_ARG_LABEL_FLAGS, Pt_SHOW_BALLOON, Pt_SHOW_BALLOON); 
-					PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, szToolTip, 0); 
+					PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, pLabel->getToolbarLabel(), 0); 
+					PtSetArg(&args[n++], Pt_ARG_LABEL_BALLOON,&popupBalloon,0);
+					PtSetArg(&args[n++], Pt_ARG_BALLOON_POSITION,Pt_BALLOON_BOTTOM,0);
+					PtSetArg(&args[n++], Pt_ARG_TEXT_FONT,"TextFont06",0);
 
 					PtSetArg(&args[n++], Pt_ARG_FILL_COLOR, Pg_TRANSPARENT, 0);
-					PtSetArg(&args[n++], Pt_ARG_FLAGS, 0, Pt_HIGHLIGHTED);
+					PtSetArg(&args[n++], Pt_ARG_FLAGS, 0, Pt_HIGHLIGHTED | Pt_GETS_FOCUS);
+					PtSetArg(&args[n++], Pt_ARG_USER_DATA, szToolTip, strlen(szToolTip));
 
-					PtSetArg(&args[n++], Pt_ARG_FLAGS, 0, Pt_GETS_FOCUS); 
 					if (image) {
-						PtSetArg(&args[n++], Pt_ARG_LABEL_TYPE, Pt_IMAGE, Pt_IMAGE);
-						//Set the flags on the image to
-						//Ph_RELEASE_IMAGE | Ph_RELEASE_PALETTE | ...
-						//PtSetArg(&args[n++], Pt_ARG_ARM_DATA, image, sizeof(*image)); 
-						PtSetArg(&args[n++], Pt_ARG_LABEL_DATA, image, sizeof(*image)); 
-					}
-					else if(!szToolTip) {
-						PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, "No Label/Icon?", 0); 
+						PtSetArg(&args[n++], Pt_ARG_LABEL_TYPE, style, 0);
+						PtSetArg(&args[n++], Pt_ARG_LABEL_IMAGE, image, sizeof(*image)); 
 					}
+
 					tb = PtCreateWidget(PtButton, tbgroup, n, args);
+					FREEP(image);
 					if (tb) {
 						tcb = (struct _cb_data *)malloc(sizeof(*tcb));
 						tcb->tb = this;
@@ -311,9 +338,6 @@ bool EV_QNXToolbar::synthesize(void)
 					}
 
 					n = 0;
-					//This would add a ? help topic thingy
-					//PtSetArg(&args[n++], Pt_ARG_HELP_TOPIC, szToolTip, 0);
-					//PtSetArg(&args[n++], Pt_ARG_EFLAGS, Pt_INTERNAL_HELP, Pt_INTERNAL_HELP);
 					//This will add the balloon help
 					PtSetArg(&args[n++], Pt_ARG_LABEL_FLAGS, Pt_SHOW_BALLOON, Pt_SHOW_BALLOON); 
 					PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, szToolTip, 0); 
@@ -327,17 +351,14 @@ bool EV_QNXToolbar::synthesize(void)
 					PtSetArg(&args[n++], Pt_ARG_FLAGS, Pt_TOGGLE, Pt_TOGGLE); 
 					PtSetArg(&args[n++], Pt_ARG_FLAGS, 0, Pt_GETS_FOCUS); 
 					if (image) {
-						//For balloons set Pt_TEXT_IMAGE ...
 						PtSetArg(&args[n++], Pt_ARG_LABEL_TYPE, Pt_IMAGE, Pt_IMAGE);
-						//Set the flags on the image to
-						//Ph_RELEASE_IMAGE | Ph_RELEASE_PALETTE | ...
-						//PtSetArg(&args[n], Pt_ARG_ARM_DATA, image, sizeof(*image)); n++;
 						PtSetArg(&args[n++], Pt_ARG_LABEL_DATA, image, sizeof(*image)); 
 					}
 					else {
 						PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, "pbut", 0); 
 					}
 					tb = PtCreateWidget(PtButton, tbgroup, n, args);
+					FREEP(image);
 					if (tb) {
 						tcb = (struct _cb_data *)malloc(sizeof(*tcb));
 						tcb->tb = this;
@@ -369,18 +390,20 @@ bool EV_QNXToolbar::synthesize(void)
 				n = 0;
 				PtSetArg(&args[n++], Pt_ARG_WIDTH, iWidth, 0); 
 				PtSetArg(&args[n++], Pt_ARG_VISIBLE_COUNT, 6, 0); 
-				PtSetArg(&args[n++], Pt_ARG_FLAGS, 0, Pt_GETS_FOCUS); 
+//This shouldn't be done for ListBoxes as it will prevent the user from scrolling using the arrow keys.
+//				PtSetArg(&args[n++], Pt_ARG_FLAGS, 0, Pt_GETS_FOCUS); 
 				PtSetArg(&args[n++], Pt_ARG_LIST_FLAGS, 
-						/*Pt_LIST_NON_SELECT |*/ 0, 
+						/*Pt_LIST_NON_SELECT |*/ Pt_FALSE, 
 						/*Pt_LIST_NON_SELECT |*/ Pt_LIST_SCROLLBAR_GETS_FOCUS);
-				PtSetArg(&args[n++], Pt_ARG_TEXT_FLAGS, 0, Pt_EDITABLE);
+				PtSetArg(&args[n++], Pt_ARG_TEXT_FLAGS, Pt_FALSE, Pt_EDITABLE);
+				PtSetArg(&args[n++], Pt_ARG_CBOX_FLAGS,Pt_TRUE,Pt_COMBOBOX_MAX_WIDTH);
 				tb = PtCreateWidget(PtComboBox, tbgroup, n, args);
 
 				// populate it
 				if (pControl && tb) {
 					pControl->populate();
 
-					const UT_Vector * v = pControl->getContents();
+					const UT_GenericVector<const char *> * v = pControl->getContents();
 					UT_ASSERT(v);
 
 					if (v) {
@@ -398,6 +421,7 @@ bool EV_QNXToolbar::synthesize(void)
 					tcb->id = id;	
 					tcb->m_widget = tb;
 					PtAddCallback(tb, Pt_CB_SELECTION, s_combo_select, tcb);
+					PtAddCallback(tb,Pt_CB_CBOX_CLOSE,s_combo_list_close,tcb);
 				}
  			}
 			break;
@@ -470,7 +494,7 @@ bool EV_QNXToolbar::bindListenerToView(AV_View * pView)
 
 bool EV_QNXToolbar::refreshToolbar(AV_View * pView, AV_ChangeMask mask)
 {
-	PtArg_t args[10];
+	PtArg_t args[12];
     struct _cb_data *tcb;       //Toolbar item call back
 	int     n;
 
@@ -654,18 +678,35 @@ XAP_QNXApp * EV_QNXToolbar::getApp(void)
 	return m_pQNXApp;
 }
 
-XAP_QNXFrame * EV_QNXToolbar::getFrame(void)
+XAP_Frame * EV_QNXToolbar::getFrame(void)
 {
-	return m_pQNXFrame;
+	return m_pFrame;
 }
 
 void EV_QNXToolbar::show(void) {
     UT_ASSERT(m_wToolbarGroup && m_wToolbar);
+
+		if(!PtWidgetIsRealized(m_wToolbarGroup))
+			PtRealizeWidget(m_wToolbarGroup);
     PtRealizeWidget(m_wToolbar);
 }
 
 void EV_QNXToolbar::hide(void) {
+		PtWidget_t *current;
     UT_ASSERT(m_wToolbarGroup && m_wToolbar);
     PtUnrealizeWidget(m_wToolbar);
-	PtSetResource(m_wToolbar, Pt_ARG_FLAGS, Pt_DELAY_REALIZE, Pt_DELAY_REALIZE);
+		PtSetResource(m_wToolbar, Pt_ARG_FLAGS, Pt_DELAY_REALIZE, Pt_DELAY_REALIZE);
+			
+
+/*		current = m_wToolbarGroup;
+		while(current = PtWidgetFamily(m_wToolbarGroup,current) )
+			if(PtWidgetIsClass(current,PtToolbar))
+			{
+				PtWidgetSkip(m_wToolbarGroup,current);
+				if(PtWidgetIsRealized(current))	
+					return;
+			}
+		PtUnrealizeWidget(m_wToolbarGroup); */ 
+
 }
+
