@@ -27,6 +27,7 @@
 #include "ut_string.h"
 #include "ut_string_class.h"
 #include "ut_debugmsg.h"
+#include "ut_Language.h"
 #include "xap_Types.h"
 #include "ev_Win32Menu.h"
 #include "xap_Win32App.h"
@@ -85,24 +86,7 @@ static const EV_Menu_Bitmap s_bitmaps[] =
 
 	// Format
 	{AP_MENU_ID_FMT_FONT, "Menu_AbiWord_Font"},
-
-	/*{AP_MENU_ID_FMT_BOLD, "Menu_AbiWord_Bold"},
-	{AP_MENU_ID_FMT_ITALIC, "Menu_AbiWord_Italic"},
-	{AP_MENU_ID_FMT_UNDERLINE, "Menu_AbiWord_Underline"},
-	{AP_MENU_ID_FMT_STRIKE, "Menu_AbiWord_Strike"},		
-	{AP_MENU_ID_FMT_TOPLINE, "Menu_AbiWord_Topline"},
-	{AP_MENU_ID_FMT_BOTTOMLINE, "Menu_AbiWord_Bottomline"},
-	{AP_MENU_ID_SUPERSCRIPT, "Menu_AbiWord_Superscript"},
-	{AP_MENU_ID_SUBSCRIPT, "Menu_AbiWord_Subscript"},
-	{AP_MENU_ID_OVERLINE, "Menu_AbiWord_Overline"},*/
-		
-
-	//{AP_MENU_ID_ALIGN_LEFT, "Menu_AbiWord_Align_Left"},
-	//{AP_MENU_ID_ALIGN_RIGHT, "Menu_AbiWord_Align_Right"},
-	//{AP_MENU_ID_ALIGN_CENTER, "Menu_AbiWord_Align_Center"},
-	//{AP_MENU_ID_ALIGN_JUSTIFY, "Menu_AbiWord_Align_Justify"},	
 	{AP_MENU_ID_TABLE_INSERT_TABLE, "Menu_AbiWord_Insert_Table"},
-	//{AP_MENU_ID_VIEW_SHOWPARA, "Menu_AbiWord_View_Showpara"},
 	{AP_MENU_ID_TOOLS_SPELL, "Menu_AbiWord_Spellcheck"},			
 	{AP_MENU_ID_TOOLS_OPTIONS, "Menu_AbiWord_Preferences"},
 	{AP_MENU_ID_TOOLS_SCRIPTS, "Menu_AbiWord_Execute"},
@@ -126,8 +110,6 @@ static const EV_Menu_Bitmap s_bitmaps[] =
 	
 
 	// Help
-	{AP_MENU_ID_HELP_CONTENTS,      "Menu_AbiWord_Help"},    
-	//{AP_MENU_ID_HELP_INDEX, GTK_STOCK_INDEX},
 	{AP_MENU_ID_HELP_SEARCH, "Menu_AbiWord_Search"},
 	{AP_MENU_ID_HELP_ABOUT, "Menu_AbiWord_About"},
 	{AP_MENU_ID_HELP_CREDITS, "Menu_AbiWord_Credits"},
@@ -158,7 +140,7 @@ static const char * _ev_GetLabelName(XAP_Win32App * pWin32App,
 	if (!szLabelName || !*szLabelName)
 		return NULL;
 
-	UT_String str = AP_Win32App::s_fromUTF8ToAnsi(szLabelName);
+	UT_String str = AP_Win32App::s_fromUTF8ToWinLocale(szLabelName);
 	szLabelName = str.c_str();
 
 	
@@ -223,7 +205,8 @@ EV_Win32Menu::EV_Win32Menu(XAP_Win32App * pWin32App,
 :	EV_Menu(pWin32App, pWin32App->getEditMethodContainer(), szMenuLayoutName, szMenuLabelSetName),
 	m_pWin32App(pWin32App),
 	m_pEEM(pEEM),
-	m_myMenu(NULL)
+	m_myMenu(NULL),
+	m_iDIR(0)
 {
 		    
 	NONCLIENTMETRICSA ncm;
@@ -250,7 +233,12 @@ EV_Win32Menu::EV_Win32Menu(XAP_Win32App * pWin32App,
 
 		DeleteObject(hBitmap);
 	}						
+
+	UT_return_if_fail( m_pMenuLabelSet );
 	
+	UT_Language l;
+	if(l.getDirFromCode(m_pMenuLabelSet->getLanguage()) == UTLANG_RTL)
+		m_iDIR = DT_RTLREADING;
 }
 
 void	EV_Win32Menu::destroy()
@@ -398,7 +386,7 @@ bool EV_Win32Menu::synthesizeMenu(XAP_Frame * pFrame, HMENU menuRoot)
 						m_vecItems.addItem(item);
 							
 						if (!m_bTrack && stack.getDepth()==2)
-							AppendMenu(m, flags,u, szLabelName);																	
+							AppendMenu(m, flags,u, szLabelName);
 						else
 							AppendMenu(m, flags|MF_OWNERDRAW,u, (const char*) item);
 					}		
@@ -411,10 +399,10 @@ bool EV_Win32Menu::synthesizeMenu(XAP_Frame * pFrame, HMENU menuRoot)
 						EV_Menu_Item*	item = new EV_Menu_Item;
 						item->id = id;
 						item->pMenu= this;						
-						strcpy (item->szText, szLabelName);																						
+						strcpy (item->szText, szLabelName);
 						m_vecItems.addItem(item);
-						
-						AppendMenu(m, MF_OWNERDRAW , u, (const char*) item);// TODO: Parameter four, right cast												
+
+						AppendMenu(m, MF_OWNERDRAW , u, (const char*) item);
 					}
 				}
 
@@ -712,15 +700,12 @@ void EV_Win32Menu::onMeasureItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 
 	// Retrieve the width and height of the item's string 
 	GetTextExtentPoint32(hdc, item->szText, lstrlen(item->szText), &size); 
-
-	UT_DEBUGMSG(("EV_Win32Menu::onMeasureItem->%s\n", item->szText));
-	
-	if (lpmis->itemHeight<item->pMenu->m_nBitmapCY)
+		
+	if (size.cy < item->pMenu->m_nBitmapCY)
 		lpmis->itemHeight = item->pMenu->m_nBitmapCY;
 	else
 		lpmis->itemHeight = size.cy;
 
-
 	lpmis->itemWidth =  size.cx + item->pMenu->m_nBitmapCX + SPACE_ICONTEXT;
 
 	SelectObject(hdc, hfontOld); 
@@ -790,9 +775,19 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 
 	/* Rect to draw the text */	
 	rect.top =  lpdis->rcItem.top;
-	rect.right = lpdis->rcItem.right;
 	rect.bottom = lpdis->rcItem.bottom;
-	rect.left = /*m_nBitmapCX*/ 20  + lpdis->rcItem.left;
+
+	if(m_iDIR)
+	{
+		rect.right = lpdis->rcItem.right - 20;
+		rect.left = lpdis->rcItem.left;
+	}
+	else
+	{
+		rect.right = lpdis->rcItem.right;
+		rect.left = /*m_nBitmapCX*/ 20  + lpdis->rcItem.left;
+	}
+	
 	
 	if (lpdis->itemState & ODS_GRAYED) 
 		crText = SetTextColor(lpdis->hDC, GetSysColor(COLOR_GRAYTEXT));	
@@ -821,7 +816,11 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 	/*Draw the background of the item*/
 	FillRect(lpdis->hDC, &lpdis->rcItem, GetSysColorBrush(colorID));
 
-	rect.left = rect.left + SPACE_ICONTEXT;  
+	if(m_iDIR)
+		rect.right = rect.right - SPACE_ICONTEXT;
+	else
+		rect.left = rect.left;
+		
 	
 	// Select the font associated with the item into the 
     // item's device context, and then draw the string.  
@@ -854,11 +853,25 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 		crBkgnd = SetBkColor(lpdis->hDC, GetSysColor(COLOR_HIGHLIGHT));
 
 	/* Draw text*/
-	DrawText(lpdis->hDC, sTextLeft.c_str(),  sTextLeft.length() , &rect, DT_LEFT | DT_VCENTER | DT_SINGLELINE);
-
-	if (sTextRight.length())
-		DrawText(lpdis->hDC, sTextRight.c_str(), sTextRight.length(), &rect, DT_RIGHT | DT_VCENTER | DT_SINGLELINE);
-
+	if(m_iDIR)
+	{
+		DrawText(lpdis->hDC, sTextLeft.c_str(),  sTextLeft.length() , &rect,
+				 DT_RIGHT | DT_VCENTER | DT_SINGLELINE | m_iDIR);
+		
+		if (sTextRight.length())
+			DrawText(lpdis->hDC, sTextRight.c_str(), sTextRight.length(), &rect,
+					 DT_LEFT | DT_VCENTER | DT_SINGLELINE | m_iDIR);
+	}
+	else
+	{
+		DrawText(lpdis->hDC, sTextLeft.c_str(),  sTextLeft.length() , &rect,
+				 DT_LEFT | DT_VCENTER | DT_SINGLELINE | m_iDIR);
+		
+		if (sTextRight.length())
+			DrawText(lpdis->hDC, sTextRight.c_str(), sTextRight.length(), &rect,
+					 DT_RIGHT | DT_VCENTER | DT_SINGLELINE | m_iDIR);
+	}
+	
 	if (lpdis->itemState & ODS_SELECTED) 
 		SetBkColor(lpdis->hDC, crBkgnd); 
 	
@@ -868,8 +881,11 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 					  		
 		HDC hdcMem = CreateCompatibleDC(lpdis->hDC);
 		SelectObject(hdcMem,(void *)hBitmap);				
-		BitBlt(lpdis->hDC, lpdis->rcItem.left+1, lpdis->rcItem.top+1, 16, 16, hdcMem, 0, 0, SRCCOPY );
 
+		if(m_iDIR)
+			BitBlt(lpdis->hDC, lpdis->rcItem.right - 20, lpdis->rcItem.top+1, 16, 16, hdcMem, 0, 0, SRCCOPY );
+		else
+			BitBlt(lpdis->hDC, lpdis->rcItem.left+1, lpdis->rcItem.top+1, 16, 16, hdcMem, 0, 0, SRCCOPY );
 		
 		DeleteDC(hdcMem);					
 				
@@ -887,10 +903,14 @@ void EV_Win32Menu::onDrawItem(HWND hwnd, WPARAM wParam, LPARAM lParam)
 			SelectObject(hdcMem, bm);
 			SetRect(&r, 0, 0, nWidth, nHeight);
 			DrawFrameControl(hdcMem, &r, DFC_MENU,DFCS_MENUCHECK);			
-        
-			BitBlt(lpdis->hDC, lpdis->rcItem.left+2, lpdis->rcItem.top+2, nWidth, nHeight,
-				hdcMem, 0, 0, SRCCOPY);
 
+			if(m_iDIR)
+				BitBlt(lpdis->hDC, lpdis->rcItem.right - 20, lpdis->rcItem.top+2, nWidth, nHeight,
+					   hdcMem, 0, 0, SRCCOPY);
+			else
+				BitBlt(lpdis->hDC, lpdis->rcItem.left+2, lpdis->rcItem.top+2, nWidth, nHeight,
+					   hdcMem, 0, 0, SRCCOPY);
+				
 			DeleteDC(hdcMem);
 			DeleteObject(bm);
 		}	
@@ -942,7 +962,7 @@ bool EV_Win32Menu::onMenuSelect(XAP_Frame * pFrame, AV_View * pView,
 	if (!szMsg || !*szMsg)
 		szMsg = "TODO This menu item doesn't have a StatusMessage defined.";
 	
-	UT_String str = AP_Win32App::s_fromUTF8ToAnsi(szMsg);
+	UT_String str = AP_Win32App::s_fromUTF8ToWinLocale(szMsg);
 	pFrame->setStatusMessage(str.c_str());
 	return true;
 }
@@ -970,7 +990,30 @@ bool EV_Win32MenuBar::synthesizeMenuBar(XAP_Frame * pFrame)
 {
 	m_myMenu = CreateMenu();
 
-	return (synthesizeMenu(pFrame, m_myMenu));
+	bool bRet = synthesizeMenu(pFrame, m_myMenu);
+
+	// when dealing with RTL interface language, we need to tell do some tricks here to
+	// make the menubar to layout RTL
+	// see http://www.microsoft.com/middleeast/msdn/faq.aspx
+	MENUITEMINFO mii;
+	char buff[81];
+	memset(buff,80,' ');
+	buff[80] = 0;
+	
+	mii.cbSize = sizeof(mii);
+	mii.dwTypeData = buff;
+	mii.fType = MF_STRING;
+	mii.cch = 80;
+	mii.fState = MFS_DEFAULT;
+	mii.fMask = MIIM_ID | MIIM_DATA | MIIM_TYPE | MIIM_SUBMENU;
+	
+	if(m_iDIR && GetMenuItemInfo(m_myMenu, 0, 1, &mii))
+	{
+		mii.fType |= MFT_RIGHTORDER;
+		SetMenuItemInfo(m_myMenu, 0, 1, &mii);
+	}
+	
+	return bRet;
 }
 
 /*****************************************************************/
@@ -995,6 +1038,30 @@ bool EV_Win32MenuPopup::synthesizeMenuPopup(XAP_Frame * pFrame)
 {
 	m_myMenu = CreatePopupMenu();
 
-	return synthesizeMenu(pFrame, m_myMenu);
+	bool bRet = synthesizeMenu(pFrame, m_myMenu);
+
+	// when dealing with RTL interface language, we need to tell do some tricks here to
+	// make the menubar to layout RTL
+	// see http://www.microsoft.com/middleeast/msdn/faq.aspx
+	MENUITEMINFO mii;
+	char buff[81];
+	memset(buff,80,' ');
+	buff[80] = 0;
+	
+	mii.cbSize = sizeof(mii);
+	mii.dwTypeData = buff;
+	mii.fType = MF_STRING;
+	mii.cch = 80;
+	mii.fState = MFS_DEFAULT;
+	mii.fMask = MIIM_ID | MIIM_DATA | MIIM_TYPE | MIIM_SUBMENU;
+	
+	if(m_iDIR && GetMenuItemInfo(m_myMenu, 0, 1, &mii))
+	{
+		mii.fType |= MFT_RIGHTORDER;
+		SetMenuItemInfo(m_myMenu, 0, 1, &mii);
+	}
+	
+	return bRet;
+	
 }
 
