@@ -54,6 +54,14 @@ public:
 	virtual UT_sint32 measureUnremappedCharForCache(UT_UCSChar cChar) const;
 	UT_sint32	measureUnRemappedChar(UT_UCSChar c);
 	virtual GR_CharWidths* newFontWidths(void) const;
+//
+// UT_Rect of glyph in Logical units.
+// rec.left = bearing Left (distance from origin to start)
+// rec.width = width of the glyph
+// rec.top = distance from the origin to the top of the glyph
+// rec.height = total height of the glyph
+
+	virtual bool glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG);
 	
 	void        selectFontIntoDC(GR_Graphics * pGr, HDC hdc);
 	
@@ -261,6 +269,10 @@ public:
 
 	virtual void          setPrintDC(HDC dc);
 	HDC                   getPrintDC() const {return m_printHDC;}
+	HDC                   getPrimaryDC() const {return m_hdc;}
+
+	void                  setPrintDCFontAllocNo(UT_uint32 i){m_iPrintDCFontAllocNo = i;}
+	void                  setDCFontAllocNo(UT_uint32 i){m_iDCFontAllocNo = i;}
 	
 	double                getXYRatio() const {return m_fXYRatio;}
 	double                getXYRatioPrint() const {return m_fXYRatioPrint;}
@@ -289,6 +301,8 @@ protected:
 
   protected:
 
+	UT_uint32               m_iDCFontAllocNo;
+	UT_uint32               m_iPrintDCFontAllocNo;
 	HDC						m_hdc;
 	HDC                     m_printHDC;
 	HWND 					m_hwnd;
