@@ -22,7 +22,14 @@
 #define UT_ABIOBJECT_H
 
 #include <stdlib.h>	// size_t
+
+/* pre-emptive dismissal; ut_types.h is needed by just about everything,
+ * so even if it's commented out in-file that's still a lot of work for
+ * the preprocessor to do...
+ */
+#ifndef UT_TYPES_H
 #include "ut_types.h"
+#endif
 
 // uncomment this for memory management (experimental)
 //#define MANAGE_MEMORY 1
