@@ -84,7 +84,7 @@ UT_String::UT_String()
 }
 
 UT_String::UT_String(const char* sz, size_t n)
-:	pimpl(new UT_Stringbuf(sz, n ? n : (sz ? strlen(sz) : 0)))
+:	pimpl(new UT_Stringbuf(sz, n ? n : (sz && *sz ? strlen(sz) : 0)))
 {
 }
 
@@ -151,7 +151,7 @@ UT_String& UT_String::operator=(const UT_String& rhs)
 
 UT_String& UT_String::operator=(const char* rhs)
 {
-  if (!rhs)
+  if (!rhs || !*rhs)
     pimpl->clear ();
   else
     pimpl->assign(rhs, strlen(rhs));
@@ -176,7 +176,7 @@ UT_String& UT_String::operator+=(const UT_String& rhs)
 
 UT_String& UT_String::operator+=(const char* rhs)
 {
-	UT_return_val_if_fail(rhs, *this);
+	UT_return_val_if_fail(rhs && *rhs, *this);
 	pimpl->append(rhs, strlen(rhs));
 	return *this;
 }
@@ -832,7 +832,7 @@ void UT_UTF8String::clear () const
 UT_UTF8String &	UT_UTF8String::operator=(const char * rhs)
 {
   // treat null string assignment as a clear
-  if (!rhs)
+  if (!rhs || !*rhs)
     pimpl->clear();
   else
     pimpl->assign (rhs);
@@ -858,7 +858,7 @@ UT_UTF8String &	UT_UTF8String::operator+=(const UT_UCS4Char            rhs)
 
 UT_UTF8String &	UT_UTF8String::operator+=(const char * rhs)
 {
-	UT_return_val_if_fail(rhs, *this);
+	UT_return_val_if_fail(rhs && *rhs, *this);
 	pimpl->append (rhs);
 	return *this;
 }
@@ -915,7 +915,7 @@ const UT_UTF8String & UT_UTF8String::escape (const UT_UTF8String & str1, const U
 	return *this;
 }
 
-/* escapes '<', '>' & '&' in the current string
+/* escapes '<', '>', '"', & '&' in the current string
  */
 const UT_UTF8String & UT_UTF8String::escapeXML ()
 {
@@ -1331,7 +1331,7 @@ UT_UCS4String::UT_UCS4String(const char * utf8_str, size_t bytelength /* 0 == ze
 :	pimpl(new UT_UCS4Stringbuf)
 {
 	if (bytelength == 0) {
-		if (utf8_str == 0) return;
+		if (utf8_str == 0 || *utf8_str == '\0') return;
 		bytelength = strlen (utf8_str);
 	}
 	while (true) {
@@ -1350,7 +1350,7 @@ UT_UCS4String::UT_UCS4String(const char * utf8_str, size_t bytelength /* 0 == ze
 :	pimpl(new UT_UCS4Stringbuf)
 {
 	if (bytelength == 0) {
-		if (utf8_str == 0) return;
+		if (utf8_str == 0 || *utf8_str == '\0') return;
 		bytelength = strlen (utf8_str);
 	}
 	UT_UCS4Char ucs4a = UT_UCS4Stringbuf::UTF8_to_UCS4 (utf8_str, bytelength);
