@@ -37,26 +37,36 @@
 #pragma warning(disable: 4292)
 #endif
 
-/*!
-    UT_GenericTree Class
-    ====================
+/*! \brief UT_GenericTree allows access to its data in a tree-like fashion: iterating over
+    branches, or siblings, or accessing individual nodes.
+	
+    UT_GenericTree Class \n
+    ==================== \n
 
-    Summary of usage
-    ----------------
+    Summary of usage \n
+    ---------------- \n
     
     UT_GenericTree allows access to its data in a tree-like fashion: iterating over
     branches, or siblings, or accessing individual nodes. The idea is to convert data in
-    format
-
+    format \n
+	 
+	 \verbatim
         <a><b/><c><d/></c></a>
-
+	 \endverbatim
+	 
     to
 
+	 \htmlonly <pre>
+	 \endhtmlonly
+	 \verbatim
 		    a
 		   / \
 		  b   c
 		       \
 		        d
+	 \endverbatim			
+	 \htmlonly </pre>
+	 \endhtmlonly
 
     After creation, the object is intialised from a sequentiall representation of the data
     (such as found in an xml file or AbiWord piecetable). This is done either by repeated
@@ -73,8 +83,8 @@
     no API to insert/delete nodes after the tree's initial construction.
 
     
-    buildTree() method
-    ------------------
+    buildTree() method \n
+    ------------------ \n
     
     The buildTree() method requires a callback function and a void pointer; the void
     pointer is passed back to the callback function (e.g., if the call back function is a
@@ -95,8 +105,8 @@
    	passed into tree.
 
 
-    Node functions
-    --------------
+    Node functions \n
+    -------------- \n
 	
 	UT_sint32 getMaxNodeLevel() const;
 	    This function returns the highest node level in the tree, that is, the distance
@@ -117,8 +127,8 @@
 	    if(iLevel1 == iLevel2 && n1 < n2) => dpos1 < dpos2
 
     
-    Cursor class
-    ---------------
+    Cursor class \n
+    --------------- \n
 	The cursor class makes it possible to iterate over the tree in structured way using
     the relationships between the indivial nodes; they return true on success.
 
@@ -154,7 +164,6 @@
     bool is_valid() -- returns true if the cursor points to a valid node.
 */
 
-typedef	enum {TNR_Parent, TNR_Sibling, TNR_Child, TNR_Uncle, TNR_None} UT_TreeNodeRelationship;
 
 template <class T> class ABI_EXPORT UT_GenericTree
 {
@@ -164,13 +173,15 @@ template <class T> class ABI_EXPORT UT_GenericTree
 	
   public:
   	class Cursor;
-
+	friend class Cursor;
+	
   	UT_GenericTree(): m_data(NULL){};
 	~UT_GenericTree() {clear();}
 
-	bool      buildTree( UT_TreeNodeRelationship (*next)(T& n, void* param), void * callerdata);
-	bool      insert(T value, UT_TreeNodeRelationship r);
+	bool      buildTree( UT_sint32 (*next)(T& n, void* param), void * callerdata);
+	bool      insert(T value, UT_sint32 iRelativeLevel);
 
+	
 	void      clear();           // empty and deallocate internal data structures (but not node data !!!)
 	void      freeData(void);    // deallocate node data using free()
 	void      purgeData(void);   // deallocate node data using delete
@@ -196,6 +207,8 @@ template <class T> class ABI_EXPORT UT_GenericTree
 		void setLevel(UT_uint32 i) {level = i;}
 		UT_uint32 getLevel()const {return level;}
 
+		bool isLeaf() const {return firstChild == NULL;}
+
 	  private:
 		T      me;
 		Node * parent;
@@ -212,7 +225,7 @@ template <class T> class ABI_EXPORT UT_GenericTree
 	{
 	  public:
 		Cursor(const UT_GenericTree<T> * owner)
-			: m_d(owner)
+			: m_d(owner), m_saved_node(NULL)
 		{
 			m_node = m_d->m_data;
 			UT_ASSERT_HARMLESS( m_node );
@@ -227,6 +240,23 @@ template <class T> class ABI_EXPORT UT_GenericTree
 			return is_valid();
 		}
 
+		bool last()
+		{
+			m_node = NULL;
+			UT_sint32 iMaxLevel = m_d->getMaxNodeLevel();
+			if(iMaxLevel < 0)
+				return false;
+
+			UT_sint32 iLastNode = m_d->getNodeCountForLevel(iMaxLevel) - 1;
+
+			if(iLastNode < 0)
+				return false;
+
+			m_node = m_d->_getNthNodeForLevel(iMaxLevel, iLastNode);
+
+			return is_valid();
+		}
+		
 		bool lastValidPos()
 		{
 			m_node = m_last_valid_node;
@@ -371,10 +401,23 @@ template <class T> class ABI_EXPORT UT_GenericTree
 			return is_valid() ? m_node->getContent() : s;
 		}
 
-		inline bool is_valid()
+		inline bool is_valid() const
 		{
 			return (m_node != NULL);
 		}
+
+		bool isLeaf() const {return is_valid() ? m_node->isLeaf() : true;}
+
+		void save()
+		{
+			m_saved_node = m_node;
+		}
+
+		void restore()
+		{
+			UT_return_if_fail( m_saved_node );
+			m_node = m_saved_node;
+		}
 		
 	  private:
 		
@@ -391,6 +434,7 @@ template <class T> class ABI_EXPORT UT_GenericTree
 		const UT_GenericTree<T>*	m_d;
 		const Node*			        m_node;
 		const Node*                 m_last_valid_node;
+		const Node*                 m_saved_node;
 	};
 
 	friend class Cursor;
@@ -400,6 +444,7 @@ private:
 	bool _insertChild(T value);
 	bool _insertSibling(T value);
 	bool _insertUncle(T value);
+	bool _dropLevel();
 
 	bool   _mapNode(Node *);
 	Node * _getNthNodeForLevel(UT_uint32 iLevel, UT_uint32 n) const;
@@ -447,16 +492,16 @@ UT_GenericTree<T>::freeData(void)
 
 
 template <class T> bool
-UT_GenericTree<T>::buildTree( UT_TreeNodeRelationship (*next)(T& t, void* param), void * callerdata)
+UT_GenericTree<T>::buildTree( UT_sint32 (*next)(T& t, void* param), void * callerdata)
 {
 	// first of all reset current state
 	clear();
 	
 	T n;
-	UT_TreeNodeRelationship r = next(n, callerdata);
+	UT_sint32 r = next(n, callerdata);
 	bool b = true;
 	
-	while (r != TNR_None && b)
+	while (r <= 1 && b)
 	{
 		b = insert(n,r);
 		r = next(n, callerdata);
@@ -466,23 +511,36 @@ UT_GenericTree<T>::buildTree( UT_TreeNodeRelationship (*next)(T& t, void* param)
 }
 
 
+/*!
+    Insert value in a relative relationship r into the tree
+    valid values or r are:
+       1  ~ child of the previous node
+       0  ~ sibling of the previous node
+       -n ~ sibling of n-th level ancestor of the previous node
+*/
 template <class T> bool
-UT_GenericTree<T>::insert(T value, UT_TreeNodeRelationship r)
+UT_GenericTree<T>::insert(T value, UT_sint32 r)
 {
-	switch (r)
+	UT_return_val_if_fail( r <= 1, false );
+	
+	if(r == 1)
+		return _insertChild(value);
+	else if(r == 0)
+		return _insertSibling(value);
+	else
 	{
-		case TNR_Child:
-			return _insertChild(value);
-		case TNR_Sibling:
-			return _insertSibling(value);
-		case TNR_Uncle:
-			return _insertUncle(value);
+		bool bOK = true;
+		for(UT_sint32 i = r; bOK && i < 0; i++)
+		{
+			bOK = _dropLevel();
+		}
 
-		default:
-			UT_ASSERT_HARMLESS( UT_NOT_IMPLEMENTED );
-			return false;
+		UT_return_val_if_fail( bOK, false );
+
+		return _insertSibling(value);
 	}
 
+	UT_ASSERT_HARMLESS( UT_NOT_REACHED );
 	return false;
 }
 
@@ -655,16 +713,19 @@ UT_GenericTree<T>::_insertUncle(T value)
 	
 	UT_uint32 iLevel = s ? s->getLevel() : 0;
 
+	bool bOK = true;
+	
 	if(iLevel)
 	{
-		do
+		while(bOK && s && s->getLevel() >= iLevel)
 		{
 			m_nodeStack.pop((void**)&s);
+			bOK = m_nodeStack.viewTop((void**)&s);
 		}
-		while(s && s->getLevel() >= iLevel);
 	}
 	
-
+	UT_return_val_if_fail( bOK, false );
+	
 	if(s)
 	{
 		n->setLevel(s->getLevel());
@@ -685,6 +746,27 @@ UT_GenericTree<T>::_insertUncle(T value)
 	return _mapNode(n);
 }
 
+template <class T> bool
+UT_GenericTree<T>::_dropLevel()
+{
+	Node * s;
+	m_nodeStack.viewTop((void**)&s);
+	
+	UT_uint32 iLevel = s ? s->getLevel() : 0;
+	bool bOK = true;
+
+	if(iLevel)
+	{
+		while(bOK && s && s->getLevel() >= iLevel)
+		{
+			m_nodeStack.pop((void**)&s);
+			bOK = m_nodeStack.viewTop((void**)&s);
+		}
+	}
+
+	return bOK;
+}
+
 template <class T> bool
 UT_GenericTree<T>::_insertSibling(T value)
 {
