@@ -28,6 +28,22 @@
 #include "ut_vector.h"
 #include "gr_Graphics.h"
 
+class XAP_CocoaFont_LayoutHelper
+{
+private:
+	NSDictionary *		m_fontattr;
+	NSTextStorage *		m_storage;
+	NSLayoutManager *	m_layout;
+
+public:
+	XAP_CocoaFont_LayoutHelper(NSFont * font);
+
+	~XAP_CocoaFont_LayoutHelper();
+
+	bool	setUnichar(UT_UCS4Char c, NSGlyph & firstGlyph);
+	bool	setString(NSString * str, NSGlyph & firstGlyph);
+};
+
 /*
   We derive our handle from GR_Font so we can be passed around the GR
   contexts as a native Cocoa font.
@@ -69,9 +85,14 @@ public:
 		rf_Symbols,
 		rf_Dings
 	};
+	bool					needsRemap()
+		{
+			return m_rfRemap;
+		}
 	static RemapFont		remapFont(NSFont * font);
 	static UT_UCS4Char		remapChar(UT_UCS4Char charCode, RemapFont rf);
-	UT_UCS4Char				remapChar(UT_UCS4Char charCode) const { return m_rfRemap ? remapChar(charCode, m_rfRemap) : charCode; }
+	UT_UCS4Char				remapChar(UT_UCS4Char charCode) const 
+		{ return m_rfRemap ? remapChar(charCode, m_rfRemap) : charCode; }
 
 private:
 	RemapFont						m_rfRemap;
@@ -80,6 +101,8 @@ private:
 	mutable NSFont *				m_fontForCache;
 	mutable NSMutableDictionary *	m_fontProps;
 
+	XAP_CocoaFont_LayoutHelper *	m_LayoutHelper;
+
 	void					_resetMetricsCache();
 	static void				_initMetricsLayouts(void);
 
@@ -90,9 +113,9 @@ private:
 
 	/* metrics cache */
 	UT_uint32						_m_size;
-	float							_m_ascent;
-	float							_m_descent;
-	float							_m_height;
+	volatile float							_m_ascent;
+	volatile float							_m_descent;
+	volatile float							_m_height;
 	UT_NumberVector *				_m_coverage;
 
 	/*! static metrics stuff */
