@@ -266,7 +266,7 @@ bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 	
 				// do not want suffix for directory names
 				err = stat(szDialogFilename, &buf);
-				if (S_ISDIR(buf.st_mode))
+				if ((err == 0) && (S_ISDIR(buf.st_mode)))
 					wantSuffix = false;
 	
 				// if the file doesn't have a suffix already, and the file type
