@@ -27,7 +27,7 @@
 #include "ut_debugmsg.h"
 #include "ut_string.h"
 #include "xap_UnixDialogHelper.h"
-
+#include "ut_string_class.h"
 #include "xap_UnixFontManager.h"
 #include "xap_UnixFontXLFD.h"
 #include "xap_EncodingManager.h"
@@ -36,6 +36,7 @@
 #include <sys/stat.h>
 #include "xap_Strings.h"
 #include "xap_Prefs.h"
+#include "gr_Graphics.h"
 
 // initialize our static member pFontManager
 XAP_UnixFontManager * XAP_UnixFontManager::pFontManager = 0;
@@ -50,20 +51,23 @@ XAP_UnixFontManager::XAP_UnixFontManager(void) : m_fontHash(256),
 
 XAP_UnixFontManager::~XAP_UnixFontManager(void)
 {
-	UT_HASH_PURGEDATA(XAP_UnixFont *, &m_fontHash, delete);
-
+	m_fontHash.purgeData();
+	UT_VECTOR_PURGEALL(XAP_UnixFont *,m_vecDeallocatedFonts);
+#if FC_MINOR < 3
 	if (m_pFontSet)
 		FcFontSetDestroy (m_pFontSet);
 
 	if (m_pConfig)
 		FcConfigDestroy (m_pConfig);
+#endif
+
 }
 
 /*!
  * compareFontNames this function is used to compare the char * strings names
  * of the Fonts with the qsort method on UT_Vector.
-\params const void * vF1  - Pointer to XAP_UnixFont pointer
-\params const void * vF2  - Pointer to a XAP_UnixFont pointer
+\param const void * vF1  - Pointer to XAP_UnixFont pointer
+\param const void * vF2  - Pointer to a XAP_UnixFont pointer
 \returns -1 if sz1 < sz2, 0 if sz1 == sz2, +1 if sz1 > sz2
 */
 static UT_sint32 compareFontNames(const void * vF1, const void * vF2)
@@ -73,9 +77,9 @@ static UT_sint32 compareFontNames(const void * vF1, const void * vF2)
 	return UT_strcmp((*pF1)->getName(), (*pF2)->getName());
 }
 
-UT_Vector * XAP_UnixFontManager::getAllFonts(void)
+UT_GenericVector<XAP_UnixFont*>* XAP_UnixFontManager::getAllFonts(void)
 {
-	UT_Vector* pVec = m_fontHash.enumerate();
+	UT_GenericVector<XAP_UnixFont*>* pVec = m_fontHash.enumerate();
 	UT_ASSERT(pVec);
 	if(pVec->getItemCount() > 1)
 		pVec->qsort(compareFontNames);
@@ -111,12 +115,16 @@ static XAP_UnixFont* buildFont(XAP_UnixFontManager* pFM, FcPattern* fp)
 	// TODO: We should follow symlinks.
 	metricFile = reinterpret_cast<char*>(fontFile);
 	size_t ffs = metricFile.size();
-	if (ffs < 4 || (fontFile[ffs - 4] != '.' && fontFile[ffs - 5] != '.'))
+	if ( !((ffs >= 4 && fontFile[ffs - 4] == '.') ||
+		   (ffs >= 5 && fontFile[ffs - 5] == '.') ))	// Separate check to avoid [-1]
 		return NULL;
 
 	// handle '.font'
 	if (fontFile[ffs - 5] == '.')
+	{
 		metricFile = UT_UTF8String(metricFile.ucs4_str().substr(0, metricFile.size() - 1));
+		--ffs; // Decrement, as we've reduced the size of the string by one.  I'm not sure why this is prefix.
+	}
 
 	metricFile.ucs4_str()[ffs - 3] = 'a';
 	metricFile.ucs4_str()[ffs - 2] = 'f';
@@ -127,7 +135,8 @@ static XAP_UnixFont* buildFont(XAP_UnixFontManager* pFM, FcPattern* fp)
 
 	// create the string representing our Pattern
 	char* xlfd = reinterpret_cast<char*>(FcNameUnparse(fp));
-
+//	UT_String sXLFD = xlfd;
+//	UT_ASSERT(sXLFD.size() < 100);
 	// get the family of the font
 	unsigned char *family;
 	FcPatternGetString(fp, FC_FAMILY, 0, &family);
@@ -189,7 +198,7 @@ bool XAP_UnixFontManager::scavengeFonts()
 			if (pFont)
 			{
 				FcFontSetAdd(m_pFontSet, fs->fonts[j]);
-				_addFont(pFont);
+				_addFont(pFont,NULL);
 			}
 		}
 
@@ -214,6 +223,14 @@ XAP_UnixFont* XAP_UnixFontManager::searchFont(const char* pszXftName)
 		sXftName = sLeft;
 		UT_DEBUGMSG(("searchFont:: Symbol replaced with %s \n",sXftName.utf8_str()));
 	}
+	else if(strstr(pszXftName,"symbol") != NULL)
+	{
+		const char * szLocm = strstr(pszXftName,"-");
+		UT_UTF8String sLeft("Standard Symbols L");
+		sLeft += szLocm;
+		sXftName = sLeft;
+		UT_DEBUGMSG(("searchFont:: symbol replaced with %s \n",sXftName.utf8_str()));
+	}
 
 	fp = FcNameParse(reinterpret_cast<const FcChar8*>(sXftName.utf8_str()));
 	FcConfigSubstitute(m_pConfig, fp, FcMatchPattern);
@@ -349,7 +366,8 @@ XAP_UnixFontManager::forceFontSynth(XAP_UnixFontManager* pFontManager,
 									const char* /* pszFontVariant */,
 									const char* pszFontWeight,
 									const char* /* pszFontStretch */,
-									const char* pszFontSize)
+									const char* pszFontSize,
+									GR_Graphics * pG)
 {
   UT_UTF8String pszFontSlant(!UT_stricmp("italic", pszFontStyle)? "100":
 							 !UT_stricmp("oblique", pszFontStyle)? "110":
@@ -383,7 +401,15 @@ XAP_UnixFontManager::forceFontSynth(XAP_UnixFontManager* pFontManager,
 	  UT_UTF8String pMetricFile (pNewFont->getMetricfile());
 	  
 	  pNewFont->setStyle(xap_pszFontStyle);
-	  UT_UTF8String newFontXLFD(pszFontFamily);
+	  UT_UTF8String newFontXLFD;
+	  if(pNewFont->isSymbol())
+	  {
+		  newFontXLFD = "Standard Symbols L";
+	  }
+	  else
+	  {
+		  newFontXLFD = pszFontFamily;
+	  }
 	  newFontXLFD += ":slant=";
 	  newFontXLFD += pszFontSlant;
 	  newFontXLFD += ":weight=";
@@ -397,7 +423,7 @@ XAP_UnixFontManager::forceFontSynth(XAP_UnixFontManager* pFontManager,
 	  
   }	
 
-  _addFont(pNewFont);
+  _addFont(pNewFont,pG);
   
   return pNewFont;
 }
@@ -413,7 +439,8 @@ XAP_UnixFont* XAP_UnixFontManager::findNearestFont(const char* pszFontFamily,
 												   const char* /* pszFontVariant */,
 												   const char* pszFontWeight,
 												   const char* /* pszFontStretch */,
-												   const char* pszFontSize)
+												   const char* pszFontSize,
+												   GR_Graphics * pG)
 {
 	UT_UTF8String st(pszFontFamily);
 
@@ -467,7 +494,7 @@ XAP_UnixFont* XAP_UnixFontManager::findNearestFont(const char* pszFontFamily,
 	if(!pFont){
 	  UT_DEBUGMSG(("The font does not exist!\n"));
 	  pFont = forceFontSynth(pFontManager, pszFontFamily, pszFontStyle, NULL, pszFontWeight,
-							 NULL, pszFontSize);
+							 NULL, pszFontSize,pG);
 	}
 	
 	// If the font still doesn't exist, search for possible alternatives
@@ -478,25 +505,28 @@ XAP_UnixFont* XAP_UnixFontManager::findNearestFont(const char* pszFontFamily,
 		&& !UT_stricmp(pszFontWeight, "bold")){
 	      pFont = findNearestFont(pszFontFamily, "normal",
 				      NULL, pszFontWeight,
-				      NULL, pszFontSize);
+				      NULL, pszFontSize,pG);
 	      if (pFont && (pFont->getStyle() == XAP_UnixFont::STYLE_NORMAL)){
 		pFont = findNearestFont(pszFontFamily, pszFontStyle,
 					NULL, "normal",
-					NULL, pszFontSize);
+					NULL, pszFontSize,pG);
 	      }
 	    } else if (!UT_stricmp(pszFontStyle, "italic") || !UT_stricmp(pszFontStyle, "oblique")){
 	      pFont = findNearestFont(pszFontFamily, "normal",
 				      NULL, pszFontWeight,
-				      NULL, pszFontSize);
+				      NULL, pszFontSize,pG);
 	    } else if (!UT_stricmp(pszFontWeight, "bold")){
 	      pFont = findNearestFont(pszFontFamily, pszFontStyle,
 				      NULL, "normal",
-				      NULL, pszFontSize);
+				      NULL, pszFontSize,pG);
 	    } else {
 	      UT_DEBUGMSG(("No Such Font!\n"));
-	      pFont = findNearestFont("Times", "normal",
-				      NULL, "normal",
-				      NULL, pszFontSize);    
+	      pFont = getFont("Times", XAP_UnixFont::STYLE_NORMAL);
+		  if (!pFont) {
+			UT_GenericVector<XAP_UnixFont*> *pFontList = m_fontHash.enumerate();
+			pFont = pFontList->getNthItem(0);
+			delete pFontList;
+		  }
 	    }
 	  }
   
@@ -512,22 +542,38 @@ XAP_UnixFont * XAP_UnixFontManager::getFont(const char * fontname,
 	g_snprintf(keyBuffer, 512, "%s@%d", fontname, s);
 	UT_upperString(keyBuffer);
 	
-	const XAP_UnixFont* entry = static_cast<const XAP_UnixFont *>(m_fontHash.pick(keyBuffer));
+	const XAP_UnixFont* entry = m_fontHash.pick(keyBuffer);
 
 	//UT_DEBUGMSG(("Found font [%p] in table.\n", entry));
 	
 	return const_cast<XAP_UnixFont *>(entry);
 }
 
-void XAP_UnixFontManager::_addFont(XAP_UnixFont * newfont)
+bool XAP_UnixFontManager::isDeallocated(XAP_UnixFont * pF)
+{
+	return (m_vecDeallocatedFonts.findItem(pF) >= 0);
+}
+
+void XAP_UnixFontManager::_addFont(XAP_UnixFont * newfont,GR_Graphics * pG)
 {
 	// we index fonts by a combined "name" and "style"
 	const char* fontkey = newfont->getFontKey();
-	const void * curfont_entry = m_fontHash.pick(fontkey);
+	const XAP_UnixFont * curfont_entry = m_fontHash.pick(fontkey);
 	if (curfont_entry)
 	{
-		const XAP_UnixFont* curfont = static_cast<const XAP_UnixFont*>(curfont_entry);
-		delete curfont;     
+		const XAP_UnixFont* curfont = curfont_entry;
+//
+// Delay the deletion of this font until this class is deleted. There could
+// be a pointer to this font in fp_Run as well as being in the GR_Graphics
+// font cache.
+//
+		UT_DEBUGMSG(("Deallocate %x \n",curfont));
+		m_vecDeallocatedFonts.addItem(curfont);
+		if(pG)
+		{
+			UT_DEBUGMSG(("Invalidate graphics cache %x \n",pG));
+			pG->invalidateCache();
+		}
 		m_fontHash.remove (fontkey, NULL);
 	} 
 
@@ -538,6 +584,7 @@ void XAP_UnixFontManager::_addFont(XAP_UnixFont * newfont)
 	   so we replace original font (that is standard) 
 	   unconditionally.
 	*/
+	xxx_UT_DEBUGMSG(("Made newfont %x \n",newfont));
 
-	m_fontHash.insert(fontkey, static_cast<void *>(newfont));
+	m_fontHash.insert(fontkey, newfont);
 }
