@@ -49,11 +49,11 @@ class AD_VersionData
   public:
 
 	// constructor for importers
-	AD_VersionData(UT_uint32 v, UT_UTF8String &uuid, time_t start, bool autorev);
-	AD_VersionData(UT_uint32 v, const char * uuid, time_t start, bool autorev);
+	AD_VersionData(UT_uint32 v, UT_UTF8String &uuid, time_t start, bool autorev, UT_uint32 xid);
+	AD_VersionData(UT_uint32 v, const char * uuid, time_t start, bool autorev, UT_uint32 xid);
 	
 	// constructor for new entries
-	AD_VersionData(UT_uint32 v, time_t start, bool autorev);
+	AD_VersionData(UT_uint32 v, time_t start, bool autorev, UT_uint32 xid);
 
 	// copy constructor
 	AD_VersionData(const AD_VersionData & v);
@@ -74,12 +74,15 @@ class AD_VersionData
 
 	bool           isAutoRevisioned()const {return m_bAutoRevision;}
 	void           setAutoRevisioned(bool autorev);
+
+	UT_uint32      getTopXID() const {return m_iTopXID;}
 	
   private:
 	UT_uint32   m_iId;
 	UT_UUID *   m_pUUID;
 	time_t      m_tStart;
 	bool        m_bAutoRevision;
+	UT_uint32   m_iTopXID;
 };
 
 enum AD_HISTORY_STATE
@@ -151,7 +154,17 @@ public:
 	UT_Error		        saveAs(const char * szFilename, int ieft, const char * props = NULL);
 	UT_Error		        saveAs(const char * szFilename, int ieft, bool cpy, const char * props = NULL);
 	UT_Error		        save(void);
-		
+	virtual bool			createDataItem(const char * szName,
+										   bool bBase64, 
+										   const UT_ByteBuf * pByteBuf,
+										   const void* pToken, 
+										   void ** ppHandle) = 0;
+	virtual bool            replaceDataItem(const char * szName, 
+											const UT_ByteBuf * pByteBuf) = 0;
+	virtual bool			getDataItemDataByName(const char * szName,
+												  const UT_ByteBuf ** ppByteBuf,
+												  const void** ppToken, 
+												  void ** ppHandle) const = 0;		
 public:
 	
 	/**
@@ -190,6 +203,7 @@ public:
 	UT_uint32       getHistoryNthEditTime(UT_uint32 i)const;
 	const UT_UUID&  getHistoryNthUID(UT_uint32 i)const;
 	bool            getHistoryNthAutoRevisioned(UT_uint32 i)const;
+	UT_uint32       getHistoryNthTopXID(UT_uint32 i)const;
 
 	AD_HISTORY_STATE       verifyHistoryState(UT_uint32 &iVersion) const;
 	const AD_VersionData * findHistoryRecord(UT_uint32 iVersion) const;
@@ -241,7 +255,7 @@ public:
 	bool                isAutoRevisioning()const {return m_bAutoRevisioning;}
 	virtual void        setAutoRevisioning(bool autorev);
 	
-	virtual void        purgeRevisionTable() = 0;
+	virtual void        purgeRevisionTable(bool bUnconditional = false) = 0;
 
 	virtual bool        acceptRejectRevision(bool bReject,
 											 UT_uint32 iStart,
@@ -251,10 +265,13 @@ public:
 	virtual bool        rejectAllHigherRevisions(UT_uint32 iLevel) = 0;
 
 	virtual bool        acceptAllRevisions() = 0;
-	
-	
-protected:
 
+	bool                purgeAllRevisions(AV_View * pView);
+	
+	
+	virtual UT_uint32   getXID() = 0;
+	virtual UT_uint32   getTopXID() const = 0;
+	
  protected:	
 	virtual UT_Error	_saveAs(const char * szFilename, int ieft, const char * props = NULL) = 0;
 	virtual UT_Error	_saveAs(const char * szFilename, int ieft, bool cpy, const char * props = NULL) = 0;
