@@ -22,16 +22,20 @@
 #include "xap_Dlg_Password.h"
 
 XAP_Dialog_Password::XAP_Dialog_Password(XAP_DialogFactory * pDlgFactory, XAP_Dialog_Id id)
-  : XAP_Dialog_NonPersistent (pDlgFactory, id), m_passwd (NULL)
+  : XAP_Dialog_NonPersistent (pDlgFactory, id), m_answer(a_Cancel)
 {
 }
 
 XAP_Dialog_Password::~XAP_Dialog_Password ()
 {
-  FREEP(m_passwd);
 }
 
 void XAP_Dialog_Password::setPassword (const char * pass)
 {
-  m_passwd = UT_strdup (pass);
+	m_passwd = pass;
+}
+
+void XAP_Dialog_Password::setPassword (const UT_UTF8String& pass)
+{
+	m_passwd = pass;
 }
