@@ -33,12 +33,11 @@
 /******************************************************************/
 
 XAP_Dialog_Print::XAP_Dialog_Print(XAP_DialogFactory * pDlgFactory, XAP_Dialog_Id id)
-	: XAP_Dialog_AppPersistent(pDlgFactory,id)
+	: XAP_Dialog_AppPersistent(pDlgFactory,id, "interface/dialogprint")
 {
 	m_bPersistValid = false;
 	m_persistNrCopies = 1;
-	m_persistCollate = false;
-	m_persistReverse = false;
+	m_persistCollate = true;
 	m_persistColorSpace = GR_Graphics::GR_COLORSPACE_COLOR;	/* full color is default */
 	m_persistPrintToFile = false;
 
@@ -203,14 +202,7 @@ bool XAP_Dialog_Print::getCollate(void) const
 {
 	UT_ASSERT(m_answer == a_OK);
 
-	return m_bCollate;
-}
-
-bool XAP_Dialog_Print::getReverse(void) const
-{
-	UT_ASSERT(m_answer == a_OK);
-
-	return m_bReverse;
+	return m_persistCollate;
 }
 
 GR_Graphics::ColorSpace XAP_Dialog_Print::getColorSpace(void) const
@@ -238,22 +230,25 @@ bool XAP_Dialog_Print::_getPrintToFilePathname(XAP_Frame * pFrame,
 	pDialog->setCurrentPathname(szSuggestedName);
 	pDialog->setSuggestFilename(true);
 
+	const char ** szDescList = NULL;
+	const char ** szSuffixList = NULL;
+	UT_sint32 * nTypeList = NULL;
 	{
 		// TODO : FIX THIS!  Make this pull dynamic types from the export
 		// TODO : filter list (creat that while you're at it).
 
 		// TODO : Right now we can just feed the dialog some static filters
-		// TODO : that will be ignored by Windows and BeOS but will be required
+		// TODO : that will be ignored by Windows but will be required
 		// TODO : by Unix.
 
 		UT_uint32 filterCount = 1;
 
-		const char ** szDescList = (const char **) UT_calloc(filterCount + 1,
+		szDescList = (const char **) UT_calloc(filterCount + 1,
 														  sizeof(char *));
-		const char ** szSuffixList = (const char **) UT_calloc(filterCount + 1,
+		szSuffixList = (const char **) UT_calloc(filterCount + 1,
 															sizeof(char *));
 		// HACK : this should be IEFileType
-		UT_sint32 * nTypeList = (UT_sint32 *) UT_calloc(filterCount + 1,
+		nTypeList = (UT_sint32 *) UT_calloc(filterCount + 1,
 													 sizeof(UT_sint32));
 
 		szDescList[0] = "PostScript 2.0";
@@ -269,9 +264,13 @@ bool XAP_Dialog_Print::_getPrintToFilePathname(XAP_Frame * pFrame,
 	bool bOK = (ans == XAP_Dialog_FileOpenSaveAs::a_OK);
 
 	if (bOK)
-		UT_cloneString(m_szPrintToFilePathname,pDialog->getPathname());
-	
-	pDialogFactory->releaseDialog(pDialog);
+		UT_cloneString(m_szPrintToFilePathname,pDialog->getPathname());       
 
+	FREEP(szDescList);
+	FREEP(szSuffixList);
+	FREEP(nTypeList);
+
+	pDialogFactory->releaseDialog(pDialog);
+	
 	return bOK;
 }
