@@ -111,6 +111,7 @@ XAP_Frame::XAP_Frame(XAP_Frame * f)
 	m_zoomType(f->m_zoomType),
 	m_pData(0),
 	m_iIdAutoSaveTimer(0),
+	m_iAutoSavePeriod(f->m_iAutoSavePeriod),
 	m_bBackupRunning(false),
 	m_isrcId(0),
 	m_isrcTBNr(0),
@@ -120,6 +121,7 @@ XAP_Frame::XAP_Frame(XAP_Frame * f)
 	m_bHasDropped(false),
 	m_bHasDroppedTB(false),
 	m_bFirstDraw(false),
+	m_bShowStatusbar(f->m_bShowStatusbar),
 	m_bIsFrameLocked(false),
 	m_pFrameImpl(f->m_pFrameImpl->createInstance(this, f->m_pApp)),
 	m_iZoomPercentage(f->m_iZoomPercentage)
@@ -758,7 +760,7 @@ XAP_Dialog_MessageBox * XAP_Frame::createMessageBox(XAP_String_Id id,
 		vsprintf(szNewMessage, static_cast<const char*>(s.c_str()), args);
 		va_end(args);
 
-		pDialog->setMessage(szNewMessage);
+		pDialog->setMessage("%s", szNewMessage);
 		
 		// XAP_MessageBox makes a copy of the message, so free it
 		FREEP(szNewMessage);
@@ -864,10 +866,11 @@ UT_Error XAP_Frame::backup(const char* szExt, UT_sint32 iEFT)
 // Don't put this auto-save in the most recent list.
 //
 	getApp()->getPrefs()->setIgnoreNextRecent();
-
+	
 	if(iEFT < 0)
 	{
-		error = m_pDoc->saveAs(backupName.c_str(), m_pDoc->getLastSavedAsType(), false);
+	        iEFT = 1; // *.abw format
+		error = m_pDoc->saveAs(backupName.c_str(), iEFT, false);
 	}
 	else
 	{
