@@ -48,6 +48,8 @@
 #include "ut_units.h"
 #include "fl_FrameLayout.h"
 #include "fp_FrameContainer.h"
+#include "fl_AutoNum.h"
+
 
 fl_ContainerLayout::fl_ContainerLayout(fl_ContainerLayout* pMyLayout, PL_StruxDocHandle sdh, PT_AttrPropIndex indexAP, PTStruxType iStrux, fl_ContainerType iType)
 	: fl_Layout(iStrux, sdh),
@@ -95,7 +97,7 @@ void fl_ContainerLayout::lookupProperties(void)
 	
 	//  Find the folded Level of the strux
 	lookupFoldedLevel();
-	if(isHidden() == FP_VISIBLE && getFoldedLevel()>0)
+	if((isHidden() == FP_VISIBLE) && (getFoldedLevel() > 0) && (getLevelInList() > getFoldedLevel()) )
 	{
 		xxx_UT_DEBUGMSG(("Table set to hidden folded \n"));
 		setVisibility(FP_HIDDEN_FOLDED);
@@ -213,7 +215,86 @@ const char*	fl_ContainerLayout::getAttribute(const char * pszName) const
 	return pszAtt;
 }
 
+/*!
+ * Return the nested List level of this structure.
+ */
+UT_sint32 fl_ContainerLayout::getLevelInList(void)
+{
+      fl_BlockLayout * pBList = NULL;
+      if(getContainerType() == FL_CONTAINER_BLOCK)
+      {
+	   pBList = static_cast<fl_BlockLayout * >(this);
+      }
+      else
+      {
+	   pBList = getPrevBlockInDocument();
+      }
+      UT_sint32 iLevel = 0;
+      bool bLoop = true;
+      while(pBList && bLoop)
+      {
+	  while(pBList && !pBList->isListItem())
+	  {
+	       pBList = pBList->getPrevBlockInDocument();
+	  }
+	  if(pBList == NULL)
+	  {
+	       bLoop = false;
+	       break;
+	  }
+	  const PP_AttrProp * pAP = NULL;
+	  pBList->getAP(pAP);
+	  const XML_Char * szLid=NULL;
+	  UT_uint32 id=0;
 
+	  if (!pAP || !pAP->getAttribute(PT_LISTID_ATTRIBUTE_NAME, szLid))
+	       szLid = NULL;
+	  if (szLid)
+	  {
+	       id = atoi(szLid);
+		
+	  }
+	  else
+	  {
+		id = 0;
+	  }
+	  if(id == 0)
+	  {
+	        bLoop = false;
+	        break;
+	  }
+	  PD_Document * pDoc = getDocLayout()->getDocument();
+	  fl_AutoNum * pAuto = pDoc->getListByID( id);
+	  if(pAuto->getLastItem() == pBList->getStruxDocHandle())
+	  {
+	        if(pAuto->getLastItem() == getStruxDocHandle())
+		{
+		     iLevel = pAuto->getLevel();
+		     bLoop = false;
+		     break;
+		}
+	        iLevel = pAuto->getLevel() -1;
+		if(iLevel < 0)
+		{
+		      iLevel = 0;
+		}
+	  }
+	  else
+	  {
+	        if(pBList == this)
+	        { 
+		      iLevel = pAuto->getLevel();
+		}
+		else
+		{
+		      iLevel = pAuto->getLevel() + 1;
+		}
+	  }
+	  bLoop = false;
+	  break;
+      }
+      return iLevel;
+}
 /*!
  * This method returns the folded level of the text.
  */
@@ -249,7 +330,7 @@ void fl_ContainerLayout::lookupFoldedLevel(void)
 	{
 		m_iFoldedLevel = atoi(pszTEXTFOLDED);
 	}
-
+	xxx_UT_DEBUGMSG(("FOlded Level is %d \n",m_iFoldedLevel));
     pszTEXTFOLDED = NULL;
 	if(!pSectionAP || !pSectionAP->getProperty("text-folded-id",pszTEXTFOLDED))
 	{
@@ -420,6 +501,10 @@ void fl_ContainerLayout::add(fl_ContainerLayout* pL)
 fl_BlockLayout* fl_ContainerLayout::getNextBlockInDocument(void) const
 {
 	fl_ContainerLayout * pNext = getNext();
+	if(getContainerType() != FL_CONTAINER_BLOCK)
+	{
+	  pNext = getFirstLayout();
+	}
 	fl_ContainerLayout * pOld = NULL;
 	UT_uint32 depth = 0;
 	next_is_null :
@@ -661,8 +746,24 @@ fl_ContainerLayout * fl_ContainerLayout::insert(PL_StruxDocHandle sdh, fl_Contai
 		break;
 	case FL_CONTAINER_TABLE:
 		pL = static_cast<fl_ContainerLayout *>(new fl_TableLayout(getDocLayout(),sdh, indexAP, this));
-		if (pPrev)
+		if(pPrev && (pPrev == this))
+		{
+		  fl_ContainerLayout * pOldFirst = pPrev->getFirstLayout();
+		  pPrev->setFirstLayout(pL);
+		  pL->setNext(pOldFirst);
+		  if(pOldFirst)
+		  {
+		    pOldFirst->setPrev(pL);
+		  }
+		  if(pPrev->getLastLayout() == NULL)
+		  {
+		    pPrev->setLastLayout(pL);
+		  }
+		}
+		else if (pPrev)
+		{
 			pPrev->_insertIntoList(pL);
+		}
 //
 // Now put the Physical Container into the vertical container that contains it.
 //
@@ -851,7 +952,11 @@ void fl_ContainerLayout::remove(fl_ContainerLayout * pL)
 			m_pFirstL = NULL;
 		}
 	}
-
+	if(getContainerType() != FL_CONTAINER_BLOCK)
+	{
+	  fl_SectionLayout * pSL = static_cast<fl_SectionLayout *>(this);
+	  pSL->removeFromUpdate(pL);
+	}
 	pL->setNext(NULL);
 	pL->setPrev(NULL);
 	pL->setContainingLayout(NULL);
@@ -912,11 +1017,13 @@ fp_Run * fl_ContainerLayout::getFirstRun(void) const
 UT_uint32 fl_ContainerLayout::getPosition(bool bActualBlockPos) const
 {
 	const fl_ContainerLayout * pL = this;
-    if(!bActualBlockPos && (getContainerType() != FL_CONTAINER_TOC))
+	if(!bActualBlockPos && (getContainerType() != FL_CONTAINER_TOC))
 	{
-		while(pL->getContainerType() != FL_CONTAINER_BLOCK && pL->getFirstLayout())
+		pL = static_cast<fl_ContainerLayout *>(getNextBlockInDocument());
+		if(pL == NULL)
 		{
-			pL = pL->getFirstLayout();
+		  PT_DocPosition pos = getDocLayout()->getDocument()->getStruxPosition(getStruxDocHandle());
+		  return pos;
 		}
 		if(pL->getContainerType() == FL_CONTAINER_BLOCK)
 		{
@@ -953,8 +1060,17 @@ bool fl_ContainerLayout::canContainPoint() const
 
 	if(bHidden)
 		return false;
-	else
-		return _canContainPoint();
+	
+	if(!_canContainPoint())
+		return false;
+
+	// see if we are not inside a containing layout that cannot contain point
+	fl_ContainerLayout * pMyLayout = myContainingLayout();
+
+	if(!pMyLayout || pMyLayout->getContainerType() == FL_CONTAINER_DOCSECTION)
+		return true;
+	
+	return pMyLayout->canContainPoint();
 }
 
 bool fl_ContainerLayout::isOnScreen() const
