@@ -161,6 +161,7 @@ public:
 	void		setPendingWordForSpell(fl_BlockLayout *pBlock, fl_PartOfBlock* pWord);
 	bool		checkPendingWordForSpell(void);
 	void        dequeueAll(void);
+	void        queueAll(UT_uint32 iReason);
 	void 		queueBlockForBackgroundCheck(UT_uint32 reason, fl_BlockLayout *pBlock, bool bHead=false);
 	bool 		dequeueBlockForBackgroundCheck(fl_BlockLayout *pBlock);
 
@@ -353,6 +354,9 @@ private:
 	UT_GenericVector<GR_EmbedManager *> m_vecEmbedManager;
 	bool                m_bAutoGrammarCheck;
 	fl_BlockLayout  *   m_PendingBlockForGrammar;
+	UT_sint32           m_iGrammarCount;
+	bool                m_bFinishedInitialCheck;
+	PT_DocPosition      m_iPrevPos;
 };
 
 #endif /* DOCLAYOUT_H */
