@@ -103,6 +103,7 @@ GR_Graphics * FV_FrameEdit::getGraphics(void) const
 
 void FV_FrameEdit::setMode(FV_FrameEditMode iEditMode)
 {
+        UT_DEBUGMSG(("Frame Edit mode set to %d \n",iEditMode));
 	if(iEditMode == FV_FrameEdit_NOT_ACTIVE)
 	{
 		m_pFrameLayout = NULL;
@@ -890,9 +891,11 @@ void FV_FrameEdit::setDragType(UT_sint32 x, UT_sint32 y, bool bDrawFrame)
 void FV_FrameEdit::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 {
 	m_bFirstDragDone = false;
+	UT_DEBUGMSG(("Mouse Left Press \n"));
 	if(!isActive())
 	{
 		setDragType(x,y,true);
+		UT_DEBUGMSG(("Was not active now %d FrameLayout %x \n",getFrameEditMode(),getFrameLayout()));
 		return;
 	}
 //
@@ -901,11 +904,23 @@ void FV_FrameEdit::mouseLeftPress(UT_sint32 x, UT_sint32 y)
 	if(FV_FrameEdit_EXISTING_SELECTED == m_iFrameEditMode )
 	{
 		setDragType(x,y,true);
+		UT_DEBUGMSG(("Was Existing Selected now %d \n",getFrameEditMode()));
 		if(FV_FrameEdit_DragNothing == m_iDraggingWhat)
 		{
 			m_bFirstDragDone = false;
 			m_iFrameEditMode = FV_FrameEdit_NOT_ACTIVE;
 			drawFrame(false);
+			if(m_pFrameContainer && m_pFrameLayout)
+			{
+			  if(m_pFrameLayout->getFrameType() > FL_FRAME_TEXTBOX_TYPE)
+			  {
+			    if(m_pFrameContainer->isTightWrapped())
+			    {
+			      //			      m_pFrameContainer->clearScreen();
+			      m_pView->updateScreen(false);
+			    }
+			  }
+			} 
 			m_pFrameLayout = NULL;
 			m_pFrameContainer = NULL;
 			DELETEP(m_pFrameImage);
@@ -1003,14 +1018,16 @@ bool FV_FrameEdit::isImageWrapper(void) const
 	return false;
 }
 
-bool FV_FrameEdit::getFrameStrings(UT_sint32 x, UT_sint32 y, 
-										UT_String & sXpos,
-										UT_String & sYpos,
-										UT_String & sWidth,
-										UT_String & sHeight,
-										UT_String & sColXpos,
-										UT_String & sColYpos,
-								   fl_BlockLayout ** pCloseBL)
+bool FV_FrameEdit::getFrameStrings(UT_sint32 x, UT_sint32 y,
+				   UT_String & sXpos,
+				   UT_String & sYpos,
+				   UT_String & sWidth,
+				   UT_String & sHeight,
+				   UT_String & sColXpos,
+				   UT_String & sColYpos,
+				   UT_String & sPageXpos,
+				   UT_String & sPageYpos,
+				   fl_BlockLayout ** pCloseBL)
 {
 //
 // Find the block that contains (x,y). We'll insert the frame after
@@ -1034,6 +1051,17 @@ bool FV_FrameEdit::getFrameStrings(UT_sint32 x, UT_sint32 y,
 		{
 			return false;
 		}
+		fl_BlockLayout * pPrevBL = pBL;
+		while(pBL && (pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_ENDNOTE) || (pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_FOOTNOTE) || (pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_TOC)|| (pBL->myContainingLayout()->getContainerType() == FL_CONTAINER_FRAME) )
+		{
+		  UT_DEBUGMSG(("Skipping Block %x \n",pBL));
+		     pPrevBL = pBL;
+		     pBL = pBL->getPrevBlockInDocument();
+		}
+		if(pBL == NULL)
+		{
+		     pBL = pPrevBL;
+		}
 		pLine = pRun->getLine();
 		if(pLine == NULL)
 		{
@@ -1098,6 +1126,13 @@ bool FV_FrameEdit::getFrameStrings(UT_sint32 x, UT_sint32 y,
 		double yPos = static_cast<double>(finalColy)/static_cast<double>(UT_LAYOUT_RESOLUTION);
 		sColXpos = UT_formatDimensionedValue(xPos,"in", NULL);
 		sColYpos = UT_formatDimensionedValue(yPos,"in", NULL);
+		//
+		// OK calculate relative to page now
+		//
+		xPos += static_cast<double>(pCol->getX())/static_cast<double>(UT_LAYOUT_RESOLUTION);
+		yPos += static_cast<double>(pCol->getY())/static_cast<double>(UT_LAYOUT_RESOLUTION);
+		sPageXpos = UT_formatDimensionedValue(xPos,"in", NULL);
+		sPageYpos = UT_formatDimensionedValue(yPos,"in", NULL);
 
 //
 // Find the screen coords of pLine, then work out the offset to the (x,y)
@@ -1151,10 +1186,10 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 //
 // If we've just selected the frame, ignore this event.
 //
-	UT_DEBUGMSG(("Doing mouse release now! \n"));
+	UT_DEBUGMSG(("Doing mouse release now! Mode %d \n", getFrameEditMode()));
 	if(FV_FrameEdit_EXISTING_SELECTED == m_iFrameEditMode)
 	{
-		UT_DEBUGMSG(("Existing Frame selected now released button \n"));
+		UT_DEBUGMSG(("Existing Frame selected now released button isActive() %d \n",isActive()));
 		return;
 	}
 	if(m_pAutoScrollTimer != NULL)
@@ -1179,22 +1214,36 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		UT_String sYpos("");
 		UT_String sColXpos("");
 		UT_String sColYpos("");
+		UT_String sPageXpos("");
+		UT_String sPageYpos("");
 		UT_String sWidth("");
 		UT_String sHeight("");
 		fl_BlockLayout * pCloseBL = NULL;
-		getFrameStrings(m_recCurFrame.left,m_recCurFrame.top,sXpos,sYpos,sWidth,sHeight,sColXpos,sColYpos,&pCloseBL);
+		getFrameStrings(m_recCurFrame.left,m_recCurFrame.top,sXpos,sYpos,sWidth,sHeight,sColXpos,sColYpos,sPageXpos,sPageYpos,&pCloseBL);
 		pf_Frag_Strux * pfFrame = NULL;
-		const XML_Char * props[22] = {"frame-type","textbox",
-									  "wrap-mode","wrapped-both",
-									 "position-to","column-above-text",
-									 "xpos",sXpos.c_str(),
-									 "ypos",sYpos.c_str(),
-									 "frame-width",sWidth.c_str(),
-									  "frame-height",sHeight.c_str(),
-									  "frame-col-xpos",sColXpos.c_str(),
-									  "frame-col-ypos",sColYpos.c_str(),
-									  "background-color", "ffffff",
-									  NULL,NULL};
+		// WARNING: Will need to change this to accomodate variable styles without constantly resetting to solid.
+		//				 Recommend to do whatever is done for thickness, which must also have a default set but not
+		//				 reverted to on every change.
+		// TODO: if(pAP->getProperty("*-thickness", somePropHolder)) sLeftThickness = XML_Char_strdup(somePropHolder); else sLeftThickness = "1px";
+		const XML_Char * props[38] = {"frame-type","textbox",
+					      "wrap-mode","wrapped-both",
+					      "position-to","column-above-text",
+					      "xpos",sXpos.c_str(),
+					      "ypos",sYpos.c_str(),
+					      "frame-width",sWidth.c_str(),
+					      "frame-height",sHeight.c_str(),
+					      "frame-col-xpos",sColXpos.c_str(),
+					      "frame-col-ypos",sColYpos.c_str(),
+					      "frame-page-xpos",sPageXpos.c_str(),
+					      "frame-page-ypos",sPageYpos.c_str(),
+					      "background-color", "ffffff",
+						  "left-style","1",
+						  "right-style","1",
+						  "top-style","1",
+						  "bot-style","1",
+						  "bg-style","1",
+					      "tight-wrap","0",
+					      NULL,NULL};
 //
 // This should place the the frame strux immediately after the block containing
 // position posXY.
@@ -1203,8 +1252,11 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		posAtXY = pCloseBL->getPosition();
 		getDoc()->insertStrux(posAtXY,PTX_SectionFrame,NULL,props,&pfFrame);
 		PT_DocPosition posFrame = pfFrame->getPos();
+		PT_DocPosition posEOD= 0;
+		m_pView->getEditableBounds(true,posEOD);
 		getDoc()->insertStrux(posFrame+1,PTX_Block);
 		getDoc()->insertStrux(posFrame+2,PTX_EndFrame);
+		m_pView->insertParaBreakIfNeededAtPos(posFrame+3);
 
 // Place the insertion point in the Frame
 
@@ -1367,6 +1419,18 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 			sVal = pszWrapMode;
 		}
 		UT_String_setProperty(sFrameProps,sProp,sVal);		
+// tight-wrap
+
+		sProp = "tight-wrap";
+		if(!pSectionAP || !pSectionAP->getProperty("tight-wrap",pszWrapMode))
+		{
+			sVal = "0";
+		}
+		else
+		{
+			sVal = pszWrapMode;
+		}
+		UT_String_setProperty(sFrameProps,sProp,sVal);		
 
 // Xpadding
 
@@ -1541,8 +1605,10 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		UT_String sHeight("");
 		UT_String sColXpos("");
 		UT_String sColYpos("");
+		UT_String sPageXpos("");
+		UT_String sPageYpos("");
 		fl_BlockLayout * pCloseBL = NULL;
-		getFrameStrings(m_recCurFrame.left,m_recCurFrame.top,sXpos,sYpos,sWidth,sHeight,sColXpos,sColYpos,&pCloseBL);
+		getFrameStrings(m_recCurFrame.left,m_recCurFrame.top,sXpos,sYpos,sWidth,sHeight,sColXpos,sColYpos,sPageXpos,sPageYpos,&pCloseBL);
 		sProp = "xpos";
 		sVal = sXpos;
 		UT_String_setProperty(sFrameProps,sProp,sVal);		
@@ -1557,6 +1623,14 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		sVal = sColYpos;
 		UT_String_setProperty(sFrameProps,sProp,sVal);		
 
+
+		sProp = "frame-page-xpos";
+		sVal = sPageXpos;
+		UT_String_setProperty(sFrameProps,sProp,sVal);		
+		sProp = "frame-page-ypos";
+		sVal = sPageYpos;
+		UT_String_setProperty(sFrameProps,sProp,sVal);		
+
 		sProp = "frame-width";
 		sVal = sWidth;
 		UT_String_setProperty(sFrameProps,sProp,sVal);		
@@ -1587,12 +1661,14 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		}
 		PT_DocPosition posStart = m_pFrameLayout->getPosition(true);
 		PT_DocPosition posEnd = posStart + m_pFrameLayout->getLength();
+		bool bHasContent = false;
 		if(isTextBox)
 		{
 			PD_DocumentRange dr_oldFrame;
-			dr_oldFrame.set(getDoc(),posStart+2,posEnd-1);
+			dr_oldFrame.set(getDoc(),posStart+1,posEnd-1);
 			UT_DEBUGMSG(("SEVIOR: Copy to local changing frame \n"));
-			m_pView->copyToLocal(posStart+2,posEnd-1);
+			m_pView->copyToLocal(posStart+1,posEnd-1);
+			bHasContent = true;
 		}
 // Delete the frame
 
@@ -1600,7 +1676,14 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		PL_StruxDocHandle sdhEnd = NULL;
 		posStart = getDoc()->getStruxPosition(sdhStart);
 		getDoc()->getNextStruxOfType(sdhStart, PTX_EndFrame, &sdhEnd);
-		posEnd = getDoc()->getStruxPosition(sdhEnd)+1;
+		if(sdhEnd == NULL)
+		{
+		    posEnd= posStart+1;
+		}
+		else
+		{
+		    posEnd = getDoc()->getStruxPosition(sdhEnd)+1;
+		}
 		UT_uint32 iRealDeleteCount;
 		PP_AttrProp * p_AttrProp_Before = NULL;
 
@@ -1636,12 +1719,22 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		PT_DocPosition posFrame = pfFrame->getPos();
 		if(isTextBox)
 		{
-			getDoc()->insertStrux(posFrame+1,PTX_Block);
-			getDoc()->insertStrux(posFrame+2,PTX_EndFrame);
+			if(!bHasContent)
+			{
+			     getDoc()->insertStrux(posFrame+1,PTX_Block);
+			     getDoc()->insertStrux(posFrame+2,PTX_EndFrame);
+			     m_pView->insertParaBreakIfNeededAtPos(posFrame+3);
+			}
+			else
+			{
+			     getDoc()->insertStrux(posFrame+1,PTX_EndFrame);
+			     m_pView->insertParaBreakIfNeededAtPos(posFrame+2);
+			}
 		}
 		else
 		{
 			getDoc()->insertStrux(posFrame+1,PTX_EndFrame);
+			m_pView->insertParaBreakIfNeededAtPos(posFrame+2);
 		}
 		delete [] atts;
 
@@ -1650,7 +1743,14 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 		if(isTextBox)
 		{
 			UT_DEBUGMSG(("SEVIOR: Pasting from clipboard Frame changed \n"));
-			m_pView->_pasteFromLocalTo(posFrame+2);
+			if(!bHasContent)
+			{
+			     m_pView->_pasteFromLocalTo(posFrame+2);
+			}
+			else
+			{
+			     m_pView->_pasteFromLocalTo(posFrame+1);
+			}
 		}
 // Finish up with the usual stuff
 		getDoc()->setDontImmediatelyLayout(false);
@@ -1736,12 +1836,16 @@ void FV_FrameEdit::mouseRelease(UT_sint32 x, UT_sint32 y)
 /*!
  * This method deletes the current selected frame
  */
-void FV_FrameEdit::deleteFrame(void)
+void FV_FrameEdit::deleteFrame(fl_FrameLayout * pFL)
 {
 	if(m_pFrameLayout == NULL)
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-		return;
+	        m_pFrameLayout = pFL;
+		if(m_pFrameLayout == NULL)
+		{
+		  UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		  return;
+		}
 	}
 	PP_AttrProp * p_AttrProp_Before = NULL;
 
@@ -1841,7 +1945,18 @@ void FV_FrameEdit::drawFrame(bool bWithHandles)
 		if(m_iDraggingWhat == FV_FrameEdit_DragWholeFrame)
 		{
 			GR_Painter painter (getGraphics());
-			m_pFrameImage = painter.genImageFromRectangle(m_recCurFrame);
+			if(m_pFrameLayout->getFrameType() == FL_FRAME_TEXTBOX_TYPE)
+			{
+			      m_pFrameImage = painter.genImageFromRectangle(m_recCurFrame);
+			}
+			else
+		        {
+			      UT_Rect rec = m_recCurFrame;
+			      rec.left = 0;
+			      rec.top = 0;
+				  UT_return_if_fail(m_pFrameLayout->getBackgroundImage());
+			      m_pFrameImage = m_pFrameLayout->getBackgroundImage()->createImageSegment(getGraphics(),rec);
+			}
 		}
 	}
 	else
