@@ -22,6 +22,8 @@
 #include "pd_Style.h"
 #include "pd_Document.h"
 #include "ut_debugmsg.h"
+#include "ut_misc.h"
+
 //#include <limits.h>
 
 PP_Revision::PP_Revision(UT_uint32 Id, PP_RevisionType eType, const XML_Char * props, const XML_Char * attrs):
@@ -42,6 +44,11 @@ PP_Revision::PP_Revision(UT_uint32 Id, PP_RevisionType eType, const XML_Char * p
 		while(p)
 		{
 			char * n = p;
+
+			// skip over spaces ...
+			while(n && *n == ' ')
+				++n;
+			
 			p = strtok(NULL, ";");
 
 			// if we have no p, that means the property is being removed ...
@@ -125,6 +132,53 @@ PP_Revision::PP_Revision(UT_uint32 Id, PP_RevisionType eType, const XML_Char **
 	}
 }
 
+/*!
+    Sets attributes taking care of any nested revision attribute (which needs to be parsed
+    and combined with the current AP set.
+*/
+bool PP_Revision::setAttributes(const XML_Char ** attributes)
+{
+	if(!PP_AttrProp::setAttributes(attributes))
+		return false;
+	
+	return _handleNestedRevAttr();
+}
+
+
+
+bool PP_Revision::_handleNestedRevAttr()
+{
+	const XML_Char * pNestedRev = NULL;
+	getAttribute("revision", pNestedRev);
+	
+	if(pNestedRev)
+	{
+		PP_RevisionAttr NestedAttr(pNestedRev);
+
+		// now remove "revision"
+		setAttribute("revision", NULL);
+		prune();
+
+		// overlay the attrs and props from the revision attribute
+		for(UT_uint32 i = 0; i < NestedAttr.getRevisionsCount(); ++i)
+		{
+			const PP_Revision * pRev = NestedAttr.getNthRevision(i);
+			UT_return_val_if_fail( pRev, false );
+					
+			// ignore inserts and deletes
+			if(pRev->getType() == PP_REVISION_ADDITION || pRev->getType() == PP_REVISION_DELETION)
+				continue;
+
+			setProperties(pRev->getProperties());
+			setAttributes(pRev->getAttributes());
+		}
+
+		prune();
+	}
+
+	return true;
+}
+
 
 /*! converts the internal vector of properties into XML string */
 const XML_Char * PP_Revision::getPropsString()
@@ -472,8 +526,12 @@ void PP_RevisionAttr::pruneForCumulativeResult(PD_Document * pDoc)
 	{
 		PP_Revision * r = (PP_Revision *)m_vRev.getNthItem(i);
 		
-		if(r->getType() == PP_REVISION_DELETION)
+		if(!bDelete && r->getType() == PP_REVISION_DELETION)
+		{
 			bDelete = true;
+			continue; // we do not want the top revision deleted
+		}
+		
 
 		if(bDelete)
 		{
@@ -737,9 +795,12 @@ bool PP_RevisionAttr::isVisible(UT_uint32 id)
 /*! adds id to the revision vector handling the special cases where id
     is already present in this attribute.
 */
-void PP_RevisionAttr::addRevision(UT_uint32 iId, PP_RevisionType eType, const XML_Char ** pAttrs, const XML_Char ** pProps)
+void PP_RevisionAttr::addRevision(UT_uint32 iId, PP_RevisionType eType,
+								  const XML_Char ** pAttrs, const XML_Char ** pProps)
 {
-	for(UT_uint32 i = 0; i < m_vRev.getItemCount(); i++)
+	UT_uint32 i;
+
+	for(i = 0; i < m_vRev.getItemCount(); i++)
 	{
 		PP_Revision * r = (PP_Revision*) m_vRev.getNthItem(i);
 		UT_uint32 r_id = r->getId();
@@ -766,6 +827,7 @@ void PP_RevisionAttr::addRevision(UT_uint32 iId, PP_RevisionType eType, const XM
 				// id, so if queried later we can work out if this
 				// whole fragment should in fact go
 
+				delete r;
 				m_vRev.deleteNthItem(i);
 
 				m_iSuperfluous = iId;
@@ -783,6 +845,7 @@ void PP_RevisionAttr::addRevision(UT_uint32 iId, PP_RevisionType eType, const XM
 				// the case we need to reset m_iSuperfluous, since
 				// this fragment can no more be superfluous
 
+				delete r;
 				m_vRev.deleteNthItem(i);
 
 				if(m_iSuperfluous == iId)
@@ -799,6 +862,7 @@ void PP_RevisionAttr::addRevision(UT_uint32 iId, PP_RevisionType eType, const XM
 				// the new, since the original action did not result
 				// in inserting new text
 
+				delete r;
 				m_vRev.deleteNthItem(i);
 
 				const PP_Revision * pRevision = new PP_Revision(iId, eType, (XML_Char*)0, (XML_Char*)0);
@@ -810,9 +874,11 @@ void PP_RevisionAttr::addRevision(UT_uint32 iId, PP_RevisionType eType, const XM
 				// but now he just wants a format change, in this case
 				// we just replace the old revision with the new
 
+				delete r;
 				m_vRev.deleteNthItem(i);
 
 				const PP_Revision * pRevision = new PP_Revision(iId, eType, pProps, pAttrs);
+				
 				m_vRev.addItem((void*)pRevision);
 			}
 			else if((eType == PP_REVISION_FMT_CHANGE) && (r_type == PP_REVISION_ADDITION))
@@ -822,7 +888,6 @@ void PP_RevisionAttr::addRevision(UT_uint32 iId, PP_RevisionType eType, const XM
 				// so we will keep the old revision record, but need
 				// to merge any existing props in the revision with
 				// the new ones
-
 				r->setProperties(pProps);
 				r->setAttributes(pAttrs);
 			}
@@ -855,6 +920,7 @@ void PP_RevisionAttr::addRevision(UT_uint32 iId, PP_RevisionType eType, const XM
 
 	// if we got here then the item is not in our vector so add it
 	const PP_Revision * pRevision = new PP_Revision(iId, eType, pProps, pAttrs);
+	
 	m_vRev.addItem((void*)pRevision);
 	m_bDirty = true;
 	m_pLastRevision = NULL;
@@ -871,6 +937,7 @@ void PP_RevisionAttr::removeRevisionIdWithType(UT_uint32 iId, PP_RevisionType eT
 
 		if((iId == r->getId()) && (eType == r->getType()))
 		{
+			delete r;
 			m_vRev.deleteNthItem(i);
 			m_bDirty = true;
 			m_pLastRevision = NULL;
@@ -890,6 +957,7 @@ void PP_RevisionAttr::removeRevisionIdTypeless(UT_uint32 iId)
 
 		if(iId == r->getId())
 		{
+			delete r;
 			m_vRev.deleteNthItem(i);
 			m_bDirty = true;
 			m_pLastRevision = NULL;
@@ -908,6 +976,7 @@ void PP_RevisionAttr::removeRevision(const PP_Revision * pRev)
 
 		if(r == pRev)
 		{
+			delete r;
 			m_vRev.deleteNthItem(i);
 			m_bDirty = true;
 			m_pLastRevision = NULL;
@@ -922,13 +991,36 @@ void PP_RevisionAttr::removeRevision(const PP_Revision * pRev)
 */
 void PP_RevisionAttr::removeAllLesserOrEqualIds(UT_uint32 iId)
 {
-	for(UT_uint32 i = 0; i < m_vRev.getItemCount(); i++)
+	for(UT_sint32 i = 0; i < (UT_sint32)m_vRev.getItemCount(); i++)
 	{
 		PP_Revision * r = (PP_Revision *)m_vRev.getNthItem(i);
 
 		if(iId >= r->getId())
 		{
+			delete r;
 			m_vRev.deleteNthItem(i);
+			--i; // the vector just shrunk
+		}
+	}
+
+	m_bDirty = true;
+	m_pLastRevision = NULL;
+}
+
+/*! removes all IDs from the attribute whose value is higher or
+    equal the given id
+*/
+void PP_RevisionAttr::removeAllHigherOrEqualIds(UT_uint32 iId)
+{
+	for(UT_sint32 i = 0; i < (UT_sint32)m_vRev.getItemCount(); i++)
+	{
+		PP_Revision * r = (PP_Revision *)m_vRev.getNthItem(i);
+
+		if(iId <= r->getId())
+		{
+			delete r;
+			m_vRev.deleteNthItem(i);
+			--i; // the vector just shrunk
 		}
 	}
 
