@@ -48,6 +48,18 @@ bool pt_PieceTable::appendStrux(PTStruxType pts, const XML_Char ** attributes, p
 	pf_Frag_Strux * pfs = NULL;
 	if(!_makeStrux(pts, attributes, pfs) || !pfs)
 		return false;
+
+	if(attributes)
+	{
+		const XML_Char * pXID = UT_getAttribute(PT_XID_ATTRIBUTE_NAME, attributes);
+		UT_uint32 iXID = 0;
+		if(pXID && *pXID)
+		{
+			iXID = atoi(pXID);
+			pfs->setXID(iXID);
+		}
+	}
+	
 	pf_Frag * pfPrev = m_fragments.getLast();
 	bool bDoInsertFmt = false;
 	if(pfPrev != NULL && pfPrev->getType() == pf_Frag::PFT_Strux)
@@ -73,7 +85,6 @@ pf_Frag * pt_PieceTable::_findLastStruxOfType(pf_Frag * pfStart, PTStruxType pst
 	UT_return_val_if_fail( pfStart, NULL );
 
 	pf_Frag * pf = pfStart;
-	
 	while(pf)
 	{
 		if(pf->getType() == pf_Frag::PFT_Strux)
@@ -161,41 +172,10 @@ pf_Frag * pt_PieceTable::_findLastStruxOfType(pf_Frag * pfStart, PTStruxType pst
 						pf = pf->getPrev();
 					}
 				}
-				if(pfs->getStruxType() == PTX_EndCell)
-				{
-					while(pf)
-					{
-						if(pf->getType() == pf_Frag::PFT_Strux)
-						{
-							pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux*>(pf);
-
-							if(pfs->getStruxType() == PTX_SectionCell)
-								break;
-						}
-
-						pf = pf->getPrev();
-					}
-				}
-				if(pfs->getStruxType() == PTX_EndTable)
-				{
-					while(pf)
-					{
-						if(pf->getType() == pf_Frag::PFT_Strux)
-						{
-							pf_Frag_Strux * pfs = static_cast<pf_Frag_Strux*>(pf);
-
-							if(pfs->getStruxType() == PTX_SectionTable)
-								break;
-						}
-
-						pf = pf->getPrev();
-					}
-				}
-			
 			}
 		}
-		
-		pf = pf->getPrev();
+		if(pf)
+			pf = pf->getPrev();
 	}
 
 	return pf;
@@ -227,7 +207,7 @@ bool pt_PieceTable::appendLastStruxFmt(PTStruxType pst, const XML_Char ** attrib
 	pf_Frag * pf = m_fragments.getLast();
 
 	UT_return_val_if_fail ( pf, false );
-
+	
 	pf = _findLastStruxOfType(pf, pst, bSkipEmbededSections);
 	
 	UT_return_val_if_fail( pf, false );
@@ -405,6 +385,17 @@ bool pt_PieceTable::appendObject(PTObjectType pto, const XML_Char ** attributes)
 	if(!_makeObject(pto,attributes,pfo) || !pfo)
 		return false;
 
+	if(attributes)
+	{
+		const XML_Char * pXID = UT_getAttribute(PT_XID_ATTRIBUTE_NAME, attributes);
+		UT_uint32 iXID = 0;
+		if(pXID && *pXID)
+		{
+			iXID = atoi(pXID);
+			pfo->setXID(iXID);
+		}
+	}
+	
 	m_fragments.appendFrag(pfo);
 	return true;
 }
@@ -429,6 +420,17 @@ bool pt_PieceTable::insertStruxBeforeFrag(pf_Frag * pF, PTStruxType pts,
 	if(!_makeStrux(pts, attributes, pfs) || !pfs)
 		return false;
 
+	if(attributes)
+	{
+		const XML_Char * pXID = UT_getAttribute(PT_XID_ATTRIBUTE_NAME, attributes);
+		UT_uint32 iXID = 0;
+		if(pXID && *pXID)
+		{
+			iXID = atoi(pXID);
+			pfs->setXID(iXID);
+		}
+	}
+	
 	m_fragments.insertFragBefore(pF, pfs);
 	if (ppfs_ret)
 		*ppfs_ret = pfs;
@@ -507,6 +509,17 @@ bool pt_PieceTable::insertObjectBeforeFrag(pf_Frag * pF, PTObjectType pto,
 	if(!_makeObject(pto,attributes,pfo) || !pfo)
 		return false;
 
+	if(attributes)
+	{
+		const XML_Char * pXID = UT_getAttribute(PT_XID_ATTRIBUTE_NAME, attributes);
+		UT_uint32 iXID = 0;
+		if(pXID && *pXID)
+		{
+			iXID = atoi(pXID);
+			pfo->setXID(iXID);
+		}
+	}
+	
 	m_fragments.insertFragBefore(pF, pfo);
 	return true;
 }
