@@ -246,6 +246,7 @@ void AP_CocoaDialog_Options::runModal(XAP_Frame * pFrame)
 
 	defn(Switch_AutoSave,				AP_PreferenceScheme::bo_AutoSave);
 	defn(Switch_CheckSpelling,			AP_PreferenceScheme::bo_CheckSpelling);
+	defn(Switch_CheckGrammar,			AP_PreferenceScheme::bo_CheckGrammar);
 	defn(Switch_CursorBlink,			AP_PreferenceScheme::bo_CursorBlink);
 	defn(Switch_DirectionMarkers,		AP_PreferenceScheme::bo_DirectionMarkers);
 	defn(Switch_DirectionRTL,			AP_PreferenceScheme::bo_DirectionRTL);
@@ -261,7 +262,6 @@ void AP_CocoaDialog_Options::runModal(XAP_Frame * pFrame)
 	defn(Switch_Ruler,					AP_PreferenceScheme::bo_Ruler);
 	defn(Switch_SaveScheme,				AP_PreferenceScheme::bo_SaveScheme);
 	defn(Switch_ScreenColor,			AP_PreferenceScheme::bo_ScreenColor);
-	defn(Switch_SmartQuotes,			AP_PreferenceScheme::bo_SmartQuotes);
 	defn(Switch_Splash,					AP_PreferenceScheme::bo_Splash);
 	defn(Switch_StatusBar,				AP_PreferenceScheme::bo_StatusBar);
 	defn(Switch_SuggestCorrections,		AP_PreferenceScheme::bo_SuggestCorrections);
@@ -339,6 +339,7 @@ void AP_CocoaDialog_Options::runModal(XAP_Frame * pFrame)
 	LocalizeControl(oSwitch_DirectionMarkers,		pSS,XAP_STRING_ID_DLG_Options_Label_DirMarkerAfterClosingParenthesis);
 
 	LocalizeControl(oSwitch_CheckSpelling,			pSS, AP_STRING_ID_DLG_Options_Label_SpellCheckAsType);
+	LocalizeControl(oSwitch_CheckGrammar,			pSS, AP_STRING_ID_DLG_Options_Label_GrammarCheck);
 	LocalizeControl(oSwitch_HighlightMisspelled,	pSS, AP_STRING_ID_DLG_Options_Label_SpellHighlightMisspelledWords);
 	LocalizeControl(oSwitch_SuggestCorrections,		pSS, AP_STRING_ID_DLG_Options_Label_SpellSuggest);
 	LocalizeControl(oSwitch_MainDictionaryOnly,		pSS, AP_STRING_ID_DLG_Options_Label_SpellMainOnly);
@@ -359,6 +360,7 @@ void AP_CocoaDialog_Options::runModal(XAP_Frame * pFrame)
 // TODO: schemes popup
 // TODO: page sizes popup
 // TODO: custom dictionaries popup
+// ?? AP_STRING_ID_DLG_Options_Label_Grammar
 
 	// ...
 
@@ -371,6 +373,23 @@ void AP_CocoaDialog_Options::runModal(XAP_Frame * pFrame)
 	[oPopUp_Units addItemsWithTitles:m_UnitsList];
 
 	[self sync];
+
+	switch (m_xap->getInitialPageNum())
+		{
+		default:
+		case 1:
+			[oTabView selectTabViewItemWithIdentifier:@"General"];
+			break;
+		case 2:
+			[oTabView selectTabViewItemWithIdentifier:@"Interface"];
+			break;
+		case 3:
+			[oTabView selectTabViewItemWithIdentifier:@"Documents"];
+			break;
+		case 4:
+			[oTabView selectTabViewItemWithIdentifier:@"Spelling"];
+			break;
+		}
 }
 
 - (IBAction)aButton:(id)sender
@@ -544,12 +563,16 @@ void AP_CocoaDialog_Options::runModal(XAP_Frame * pFrame)
 
 	if (m_pActiveScheme)
 		{
-			[oStepper_Minutes setEnabled:YES];
+			bool bEditable = false;
 
-			[oField_Minutes setEnabled:YES];
+			BOOL bAutoSaveEnabled = m_pActiveScheme->getBoolOptionValue(AP_PreferenceScheme::bo_AutoSave, bEditable) ? YES : NO;
+
+			[oStepper_Minutes setEnabled:bAutoSaveEnabled];
+
+			[oField_Minutes setEnabled:bAutoSaveEnabled];
 			[oField_Minutes setStringValue:[NSString stringWithUTF8String:(m_pActiveScheme->getAutoSaveMinutes())]];
 
-			[oField_Extension setEnabled:YES];
+			[oField_Extension setEnabled:bAutoSaveEnabled];
 			[oField_Extension setStringValue:[NSString stringWithUTF8String:(m_pActiveScheme->getAutoSaveExtension())]];
 
 			[oTableView_InterfaceLanguage selectRow:((int) m_pActiveScheme->getUILangIndex()) byExtendingSelection:NO];
