@@ -303,7 +303,7 @@ AP_UnixDialog_Goto::onJumpClicked ()
 		return;		
 
 	UT_uint32 len = UT_XML_strlen (text);
-	UT_UCSChar *number = (UT_UCSChar *) malloc (sizeof (UT_UCSChar) * len + 1);
+	UT_UCSChar *number = (UT_UCSChar *) malloc (sizeof (UT_UCSChar) * (len + 1));
 	UT_UCS4_strcpy_utf8_char (number, text);
 	UT_DEBUGMSG (("ROB: onJumpClicked () gotoTarget () target='%d' number='%s'\n", m_JumpTarget, text));
 	getView()->gotoTarget (m_JumpTarget, number);
