@@ -17,6 +17,12 @@
  * 02111-1307, USA.
  */
 
+/*
+ * Port to Maemo Development Platform
+ * Author: INdT - Renato Araujo <renato.filho@indt.org.br>
+ */
+
+
 #undef GTK_DISABLE_DEPRECATED
 
 #include <stdlib.h>
@@ -35,8 +41,8 @@
 #include "xap_Frame.h"
 
 #include "ap_Dialog_Id.h"
-
 #include "ap_Strings.h"
+#include "ap_EditMethods.h"
 
 #include "ap_Preview_Paragraph.h"
 #include "ap_UnixDialog_Paragraph.h"
@@ -111,8 +117,11 @@ static gboolean do_update(gpointer p)
 // FIXME!!! Could get nasty crash if the dlg is destroyed while 
 // a redraw is pending....
 //
+#ifdef HAVE_HILDON
+#else
 	AP_UnixDialog_Paragraph * dlg = (AP_UnixDialog_Paragraph *) p;
 	dlg->event_PreviewAreaExposed();
+#endif
 	return FALSE;
 }
 
@@ -145,6 +154,8 @@ void AP_UnixDialog_Paragraph::runModal(XAP_Frame * pFrame)
 	// Show the top level dialog,
 	gtk_widget_show(mainWindow);
 
+#ifdef HAVE_HILDON
+#else
 	// *** this is how we add the gc ***
 	{
 		// attach a new graphics context to the drawing area
@@ -167,18 +178,27 @@ void AP_UnixDialog_Paragraph::runModal(XAP_Frame * pFrame)
 	// sync all controls once to get started
 	// HACK: the first arg gets ignored
 	_syncControls(id_MENU_ALIGNMENT, true);
+#endif
 
-
-	switch(abiRunModalDialog(GTK_DIALOG(mainWindow), pFrame, this, BUTTON_CANCEL, false))
-	{
-	case BUTTON_OK:
-	  event_OK(); break;
-	case BUTTON_TABS:
-	  event_Tabs (); break;
-	default:
-	  event_Cancel(); break;
-	}
-
+	bool tabs;
+	do {
+		switch(abiRunModalDialog(GTK_DIALOG(mainWindow), pFrame, this, BUTTON_CANCEL, false))
+		{
+		case BUTTON_OK:
+		  event_OK(); 
+		  tabs = false;
+		  break;
+		case BUTTON_TABS:
+		  event_Tabs ();
+		  tabs = true;
+		  break;
+		default:
+		  event_Cancel();
+		  tabs = false;
+		  break;
+		}
+	} while (tabs);
+	
 	abiDestroyWidget(mainWindow);
 }
 
@@ -196,6 +216,8 @@ void AP_UnixDialog_Paragraph::event_Cancel(void)
 
 void AP_UnixDialog_Paragraph::event_Tabs(void)
 {
+	AV_View *pView = m_pFrame->getCurrentView();
+	s_doTabDlg(static_cast<FV_View*>(pView));
 	m_answer = AP_Dialog_Paragraph::a_TABS;
 }
 
@@ -271,8 +293,11 @@ void AP_UnixDialog_Paragraph::event_CheckToggled(GtkWidget * widget)
 
 void AP_UnixDialog_Paragraph::event_PreviewAreaExposed(void)
 {
+#ifdef HAVE_HILDON
+#else
 	if (m_paragraphPreview)
 		m_paragraphPreview->draw();
+#endif	
 }
 
 /*****************************************************************/
@@ -389,7 +414,11 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 
 
 	// "Indents and Spacing" page
+#ifdef HAVE_HILDON
+	boxSpacing = gtk_table_new (3, 4, FALSE);
+#else	
 	boxSpacing = gtk_table_new (7, 4, FALSE);
+#endif
 	gtk_widget_show (boxSpacing);
 	gtk_table_set_row_spacings (GTK_TABLE(boxSpacing), 5);
 	gtk_table_set_col_spacings (GTK_TABLE(boxSpacing), 5);
@@ -426,6 +455,11 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
 
 	listAlignment_menu = gtk_menu_new ();
+	
+	glade_menuitem = gtk_menu_item_new_with_label(" "); // add an empty menu option to fix bug 594
+	gtk_widget_show (glade_menuitem);
+	gtk_menu_shell_append (GTK_MENU_SHELL (listAlignment_menu), glade_menuitem);
+	
 	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_AlignLeft,s);
 	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	glade_menuitem = gtk_menu_item_new_with_label (unixstr);
@@ -545,17 +579,33 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 	gtk_table_attach ( GTK_TABLE(boxSpacing), labelSpecial, 2,3, 2,3,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
+#ifdef HAVE_HILDON
+	gtk_label_set_justify (GTK_LABEL (labelSpecial), GTK_JUSTIFY_RIGHT);
+	gtk_misc_set_alignment (GTK_MISC (labelSpecial), 1, 0.5);
+#else
 	gtk_label_set_justify (GTK_LABEL (labelSpecial), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment (GTK_MISC (labelSpecial), 0, 0.5);
+#endif
 
 	listSpecial = gtk_option_menu_new ();
 	/**/ g_object_set_data(G_OBJECT(listSpecial), WIDGET_ID_TAG, (gpointer) id_MENU_SPECIAL_INDENT);
 	gtk_widget_show (listSpecial);
+#ifdef HAVE_HILDON
+	gtk_table_attach ( GTK_TABLE(boxSpacing), listSpecial, 3, 4, 2, 3,
+                    (GtkAttachOptions) (GTK_SHRINK),
+                    (GtkAttachOptions) (GTK_FILL), 0, 0);
+#else
 	gtk_table_attach ( GTK_TABLE(boxSpacing), listSpecial, 2,3, 3,4,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
+
+#endif
 	listSpecial_menu = gtk_menu_new ();
 
+	glade_menuitem = gtk_menu_item_new_with_label(" "); // add an empty menu option to fix bug 594
+	gtk_widget_show (glade_menuitem);
+	gtk_menu_shell_append (GTK_MENU_SHELL (listSpecial_menu), glade_menuitem);
+
 	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_SpecialNone,s);
 	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	glade_menuitem = gtk_menu_item_new_with_label (unixstr);
@@ -590,19 +640,30 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 	labelBy = gtk_label_new (unixstr);
 	FREEP(unixstr);
 	gtk_widget_show (labelBy);
+#ifdef HAVE_HILDON	
+	gtk_table_attach ( GTK_TABLE(boxSpacing), labelBy, 2, 3, 3, 4,
+                    (GtkAttachOptions) (GTK_FILL),
+                    (GtkAttachOptions) (GTK_FILL), 0, 0);
+	gtk_label_set_justify (GTK_LABEL (labelBy), GTK_JUSTIFY_RIGHT);
+	gtk_misc_set_alignment (GTK_MISC (labelBy), 1, 0.5);
+#else
 	gtk_table_attach ( GTK_TABLE(boxSpacing), labelBy, 3,4, 2,3,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
 	gtk_label_set_justify (GTK_LABEL (labelBy), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment (GTK_MISC (labelBy), 0, 0.5);
-
+#endif
 //	spinbuttonBy_adj = gtk_adjustment_new (0.5, 0, 100, 0.1, 10, 10);
 //	spinbuttonBy = gtk_spin_button_new (NULL, 1, 1);
 	spinbuttonBy = gtk_entry_new();
 	/**/ g_object_set_data(G_OBJECT(spinbuttonBy), WIDGET_ID_TAG, (gpointer) id_SPIN_SPECIAL_INDENT);
 	gtk_widget_show (spinbuttonBy);
 	gtk_table_attach ( GTK_TABLE(boxSpacing), spinbuttonBy, 3,4, 3,4,
+#ifdef HAVE_HILDON
+                    (GtkAttachOptions) (GTK_SHRINK|GTK_EXPAND),
+#else
                     (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
+#endif
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
 
 
@@ -613,13 +674,11 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	labelSpacing = gtk_label_new (unixstr);
 	FREEP(unixstr);
-	gtk_widget_show (labelSpacing);
 	gtk_box_pack_start (GTK_BOX (hboxSpacing), labelSpacing, FALSE, FALSE, 0);
 	gtk_label_set_justify (GTK_LABEL (labelSpacing), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment (GTK_MISC (labelSpacing), 0, 0.5);
 
 	hseparator1 = gtk_hseparator_new ();
-	gtk_widget_show (hseparator1);
 	gtk_box_pack_start (GTK_BOX (hboxSpacing), hseparator1, TRUE, TRUE, 0);
 	gtk_table_attach ( GTK_TABLE(boxSpacing), hboxSpacing, 0,4, 4,5,
                     (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
@@ -629,7 +688,6 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	labelBefore = gtk_label_new (unixstr);
 	FREEP(unixstr);
-	gtk_widget_show (labelBefore);
 	gtk_table_attach ( GTK_TABLE(boxSpacing), labelBefore, 0,1, 5,6,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
@@ -640,7 +698,6 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 //	spinbuttonBefore = gtk_spin_button_new (NULL, 1, 1);
 	spinbuttonBefore = gtk_entry_new();
 	/**/ g_object_set_data(G_OBJECT(spinbuttonBefore), WIDGET_ID_TAG, (gpointer) id_SPIN_BEFORE_SPACING);
-	gtk_widget_show (spinbuttonBefore);
 	gtk_table_attach ( GTK_TABLE(boxSpacing), spinbuttonBefore, 1,2, 5,6,
                     (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
@@ -649,7 +706,6 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	labelAfter = gtk_label_new (unixstr);
 	FREEP(unixstr);
-	gtk_widget_show (labelAfter);
 	gtk_table_attach ( GTK_TABLE(boxSpacing), labelAfter, 0,1, 6,7,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
@@ -660,7 +716,6 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 //	spinbuttonAfter = gtk_spin_button_new (NULL, 1, 1);
 	spinbuttonAfter = gtk_entry_new();
 	/**/ g_object_set_data(G_OBJECT(spinbuttonAfter), WIDGET_ID_TAG, (gpointer) id_SPIN_AFTER_SPACING);
-	gtk_widget_show (spinbuttonAfter);
 	gtk_table_attach ( GTK_TABLE(boxSpacing), spinbuttonAfter, 1,2, 6,7,
                     (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
@@ -669,7 +724,6 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	labelLineSpacing = gtk_label_new (unixstr);
 	FREEP(unixstr);
-	gtk_widget_show (labelLineSpacing);
 	gtk_table_attach ( GTK_TABLE(boxSpacing), labelLineSpacing, 2,3, 5,6,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
@@ -678,11 +732,15 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 
 	listLineSpacing = gtk_option_menu_new ();
 	/**/ g_object_set_data(G_OBJECT(listLineSpacing), WIDGET_ID_TAG, (gpointer) id_MENU_SPECIAL_SPACING);
-	gtk_widget_show (listLineSpacing);
 	gtk_table_attach ( GTK_TABLE(boxSpacing), listLineSpacing, 2,3, 6,7,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
 	listLineSpacing_menu = gtk_menu_new ();
+	
+	glade_menuitem = gtk_menu_item_new_with_label(" "); // add an empty menu option to fix bug 594
+	gtk_widget_show (glade_menuitem);
+	gtk_menu_shell_append (GTK_MENU_SHELL (listLineSpacing_menu), glade_menuitem);	
+	
 	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_SpacingSingle,s);
 	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	glade_menuitem = gtk_menu_item_new_with_label (unixstr);
@@ -743,7 +801,6 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	labelAt = gtk_label_new (unixstr);
 	FREEP(unixstr);
-	gtk_widget_show (labelAt);
 	gtk_table_attach ( GTK_TABLE(boxSpacing), labelAt, 3,4, 5,6,
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
@@ -754,12 +811,24 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 //	spinbuttonAt = gtk_spin_button_new (NULL, 1, 1);
 	spinbuttonAt = gtk_entry_new();
 	/**/ g_object_set_data(G_OBJECT(spinbuttonAt), WIDGET_ID_TAG, (gpointer) id_SPIN_SPECIAL_SPACING);
-	gtk_widget_show (spinbuttonAt);
 	gtk_table_attach ( GTK_TABLE(boxSpacing), spinbuttonAt, 3,4, 6,7,
                     (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
                     (GtkAttachOptions) (GTK_FILL), 0, 0);
 
-
+//TODO: In hildon only hide components for future features
+#ifdef HAVE_HILDON					
+#else
+	gtk_widget_show (labelSpacing);
+	gtk_widget_show (hseparator1);
+	gtk_widget_show (labelBefore);
+	gtk_widget_show (spinbuttonBefore);
+	gtk_widget_show (labelAfter);
+	gtk_widget_show (spinbuttonAfter);
+	gtk_widget_show (labelLineSpacing);
+	gtk_widget_show (listLineSpacing);
+	gtk_widget_show (labelAt);	
+	gtk_widget_show (spinbuttonAt);
+#endif /* HAVE_HILDON */ 	
 
 	// The "Line and Page Breaks" page
 	boxBreaks = gtk_table_new (6, 2, FALSE);
@@ -865,8 +934,9 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
                     (GtkAttachOptions) (GTK_FILL),
                     (GtkAttachOptions) (GTK_FILL), 0, 0 );
 
-
 	// End of notebook. Next comes the preview area.
+#ifdef HAVE_HILDON
+#else
 	hboxPreview = gtk_hbox_new (FALSE, 5);
 	gtk_widget_show (hboxPreview);
 
@@ -899,7 +969,7 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 	drawingareaPreview = createDrawingArea ();
 	gtk_widget_show (drawingareaPreview);
 	gtk_container_add (GTK_CONTAINER (framePreview), drawingareaPreview);
-
+#endif
 
 	// Update member variables with the important widgets that
 	// might need to be queried or altered later.
@@ -926,7 +996,11 @@ GtkWidget * AP_UnixDialog_Paragraph::_constructWindowContents(GtkWidget *windowM
 //	m_spinbuttonAt_adj = spinbuttonAt_adj;
 	m_spinbuttonAt = spinbuttonAt;
 
+#ifdef HAVE_HILDON
+	m_drawingareaPreview = NULL;
+#else
 	m_drawingareaPreview = drawingareaPreview;
+#endif 
 
 	m_checkbuttonWidowOrphan = checkbuttonWidowOrphan;
 	m_checkbuttonKeepLines = checkbuttonKeepLines;
@@ -1011,11 +1085,14 @@ void AP_UnixDialog_Paragraph::_connectCallbackSignals(void)
 	g_signal_connect(G_OBJECT(m_checkbuttonDomDirection), "toggled",
 					   G_CALLBACK(s_check_toggled), (gpointer) this);
 
+#ifdef HAVE_HILDON
+#else
 	// the expose event off the preview
 	g_signal_connect(G_OBJECT(m_drawingareaPreview),
 					   "expose_event",
 					   G_CALLBACK(s_preview_exposed),
 					   (gpointer) this);
+#endif
 }
 
 void AP_UnixDialog_Paragraph::_populateWindowData(void)
@@ -1154,16 +1231,22 @@ void AP_UnixDialog_Paragraph::_syncControls(tControl changed, bool bAll /* = fal
 		{
 		case id_SPIN_LEFT_INDENT:
 			gtk_entry_set_text(GTK_ENTRY(m_spinbuttonLeft), 	_getSpinItemValue(id_SPIN_LEFT_INDENT));
+			break;
 		case id_SPIN_RIGHT_INDENT:
 			gtk_entry_set_text(GTK_ENTRY(m_spinbuttonRight), 	_getSpinItemValue(id_SPIN_RIGHT_INDENT));
+			break;
 		case id_SPIN_SPECIAL_INDENT:
 			gtk_entry_set_text(GTK_ENTRY(m_spinbuttonBy), 		_getSpinItemValue(id_SPIN_SPECIAL_INDENT));
+			break;
 		case id_SPIN_BEFORE_SPACING:
 			gtk_entry_set_text(GTK_ENTRY(m_spinbuttonBefore), 	_getSpinItemValue(id_SPIN_BEFORE_SPACING));
+			break;
 		case id_SPIN_AFTER_SPACING:
 			gtk_entry_set_text(GTK_ENTRY(m_spinbuttonAfter), 	_getSpinItemValue(id_SPIN_AFTER_SPACING));
+			break;
 		case id_SPIN_SPECIAL_SPACING:
 			gtk_entry_set_text(GTK_ENTRY(m_spinbuttonAt), 		_getSpinItemValue(id_SPIN_SPECIAL_SPACING));
+			break;
 		default:
 			break;
 		}
