@@ -510,7 +510,7 @@ void AP_Win32FrameImpl::_setXScrollRange(AP_FrameData * pData, AV_View *pView)
 	GetClientRect(m_hwndDocument, &r);
 	const UT_uint32 iWindowWidth = r.right - r.left;
 	const UT_uint32 iWidth = pView->getGraphics()->tdu(pData->m_pDocLayout->getWidth());
-
+	XAP_Frame::tZoomType tZoom = getFrame()->getZoomType();	
 	SCROLLINFO si = { 0 };
 
 	si.cbSize = sizeof(si);
@@ -522,6 +522,34 @@ void AP_Win32FrameImpl::_setXScrollRange(AP_FrameData * pData, AV_View *pView)
 	SetScrollInfo(m_hWndHScroll, SB_CTL, &si, TRUE);
 
 	pView->sendHorizontalScrollEvent(pView->getGraphics()->tlu(si.nPos),pView->getGraphics()->tlu(si.nMax-si.nPage));
+
+	// hide the horizontal scrollbar if the scroll range is such that the window can contain it all
+	// show it otherwise
+	// Hide the horizontal scrollbar if we've set to page width or fit to page.
+	// This stops a resizing race condition.
+ 	if (iWidth <= iWindowWidth || tZoom == XAP_Frame::z_PAGEWIDTH || tZoom == XAP_Frame::z_WHOLEPAGE)
+	{	 	
+		if (IsWindowVisible (m_hWndHScroll)) {
+			RECT r;
+
+			ShowWindow (m_hWndHScroll, SW_HIDE);
+			ShowWindow (m_hWndGripperHack, SW_HIDE);
+			GetClientRect(m_hwndContainer, &r);
+			_onSize(pData, r.right - r.left, r.bottom - r.top);
+		}
+	}
+ 	else
+	{	
+		if (!IsWindowVisible (m_hWndHScroll)) {
+			RECT r;
+
+			ShowWindow(m_hWndHScroll, SW_NORMAL);
+			ShowWindow (m_hWndGripperHack, SW_NORMAL);
+			GetClientRect(m_hwndContainer, &r);
+			_onSize(pData, r.right - r.left, r.bottom - r.top);
+		}		
+	}
+
 }
 
 void AP_Win32FrameImpl::_setYScrollRange(AP_FrameData * pData, AV_View *pView)
@@ -646,17 +674,20 @@ void AP_Win32FrameImpl::_onSize(AP_FrameData* pData, int nWidth, int nHeight)
 	UT_return_if_fail(m_hWndHScroll);
 	UT_return_if_fail(m_hWndGripperHack);
 
-	const int cyHScroll = GetSystemMetrics(SM_CYHSCROLL);
+	int cyHScroll = GetSystemMetrics(SM_CYHSCROLL);
 	const int cxVScroll = GetSystemMetrics(SM_CXVSCROLL);
 	int yTopRulerHeight = 0;
-	int xLeftRulerWidth = 0;
+	int xLeftRulerWidth = 0;	
 
 	_getRulerSizes(pData, yTopRulerHeight, xLeftRulerWidth);
 
-	MoveWindow(m_hWndVScroll, nWidth-cxVScroll, 0, cxVScroll, nHeight-cyHScroll, TRUE);
 	MoveWindow(m_hWndHScroll, 0, nHeight-cyHScroll, nWidth - cxVScroll, cyHScroll, TRUE);
 	MoveWindow(m_hWndGripperHack, nWidth-cxVScroll, nHeight-cyHScroll, cxVScroll, cyHScroll, TRUE);
 
+	if (IsWindowVisible (m_hWndHScroll) == FALSE)
+		cyHScroll = 0;	
+	
+	MoveWindow(m_hWndVScroll, nWidth-cxVScroll, 0, cxVScroll, nHeight-cyHScroll, TRUE);		
 
 	if (m_hwndTopRuler)
 	{
@@ -670,7 +701,7 @@ void AP_Win32FrameImpl::_onSize(AP_FrameData* pData, int nWidth, int nHeight)
 
 	MoveWindow(m_hwndDocument, xLeftRulerWidth, yTopRulerHeight,
 			   nWidth - xLeftRulerWidth - cxVScroll,
-			   nHeight - yTopRulerHeight - cyHScroll, TRUE);
+			   nHeight - yTopRulerHeight - cyHScroll, TRUE);	
 }
 
 
@@ -1011,7 +1042,7 @@ LRESULT CALLBACK AP_Win32FrameImpl::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARA
 {
 	AP_Win32Frame * f = reinterpret_cast<AP_Win32Frame *>(GetWindowLong(hwnd, GWL_USERDATA));
 
-	if (!f)
+	if (!f || !IsWindow(hwnd))
 	{
 		return UT_DefWindowProc(hwnd, iMsg, wParam, lParam);
 	}
