@@ -22,6 +22,7 @@
 #include "ut_types.h"
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
+#include "ut_Win32OS.h"
 #include "ap_Win32LeftRuler.h"
 #include "gr_Win32Graphics.h"
 #include "xap_Win32App.h"
@@ -74,27 +75,15 @@ void AP_Win32LeftRuler::setView(AV_View * pView)
 
 bool AP_Win32LeftRuler::RegisterClass(XAP_Win32App * app)
 {
-	WNDCLASSEX  wndclass;
 	ATOM a;
 	
 	// register class for the top ruler
 	sprintf(s_LeftRulerWndClassName, "%sLeftRuler", app->getApplicationName());
 
-	memset(&wndclass, 0, sizeof(wndclass));
-	wndclass.cbSize        = sizeof(wndclass);
-	wndclass.style         = CS_DBLCLKS | CS_OWNDC;
-	wndclass.lpfnWndProc   = AP_Win32LeftRuler::_LeftRulerWndProc;
-	wndclass.cbClsExtra    = 0;
-	wndclass.cbWndExtra    = 0;
-	wndclass.hInstance     = app->getInstance();
-	wndclass.hIcon         = NULL;
-	wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
-	wndclass.hbrBackground = GetSysColorBrush(COLOR_BTNFACE);
-	wndclass.lpszMenuName  = NULL;
-	wndclass.lpszClassName = s_LeftRulerWndClassName;
-	wndclass.hIconSm       = NULL;
-
-	a = RegisterClassEx(&wndclass);
+	a = UT_RegisterClassEx(CS_DBLCLKS | CS_OWNDC, AP_Win32LeftRuler::_LeftRulerWndProc, app->getInstance(),
+						   NULL, LoadCursor(NULL, IDC_ARROW), GetSysColorBrush(COLOR_BTNFACE), NULL,
+						   NULL, s_LeftRulerWndClassName);
+	
 	UT_ASSERT_HARMLESS(a);
 
 	return true;
@@ -106,7 +95,7 @@ HWND AP_Win32LeftRuler::createWindow(HWND hwndContainer,
 {
 		
 	XAP_Win32App * app = static_cast<XAP_Win32App *>(m_pFrame->getApp());
-	m_hwndLeftRuler = CreateWindowEx(0, s_LeftRulerWndClassName, NULL,
+	m_hwndLeftRuler = UT_CreateWindowEx(0, s_LeftRulerWndClassName, NULL,
 									 WS_CHILD | WS_VISIBLE,
 									 left, top, s_iFixedWidth, height,
 									 hwndContainer, NULL, app->getInstance(), NULL);
@@ -151,7 +140,7 @@ LRESULT CALLBACK AP_Win32LeftRuler::_LeftRulerWndProc(HWND hwnd, UINT iMsg, WPAR
 	AP_Win32LeftRuler * pRuler = GWL(hwnd);
 
 	if (!pRuler)
-		return DefWindowProc(hwnd, iMsg, wParam, lParam);
+		return UT_DefWindowProc(hwnd, iMsg, wParam, lParam);
 
 	GR_Win32Graphics * pG = static_cast<GR_Win32Graphics *>(pRuler->m_pG);
 		
@@ -232,5 +221,5 @@ LRESULT CALLBACK AP_Win32LeftRuler::_LeftRulerWndProc(HWND hwnd, UINT iMsg, WPAR
 		break;
 	}
 
-	return DefWindowProc(hwnd, iMsg, wParam, lParam);
+	return UT_DefWindowProc(hwnd, iMsg, wParam, lParam);
 }
