@@ -28,6 +28,7 @@
 #include "ut_assert.h"
 #include "ut_string.h"
 #include "ut_debugmsg.h"
+#include "ut_locale.h"
 
 #include "xap_App.h"
 #include "xap_Dialog_Id.h"
@@ -876,6 +877,7 @@ static UT_UTF8String s_canonical_thickness (float thickness)
 	}
 	else {
 		char buf[16];
+		UT_LocaleTransactor t(LC_NUMERIC, "C");
 		sprintf(buf, "%.2fpt", thickness);
 		sThick = buf;
 	}
@@ -898,6 +900,7 @@ static UT_UTF8String s_canonical_thickness (const UT_UTF8String & sThickness, fl
 	}
 	else {
 		char buf[16];
+		UT_LocaleTransactor t(LC_NUMERIC, "C");
 		sprintf(buf, "%.2fpt", thickness);
 		sThick = buf;
 	}
@@ -1064,6 +1067,7 @@ void AP_Dialog_FormatFrame::_createPreviewFromGC(GR_Graphics * gc,
 {
 	UT_return_if_fail (gc);
 
+	DELETEP(m_pFormatFramePreview);
 	m_pFormatFramePreview = new AP_FormatFrame_preview(gc,this);
 	UT_return_if_fail (m_pFormatFramePreview);
 
@@ -1245,8 +1249,7 @@ void AP_FormatFrame_preview::draw(void)
 		else
 			m_gc->setLineProperties(1, GR_Graphics::JOIN_MITER, GR_Graphics::CAP_BUTT, GR_Graphics::LINE_SOLID);
 
-		m_gc->setColor(m_pFormatFrame->borderColorTop());
-
+		m_gc->setColor(m_pFormatFrame->borderColorTop());		
 		UT_sint32 iTopThickness = UT_convertToLogicalUnits(m_pFormatFrame->getBorderThicknessTop().utf8_str());
 		m_gc->setLineWidth(iTopThickness);
 
