@@ -187,6 +187,8 @@ menuitem(FMT_BOTTOMLINE)
 menuitem(FMT_SUPERSCRIPT)
 menuitem(FMT_SUBSCRIPT)
 menuitem(FMT_DIRECTION)
+menuitem(FMT_DIRECTION_DOCD_RTL)
+menuitem(FMT_DIRECTION_SD_RTL)
 menuitem(FMT_DIRECTION_DD_RTL)
 menuitem(FMT_DIRECTION_DO_LTR)
 menuitem(FMT_DIRECTION_DO_RTL)
