@@ -1,25 +1,28 @@
 /* AbiWord
- * Copyright (C) 1998-2001 AbiSource, Inc.
- * 
+ * Copyright (C) 1998-2000 AbiSource, Inc.
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
 
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+
+#include "ap_Features.h"
+
 #include "ut_types.h"
 #include "ut_assert.h"
 #include "ut_string.h"
@@ -31,6 +34,7 @@
 #include "ev_Toolbar_Actions.h"
 #include "ev_Toolbar_Labels.h"
 #include "xap_App.h"
+#include "xap_Prefs.h"
 #include "xap_Clipboard.h"
 #include "xap_Frame.h"
 #include "fv_View.h"
@@ -43,24 +47,27 @@
 #include "ut_Script.h"
 #include "spell_manager.h"
 #include "ap_EditMethods.h"
+#include "fp_TableContainer.h"
+
+#define ABIWORD_VIEW  	FV_View * pView = static_cast<FV_View *>(pAV_View)
+
+#if 0
 
 static bool s_ToolbarFunctions_check_inc_load(FV_View * pView)
 {
-  // todo: we probably need to make this function smarter. 
+  // todo: we probably need to make this function smarter.
   // see ap_EditMethods.cpp around line 1024
 
   //XAP_Frame * pFrame = XAP_App::getApp()->getLastFocussedFrame();
-
-  if(pView && ((pView->getPoint() == 0) || pView->isLayoutFilling()))
+	UT_DEBUGMSG(("pView = %x \n",pView));
+	if(pView && ((pView->getPoint() == 0) || pView->isLayoutFilling()))
     {
-      return true;
+		return true;
     }
 
-  return false ;
+	return false ;
 }
 
-#define ABIWORD_VIEW  	FV_View * pView = static_cast<FV_View *>(pAV_View)
-#if 0
 #define CHECK_INC_LOAD if(s_ToolbarFunctions_check_inc_load(pView)) return EV_TIS_Gray;
 #else
 #define CHECK_INC_LOAD
@@ -81,18 +88,104 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_ScriptsActive)
   return s;
 }
 
+Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_BookmarkOK)
+{
+	ABIWORD_VIEW;
+	CHECK_INC_LOAD;
+	EV_Toolbar_ItemState s = EV_TIS_ZERO;
+
+	if(pView->isTOCSelected())
+	{
+	    s = EV_TIS_Gray ;
+		return s;
+	}
+	PT_DocPosition posStart = pView->getPoint();
+	PT_DocPosition posEnd = pView->getSelectionAnchor();
+	fl_BlockLayout * pBL1 = pView->getBlockAtPosition(posStart);
+	fl_BlockLayout * pBL2 = pView->getBlockAtPosition(posEnd);
+	if((pBL1 == NULL) || (pBL2 == NULL)) // make sure we get valid blocks from selection beginning and end
+	{
+		s = EV_TIS_Gray;
+		return s;
+	}
+	if(pBL1 != pBL2) // don't allow Insert bookmark if selection spans multiple blocks
+	{
+	    s = EV_TIS_Gray ;
+		return s;
+	}
+
+	return s ;
+}
+
+Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_HyperlinkOK)
+{
+	ABIWORD_VIEW ;
+	CHECK_INC_LOAD;
+	UT_return_val_if_fail (pView, EV_TIS_Gray);
+
+	EV_Toolbar_ItemState s = EV_TIS_ZERO;
+	if ( pView->isSelectionEmpty())
+	  {
+		  if(pView->getHyperLinkRun(pView->getPoint()) == NULL)
+		  {
+			  s = EV_TIS_Gray ;
+			  return s;
+		  }
+		  return s;
+	  }
+	if(pView->isTOCSelected())
+	{
+	    s = EV_TIS_Gray ;
+		return s;
+	}
+	PT_DocPosition posStart = pView->getPoint();
+	PT_DocPosition posEnd = pView->getSelectionAnchor();
+	fl_BlockLayout * pBL1 = pView->getBlockAtPosition(posStart);
+	fl_BlockLayout * pBL2 = pView->getBlockAtPosition(posEnd);
+	if((pBL1 == NULL) || (pBL2 == NULL)) // make sure we get valid blocks from selection beginning and end
+	{
+		s = EV_TIS_Gray;
+		return s;
+	}
+	if(pBL1 != pBL2) // don't allow Insert Hyperlink if selection spans multiple blocks
+	{
+	    s = EV_TIS_Gray ;
+		return s;
+	}
+	if(pBL1->getLength() == 1)
+	{
+	    s = EV_TIS_Gray ;
+	    return s;
+	}
+	PT_DocPosition posTemp =0;
+	if(posStart > posEnd)
+	{
+	  posTemp = posStart;
+	  posStart = posEnd;
+	  posEnd = posStart;
+	}
+	if(posStart < pBL1->getPosition(true))
+	{
+	    s = EV_TIS_Gray ;
+	    return s;
+	}
+
+
+	return s ;
+}
+
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Spelling)
 {
-  ABIWORD_VIEW;
+	//ABIWORD_VIEW;
   CHECK_INC_LOAD;
 
   EV_Toolbar_ItemState s = EV_TIS_ZERO;
 
   XAP_Prefs *pPrefs = XAP_App::getApp()->getPrefs();
-  UT_ASSERT( pPrefs );
+  UT_return_val_if_fail ( pPrefs, EV_TIS_Gray );
 
   bool b = true ;
-  pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_AutoSpellCheck,&b) ;
+  pPrefs->getPrefsValueBool(static_cast<const XML_Char *>(AP_PREF_KEY_AutoSpellCheck),&b) ;
 
   // if there are no loaded dictionaries and we are spell checking
   // as we type
@@ -109,15 +202,19 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Changes)
 	ABIWORD_VIEW;
 	CHECK_INC_LOAD;
 
+	EV_Toolbar_ItemState s = EV_TIS_ZERO;
+	
+	if(!pView)
+	{
+		return s;
+	}
 	if (pszState)
 		*pszState = NULL;
-	
-	EV_Toolbar_ItemState s = EV_TIS_ZERO;
 
 	switch (id)
 	{
-	case AP_TOOLBAR_ID_FILE_SAVE:
-	  if (!pView->getDocument()->isDirty() || !pView->canDo(true))
+	case AP_TOOLBAR_ID_FILE_SAVE: // see bug 7580
+		if (!pView->getDocument()->isDirty() /*|| !pView->canDo(true)*/ || pView->getDocument()->getFilename() == NULL)
 	    s = EV_TIS_Gray;
 	  break;
 	case AP_TOOLBAR_ID_EDIT_UNDO:
@@ -131,7 +228,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Changes)
 		break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -158,7 +255,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Selection)
 		break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -167,7 +264,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Selection)
 
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Clipboard)
 {
-	ABIWORD_VIEW;	
+	ABIWORD_VIEW;
 
 	if (pszState)
 		*pszState = NULL;
@@ -177,19 +274,19 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Clipboard)
 	switch (id)
 	{
 	case AP_TOOLBAR_ID_EDIT_PASTE:
-		s = ( (pView->getApp()->canPasteFromClipboard()) ? EV_TIS_ZERO : EV_TIS_Gray );
+		s = ((XAP_App::getApp()->canPasteFromClipboard()) ? EV_TIS_ZERO : EV_TIS_Gray );
 		break;
 
 	case AP_TOOLBAR_ID_FMTPAINTER:
-	  if ( pView->getApp()->canPasteFromClipboard() &&
-	       !pView->isSelectionEmpty() )
+	  if (pView && XAP_App::getApp()->canPasteFromClipboard() &&
+	       !pView->isSelectionEmpty() && !pView->getDocument()->areStylesLocked())
 	    s = EV_TIS_ZERO;
 	  else
 	    s = EV_TIS_Gray;
 	  break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -200,7 +297,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Clipboard)
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_HdrFtr)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_TIS_Gray);
 
 	if (pszState)
 		*pszState = NULL;
@@ -211,16 +308,16 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_HdrFtr)
 	switch (id)
 	{
 	case AP_TOOLBAR_ID_EDIT_REMOVEHEADER:
-		if (!pView->isHeaderOnPage()) 
+		if (!pView->isHeaderOnPage())
 		  s = EV_TIS_Gray;
 		break;
 
 	case AP_TOOLBAR_ID_EDIT_REMOVEFOOTER:
-		if (!pView->isFooterOnPage()) 
+		if (!pView->isFooterOnPage())
 		  s = EV_TIS_Gray;
 		break;
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 	return s;
@@ -237,7 +334,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Style)
 	{
 	case AP_TOOLBAR_ID_FMT_STYLE:
 		{
-			const XML_Char * sz;
+			const XML_Char * sz = NULL;
 			if (!pView->getStyle(&sz))
 			{
 				static const char * sz2 ="None";
@@ -246,8 +343,8 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Style)
 			}
 
 			if (sz)
-			{	
-				static const char * sz2;
+			{
+				static const char * sz2 = "None";
 				sz2 = sz;
 				*pszState = sz2;
 				s = EV_TIS_UseString;
@@ -262,7 +359,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Style)
 		}
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 	return s;
@@ -275,18 +372,14 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Bullets)
 	CHECK_INC_LOAD;
 
 	EV_Toolbar_ItemState s = EV_TIS_ZERO;
-	if(pView->getDocument()->areStylesLocked()) 
-	{
-		return EV_TIS_Gray;
-	}
 
-	if(pView->isHdrFtrEdit()) 
+	if(pView->isHdrFtrEdit()  || pView->isInHdrFtr(pView->getPoint()))
 	{
 		return EV_TIS_Gray;
 	}
 
 	fl_BlockLayout * pBlock = pView->getCurrentBlock();
-	UT_ASSERT(pBlock);
+	UT_return_val_if_fail (pBlock, EV_TIS_Gray);
 	if(pBlock->isListItem() == false)
 		return s;
 	if(pBlock->getListType() == BULLETED_LIST)
@@ -301,17 +394,13 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Numbers)
 	CHECK_INC_LOAD;
 
 	EV_Toolbar_ItemState s = EV_TIS_ZERO;
-	if(pView->getDocument()->areStylesLocked()) 
-	{
-		return EV_TIS_Gray;
-	}
-	if(pView->isHdrFtrEdit()) 
+	if(pView->isHdrFtrEdit()  || pView->isInHdrFtr(pView->getPoint()))
 	{
 		return EV_TIS_Gray;
 	}
 
 	fl_BlockLayout * pBlock = pView->getCurrentBlock();
-        UT_ASSERT(pBlock);
+    UT_return_val_if_fail (pBlock, EV_TIS_Gray);
 	if(pBlock->isListItem() == false)
 	        return s;
 	if(pBlock->getListType() == NUMBERED_LIST)
@@ -324,17 +413,17 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Indents)
 	ABIWORD_VIEW;
 
 	EV_Toolbar_ItemState s = EV_TIS_ZERO;
-	
-	double margin_left = 0., margin_right = 0., allowed = 0., 
-		page_margin_left = 0., page_margin_right = 0.;
-	
- 	s_getPageMargins(pView, margin_left, margin_right,
- 					 page_margin_left, page_margin_right);
 
-#ifdef BIDI_ENABLED
-	FriBidiCharType iBlockDir = pView->getCurrentBlock()->getDominantDirection();
-#endif
-	
+	double margin_left = 0., margin_right = 0., allowed = 0.,
+		page_margin_left = 0., page_margin_right = 0.;
+
+	s_getPageMargins(pView, margin_left, margin_right,
+					 page_margin_left, page_margin_right);
+
+	UT_BidiCharType iBlockDir = UT_BIDI_LTR;
+	if(pView->getCurrentBlock())
+		iBlockDir = pView->getCurrentBlock()->getDominantDirection();
+
 	switch(id)
 	{
 		case AP_TOOLBAR_ID_INDENT:
@@ -343,17 +432,13 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Indents)
 				s = EV_TIS_Gray;
 			break;
 		case AP_TOOLBAR_ID_UNINDENT:
-#ifdef BIDI_ENABLED
-			allowed = iBlockDir == FRIBIDI_TYPE_LTR ? margin_left : margin_right;
-#else
-			allowed = margin_left;
-#endif
+			allowed = iBlockDir == UT_BIDI_LTR ? margin_left : margin_right;
 
 			if (allowed <= 0.)
 				s = EV_TIS_Gray;
 			break;
 	}
-	
+
 	return s;
 }
 
@@ -371,7 +456,8 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_CharFmt)
 
 	EV_Toolbar_ItemState s = EV_TIS_ZERO;
 
-	if(pView->getDocument()->areStylesLocked()) {
+	// todo: rtl/ltr/dom-dir are legal
+	if(pView->getDocument()->areStylesLocked() && !(AP_TOOLBAR_ID_FMT_SUPERSCRIPT == id || AP_TOOLBAR_ID_FMT_SUBSCRIPT == id)) {
 	  return EV_TIS_Gray;
 	}
 
@@ -442,7 +528,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_CharFmt)
 		val  = "subscript";
 		bMultiple = true;
 		break;
-#ifdef BIDI_ENABLED
+
 	case AP_TOOLBAR_ID_FMT_DIR_OVERRIDE_LTR:
 		prop = "dir-override";
 		val  = "ltr";
@@ -452,9 +538,9 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_CharFmt)
 		prop = "dir-override";
 		val  = "rtl";
 		break;
-#endif	
+
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -474,21 +560,21 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_CharFmt)
 		if (sz)
 		{
 			if (bSize)
-			{	
+			{
 				static char buf[7];
-				sprintf(buf, "%s", std_size_string((float)UT_convertToPoints(sz)));
+				sprintf(buf, "%s", std_size_string(static_cast<float>(UT_convertToPoints(sz))));
 				*pszState = buf;
 				s = EV_TIS_UseString;
 			}
 			else if (bString)
-			{	
+			{
 				static const char * sz2;
 				sz2 = sz;
 				*pszState = sz2;
 				s = EV_TIS_UseString;
 			}
 			else if (bMultiple)
-			{	
+			{
 				// some properties have multiple values
 				if (strstr(sz, val))
 					s = EV_TIS_Toggled;
@@ -499,7 +585,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_CharFmt)
 					s = EV_TIS_Toggled;
 			}
 		}
-		
+
 		free(props_in);
 	}
 
@@ -519,10 +605,10 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_SectionFmt)
 
 	EV_Toolbar_ItemState s = EV_TIS_ZERO;
 
-	if(pView->isHdrFtrEdit())
+	if(pView->isHdrFtrEdit()  || pView->isInHdrFtr(pView->getPoint()))
 	  {
 	    switch(id)
-	      {	
+	      {
 		// only 1 column allowed
 	      case AP_TOOLBAR_ID_1COLUMN:
 		return EV_TIS_Toggled;
@@ -530,10 +616,14 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_SectionFmt)
 		// not allowed
 	      case AP_TOOLBAR_ID_2COLUMN:
 	      case AP_TOOLBAR_ID_3COLUMN:
+		  case AP_TOOLBAR_ID_INSERT_TABLE:
 	        return EV_TIS_Gray;
 	      }
 	  }
-
+	if(id == AP_TOOLBAR_ID_INSERT_TABLE)
+	{
+		return EV_TIS_ZERO;
+	}
 	switch (id)
 	{
 	case AP_TOOLBAR_ID_1COLUMN:
@@ -549,7 +639,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_SectionFmt)
 		val = "3";
 		break;
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 	s = EV_TIS_ZERO;
@@ -580,7 +670,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_SectionFmt)
 				s = EV_TIS_ZERO;
 			}
 		}
-		
+
 		free(props_in);
 	}
 
@@ -605,7 +695,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_BlockFmt)
         if(pView->getDocument()->areStylesLocked()) {
 	    return EV_TIS_Gray;
         }
-	
+
 	switch (id)
 	{
 	case AP_TOOLBAR_ID_ALIGN_LEFT:
@@ -650,20 +740,19 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_BlockFmt)
 		prop = "line-height";
 		val = "2.0";
 		break;
-#ifdef BIDI_ENABLED
+
 	case AP_TOOLBAR_ID_FMT_DOM_DIRECTION:
 		prop = "dom-dir";
 		val = "rtl";
 		xxx_UT_DEBUGMSG(("ap_ToolbarGetState_BlockFmt: dom-dir\n"));
 		break;
-#endif
 
 	default:
 		UT_DEBUGMSG(("id=%d", id));
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
-	
+
 	if (prop && val)
 	{
 		// get current block properties from pView
@@ -676,12 +765,12 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_BlockFmt)
 		// NB: maybe *no* properties are consistent across the selection
 		if (props_in && props_in[0])
 			sz = UT_getAttribute(prop, props_in);
-	
+
 		if (sz)
 		{
 			if (bPoints)
 			{
-				if (((int) UT_convertToPoints(sz)) == ((int) UT_convertToPoints(val)))
+				if ((static_cast<int>(UT_convertToPoints(sz))) == (static_cast<int>(UT_convertToPoints(val))))
 					s = EV_TIS_Toggled;
 			}
 			else
@@ -690,7 +779,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_BlockFmt)
 					s = EV_TIS_Toggled;
 			}
 		}
-		
+
 		free(props_in);
 	}
 
@@ -700,32 +789,32 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_BlockFmt)
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Zoom)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_TIS_Gray);
 
 	XAP_Frame * pFrame = static_cast<XAP_Frame *> ( pView->getParentData());
 	const XAP_StringSet * pSS = XAP_App::getApp()->getStringSet();
-	
+
 	EV_Toolbar_ItemState s = EV_TIS_UseString;
 
-	static char buf[10];
 	UT_uint32 iZoom = 0 ;
-
+	static UT_UTF8String str;
+	
 	switch(pFrame->getZoomType())
 	{
 	// special cases
 	case XAP_Frame::z_PAGEWIDTH:
-		sprintf(buf, "%s", pSS->getValue(XAP_STRING_ID_TB_Zoom_PageWidth));
+		pSS->getValueUTF8(XAP_STRING_ID_TB_Zoom_PageWidth, str);
 		break;
 	case XAP_Frame::z_WHOLEPAGE:
-		sprintf(buf, "%s", pSS->getValue(XAP_STRING_ID_TB_Zoom_WholePage));
+		pSS->getValueUTF8(XAP_STRING_ID_TB_Zoom_WholePage, str);
 		break;
 	default:
-	  iZoom = pView->getGraphics()->getZoomPercentage();	
-	  sprintf(buf, "%d%%", iZoom);
+	  iZoom = pView->getGraphics()->getZoomPercentage();
+	  UT_UTF8String_sprintf(str, "%d%%", iZoom);
 	  break;
 	}
 
-	*pszState = buf;
+	*pszState = str.utf8_str();
 
 	return s;
 }
@@ -733,13 +822,13 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Zoom)
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_View)
 {
 	ABIWORD_VIEW;
-	UT_ASSERT(pView);
+	UT_return_val_if_fail (pView, EV_TIS_Gray);
 
 	XAP_Frame * pFrame = static_cast<XAP_Frame *> (pAV_View->getParentData());
-	UT_ASSERT(pFrame);
+	UT_return_val_if_fail (pFrame, EV_TIS_Gray);
 
 	AP_FrameData *pFrameData = static_cast<AP_FrameData *> (pFrame->getFrameData());
-	UT_ASSERT(pFrameData);
+	UT_return_val_if_fail (pFrameData, EV_TIS_Gray);
 
 	EV_Toolbar_ItemState s = EV_TIS_ZERO;
 
@@ -751,7 +840,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_View)
         break;
 
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		break;
 	}
 
@@ -761,7 +850,7 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_View)
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_StylesLocked)
 {
         ABIWORD_VIEW;
-        UT_ASSERT(pView);
+        UT_return_val_if_fail (pView, EV_TIS_Gray);
 
         if(pView->getDocument()->areStylesLocked()) {
             return EV_TIS_Gray;
@@ -770,3 +859,58 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_StylesLocked)
         return EV_TIS_ZERO;
 }
 
+Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Table)
+{
+  ABIWORD_VIEW;
+  UT_return_val_if_fail (pView, EV_TIS_Gray);
+  
+  if(pView->isInTable() )
+  {
+	  return EV_TIS_ZERO;
+  }
+  return EV_TIS_Gray;
+}
+
+
+Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_TableOK)
+{
+  ABIWORD_VIEW;
+  UT_return_val_if_fail (pView, EV_TIS_Gray);
+  
+  if(pView->isInTable() && (pView->isHdrFtrEdit() || pView->isInHdrFtr(pView->getPoint())))
+  {
+	  return EV_TIS_Gray;
+  }
+  else if(pView->isInFootnote())
+  {
+	  return EV_TIS_Gray;
+  }
+  else if(pView->isInEndnote())
+  {
+	  return EV_TIS_Gray;
+  } 
+  else if(pView->getHyperLinkRun(pView->getPoint()) != NULL)
+  {
+	  return EV_TIS_Gray;
+  }
+
+  return EV_TIS_ZERO;
+}
+
+Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_TableMerged)
+{
+  ABIWORD_VIEW;
+  UT_return_val_if_fail (pView, EV_TIS_Gray);
+  
+  if(pView->isInTable())
+  {
+	  return EV_TIS_ZERO;
+  }
+  return EV_TIS_Gray;
+}
+
+// HACK TO ALWAYS DISABLE A TOOLBAR ITEM... DELETE ME
+Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_AlwaysDisabled)
+{
+  return EV_TIS_Gray;
+}
