@@ -23,6 +23,7 @@
 
 
 void IE_ImpExp_RegisterXP ();
-
+void IE_ImpExp_RegisterPlatform ();	// must be implemented by the platform
+void IE_ImpExp_UnRegisterXP ();
 
 #endif
