@@ -565,7 +565,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 		UT_UTF8String dest;
 		IEFileType ieft;
 
-		if (pToks->getItemCount () < 1)
+		if (pToks->getItemCount () < 2)
 			return -1;
 
 		// input filename is pToks[1]
@@ -679,15 +679,15 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 		printf ("load <filename>             - Load <filename> replacing the current document.\n");
 		printf ("printfile <filename1> <filename2> <...> - Print the current document into the\n");
 		printf ("                              filenames listed.\n");
-		printf ("replaceall <find> <target>  - Replace every occurance of <find> with <target>\n");
+		printf ("replaceall <find> <target>  - Replace every occurrence of <find> with <target>\n");
 		printf ("                              in the current document.\n");
-		printf ("replacenext <find> <target> - Replace the next occurance of <find> with <target>\n");
+		printf ("replacenext <find> <target> - Replace the next occurrence of <find> with <target>\n");
 		printf ("                              in the current document.\n");
 		printf ("inserttext <target>         - Insert <target> at the current point in the\n");
 		printf ("                              document.\n");
 		printf ("delete <args>               - Delete <args> characters at the current point\n");
 		printf ("                              in the document.\n");
-		printf ("replacenext <find> <target> - Replace the next occurance of <find> with <target>\n");
+		printf ("replacenext <find> <target> - Replace the next occurrence of <find> with <target>\n");
 		printf ("                              in the current document.\n");
 		printf ("movept <arg>                - Move the current point to another location in\n");
 		printf ("                              the current document.\n");
@@ -695,9 +695,9 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 		printf ("                              BOL,EOL,BOW,+num,-num,num\n");
 		printf ("selectstart                 - Start a selection at the current point\n");
 		printf ("selectclear                 - Clear the current selection.\n");
-		printf ("findnext <target>           - Find the next occurance of target and select it.\n");
+		printf ("findnext <target>           - Find the next occurrence of target and select it.\n");
 		printf ("save <filename>             - Save the current document.\n");
-		printf ("                              If filename is omitted the file is saved to it's\n");
+		printf ("                              If filename is omitted the file is saved to its\n");
 		printf ("                              original name.\n");
 		printf ("                              Otherwise the extension of the filename is used\n");
 		printf ("                              to determine the format of the file.\n");
@@ -705,7 +705,7 @@ AbiCommand::parseTokens (UT_Vector * pToks)
 		printf ("                              file named <dest>.\n");
 		printf ("convert <src> <dest> <type> - Convert the file given in <src> to the file named\n");
 		printf ("                              <dest>. The type of conversion is given by the\n");
-		printf ("                              third paramter (abw,html,odt, etc.).\n");
+		printf ("                              third parameter (abw,html,odt, etc.).\n");
 		printf ("writepid <file>             - Write the PID of this process to the file <file>\n");
 		printf ("server <error file>         - This is being run as remote process. Write an\n");
 		printf ("                              error file on error.\n");
@@ -923,7 +923,7 @@ AbiCommand::insertText (UT_Vector * pToks)
 bool
 AbiCommand::deleteText (UT_Vector * pToks)
 {
-	if (m_pCurView != NULL)
+	if ((m_pCurView != NULL) && (pToks->getItemCount() > 1))
 	{
 		UT_UTF8String *pCom1 =
 			const_cast < UT_UTF8String * >(static_cast < const UT_UTF8String * >(pToks->getNthItem (1)));
@@ -977,6 +977,8 @@ AbiCommand::replaceNext (UT_Vector * pToks)
 bool
 AbiCommand::printFiles (UT_Vector * pToks)
 {
+	UT_return_val_if_fail(m_pCurDoc, false);
+
 	for (UT_uint32 i = 1; i < pToks->getItemCount (); i++)
 	{
 		UT_UTF8String *pFile =
@@ -987,6 +989,7 @@ AbiCommand::printFiles (UT_Vector * pToks)
 				static_cast < XAP_UnixApp * >(m_pApp)->getFontManager (), true, m_pApp);
 
 		PS_Graphics *pG = (PS_Graphics *) m_pApp->newGraphics (ai);
+		UT_return_val_if_fail(pG, false);
 
 		// create a new layout and view object for the doc
 		FL_DocLayout *pDocLayout = new FL_DocLayout (m_pCurDoc, pG);
