@@ -36,7 +36,8 @@
 #include <px_CR_Span.h>
 #include <px_CR_Strux.h>
 #include <px_CR_Object.h>
-
+#include <ut_mbtowc.h>
+#include <ut_locale.h>
 
 /**
  * Constructor
@@ -132,22 +133,7 @@ bool ODe_AbiDocListener::populate(PL_StruxFmtHandle sfh,
                 {
                     _closeSpan();
                     _closeField();
-                    //_openTag("math","/",false,api,pcr->getXID());
-                    /*const XML_Char* image_name = getObjectKey(api, static_cast<const XML_Char*>("dataid"));
-                    if (image_name)
-                    {
-                        UT_DEBUGMSG(("resource name #%s# recorded \n",image_name));
-                        UT_UTF8String * sPNGname = new UT_UTF8String("snapshot-png-");
-                        m_vecSnapNames.addItem(sPNGname);
-                        *sPNGname += image_name;
-                        UT_DEBUGMSG(("resource name #%s# recorded \n",sPNGname->utf8_str()));
-                    }
-                    const XML_Char* latex_name = getObjectKey(api, static_cast<const XML_Char*>("latexid"));
-                    if(latex_name)
-                    {
-                        UT_DEBUGMSG(("resource name #%s# recorded \n",latex_name));
-                        m_pUsedImages.insert(latex_name);
-                    }*/
+                    _insertMath(api);
                     return true;
                 }
                 
@@ -1027,12 +1013,91 @@ void ODe_AbiDocListener::_insertInlinedImage(PT_AttrPropIndex api) {
 }
 
 
+/**
+ * 
+ */
+void ODe_AbiDocListener::_insertMath(PT_AttrPropIndex api) {
+
+    const XML_Char* szMath = NULL;
+    szMath = _getObjectKey(api, static_cast<const XML_Char*>("dataid"));
+
+    UT_return_if_fail(szMath);
+
+    const UT_ByteBuf * pByteBuf = NULL;
+    bool bOK = m_pDocument->getDataItemDataByName(szMath, const_cast<const UT_ByteBuf **>(&pByteBuf), NULL, NULL);
+
+    UT_return_if_fail(bOK);
+
+    UT_UCS4_mbtowc myWC;
+    UT_UTF8String sMathML;
+    sMathML.appendBuf(*pByteBuf, myWC);
+
+    UT_return_if_fail(!sMathML.empty());
+
+    UT_UCS4String buf = sMathML.utf8_str();
+    UT_UTF8String output = "";
+
+    _openSpan(api);
+
+    const PP_AttrProp * pAP = NULL;
+    bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+    UT_LocaleTransactor t(LC_NUMERIC, "C");
+    UT_UTF8String dimension;
+    double dInch;
+
+    UT_return_if_fail(bHaveProp && pAP);
+
+    if(pAP->getProperty("width", szMath)) {
+        dInch = static_cast<double>(atoi(szMath))/UT_LAYOUT_RESOLUTION;
+        UT_UTF8String_sprintf(dimension,"%fin",dInch);
+        output += "<draw:frame svg:width=\"";
+        output += dimension;
+        output += "\" svg:height=\"";
+    } else {
+        UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+        _closeSpan();
+        return;
+    }
+
+    if(pAP->getProperty("height", szMath)) {
+        dInch = static_cast<double>(atoi(szMath))/UT_LAYOUT_RESOLUTION;
+        dimension.clear();
+        UT_UTF8String_sprintf(dimension,"%fin",dInch);
+        output += dimension;
+        output += "\"><draw:object>";
+    } else {
+        UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+        _closeSpan();
+        return;
+    }
+
+    for (int i = 0; i < buf.length(); i++) {
+        if (buf[i] == '<') {
+            if (((i + 1) < buf.length()) && (buf[i+1] == '/')) {
+                output += "</math:";
+                i++; // skip the '/'
+            } else if ((i + 1) < buf.length()) {
+                output += "<math:";
+            } else {
+                UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+            }
+        } else {
+            output += buf[i];
+        }
+    }
+    output += "</draw:object></draw:frame>";
+    m_pCurrentImpl->insertText(output);
+    _closeSpan();
+}
+
+
 /**
  * 
  */
 void ODe_AbiDocListener::_outputData(const UT_UCSChar* pData, UT_uint32 length) {
     UT_UTF8String sBuf;
     const UT_UCSChar* p;
+    UT_uint32 nSpaces = 0;
 
     UT_ASSERT(sizeof(UT_Byte) == sizeof(char));
 
@@ -1041,21 +1106,46 @@ void ODe_AbiDocListener::_outputData(const UT_UCSChar* pData, UT_uint32 length)
         switch (*p)
         {
         case '<':
+            if(nSpaces > 1)
+                _appendSpaces(&sBuf, nSpaces);
+            nSpaces = 0;
+
             sBuf += "&lt;";
             p++;
             break;
 
         case '>':
+            if(nSpaces > 1)
+                _appendSpaces(&sBuf, nSpaces);
+            nSpaces = 0;
+
             sBuf += "&gt;";
             p++;
             break;
 
         case '&':
+            if(nSpaces > 1)
+                _appendSpaces(&sBuf, nSpaces);
+            nSpaces = 0;
+
             sBuf += "&amp;";
             p++;
             break;
 
+        case ' ':
+            nSpaces++;
+
+            if(nSpaces == 1)
+                sBuf.appendUCS4 (p, 1);
+
+            p++;
+            break;
+
         case UCS_LF:               // LF -- representing a Forced-Line-Break
+            if(nSpaces > 1)
+                _appendSpaces(&sBuf, nSpaces);
+            nSpaces = 0;
+
             m_pCurrentImpl->insertText(sBuf);
             m_pCurrentImpl->insertLineBreak();
             sBuf.clear();
@@ -1063,6 +1153,10 @@ void ODe_AbiDocListener::_outputData(const UT_UCSChar* pData, UT_uint32 length)
             break;
 
         case UCS_VTAB:             // VTAB -- representing a Forced-Column-Break
+            if(nSpaces > 1)
+                _appendSpaces(&sBuf, nSpaces);
+            nSpaces = 0;
+
             m_pCurrentImpl->insertText(sBuf);
             m_pCurrentImpl->insertColumnBreak();
             sBuf.clear();
@@ -1070,11 +1164,19 @@ void ODe_AbiDocListener::_outputData(const UT_UCSChar* pData, UT_uint32 length)
             break;
 
         case UCS_TAB:
+            if(nSpaces > 1)
+                _appendSpaces(&sBuf, nSpaces);
+            nSpaces = 0;
+
             m_pCurrentImpl->insertTabChar();
             p++;
             break;
 
         case UCS_FF:               // FF -- representing a Forced-Page-Break
+            if(nSpaces > 1)
+                _appendSpaces(&sBuf, nSpaces);
+            nSpaces = 0;
+
             m_pCurrentImpl->insertText(sBuf);
             m_pCurrentImpl->insertPageBreak();
             sBuf.clear();
@@ -1083,21 +1185,44 @@ void ODe_AbiDocListener::_outputData(const UT_UCSChar* pData, UT_uint32 length)
 
         default:
             if (*p < 0x20)         // Silently eat these characters.
+            {
+                if(nSpaces > 1)
+                    _appendSpaces(&sBuf, nSpaces);
+                nSpaces = 0;
                 p++;
+            }
             else
-                {
-                    sBuf.appendUCS4 (p, 1);
-                    p++;
-                }
+            {
+                if(nSpaces > 1)
+                    _appendSpaces(&sBuf, nSpaces);
+                nSpaces = 0;
+
+                sBuf.appendUCS4 (p, 1);
+                p++;
+            }
         }
     }
 
     if (!sBuf.empty()) {
+        if(nSpaces > 1)
+            _appendSpaces(&sBuf, nSpaces);
+        nSpaces = 0;
         m_pCurrentImpl->insertText(sBuf);
     }
 }
 
 
+/**
+ * 
+ */
+void ODe_AbiDocListener::_appendSpaces(UT_UTF8String* sBuf, UT_uint32 count) {
+    if (count == 2)
+        *sBuf += "<text:s/>";
+    else if(count > 2)
+        *sBuf += UT_UTF8String_sprintf("<text:s text:c=\"%u\"/>", count - 1);
+}
+
+
 /**
  * 
  */
