@@ -31,6 +31,7 @@
 #include <fp_types.h>
 #include <ut_string_class.h>
 #include <ut_units.h>
+#include <ut_locale.h>
 
 //External includes
 #include <stdlib.h>
@@ -205,6 +206,7 @@ void ODi_ListLevelStyle::getAbiProperties(UT_UTF8String& rProps,
 
     double styMarginLeft_cm;
     XML_Char buffer[100];
+    UT_LocaleTransactor lt(LC_NUMERIC, "C");
     
     spaceBefore_cm = UT_convertToDimension(m_spaceBefore.utf8_str(),
                                            DIM_CM);
