@@ -22,7 +22,7 @@
 #include <gsf/gsf-infile.h>
 #include <gsf/gsf-infile-zip.h>
 
-#include <locale.h>
+#include "ut_locale.h"
 #include <math.h>
 
 #include "ie_impexp_OpenWriter.h"
@@ -385,9 +385,8 @@ public:
 	int spacing;
 	
 	sscanf (val, "%d%%", &spacing);	
-	const char * old_loc = setlocale(LC_NUMERIC, "C");
+	UT_LocaleTransactor lt(LC_NUMERIC, "C");
 	m_lineHeight = UT_String_sprintf ("line-height: %f;", (double)spacing/100.);
-	setlocale (LC_NUMERIC, old_loc);
       }
       else
 	m_lineHeight = UT_String_sprintf ("line-height: %s;", val);
@@ -547,8 +546,7 @@ IE_Imp_OpenWriter_Sniffer::~IE_Imp_OpenWriter_Sniffer ()
  */
 UT_Confidence_t IE_Imp_OpenWriter_Sniffer::recognizeSuffix (const char * szSuffix)
 {
-  if (!UT_stricmp(szSuffix, ".sxw") || 
-	  !UT_stricmp(szSuffix, ".odt"))
+  if (!UT_stricmp(szSuffix, ".sxw"))
     return UT_CONFIDENCE_PERFECT;
   return UT_CONFIDENCE_ZILCH;
 }
@@ -589,8 +587,8 @@ bool IE_Imp_OpenWriter_Sniffer::getDlgLabels (const char ** szDesc,
 					      const char ** szSuffixList,
 					      IEFileType * ft)
 {
-  *szDesc = "OpenOffice Writer (.sxw, .odt)";
-  *szSuffixList = "*.sxw; *.odt";
+  *szDesc = "OpenOffice Writer (.sxw)";
+  *szSuffixList = "*.sxw";
   *ft = getFileType();
   return true;
 }
@@ -659,14 +657,14 @@ UT_Error IE_Imp_OpenWriter::importFile(const char * szFilename)
   
   UT_Error err = UT_OK;
 
-  if ( UT_OK != (err = _handleMimetype ()))
-    return err;
-  if ( UT_OK != (err = _handleMetaStream ()))
-    return err;
-  if ( UT_OK != (err = _handleStylesStream ()))
-    return err;
+  if ( UT_OK != _handleMimetype ())
+    ; // TODO: give warning about missing mimetype stream
+  if ( UT_OK != _handleMetaStream ())
+    ; // TODO: give warning about missing meta.xml stream
+  if ( UT_OK != _handleStylesStream ())
+    ; // TODO: give warning about missing styles.xml stream
   if ( UT_OK != (err = _handleContentStream ()))
-    return err;
+    return err; // abort because content.xml stream is compulsory
     
   return UT_OK;
 }
@@ -1135,7 +1133,11 @@ private:
 UT_Error IE_Imp_OpenWriter::_handleStylesStream ()
 {
   m_pSSListener = new OpenWriter_StylesStream_Listener(this, m_bOpenDocument);
-  return handleStream (m_oo, "styles.xml", *m_pSSListener);
+  // Collect styles from both styles.xml and content.xml streams
+  UT_Error result1 = handleStream (m_oo, "styles.xml", *m_pSSListener);
+  UT_Error result2 = handleStream (m_oo, "content.xml", *m_pSSListener);
+  // return the biggest negative value or 0 if both are UT_OK
+  return (result1 < result2 ? result1 : result2);
 }
 
 /*****************************************************************************/
