@@ -364,6 +364,12 @@ void s_DocBook_Listener :: _closeChapter (void)
 	if(!m_bInChapter)
 		return;
 
+	if(m_bInTable) // bad .doc import can lead to <section>s being closed in <cell>s; try to export valid XML regardless
+	{
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
+		_closeTable();
+	}
+
 	_closeSection(0); //close any open sections
 	_tagClose(TT_CHAPTER,"chapter");
 	m_bInChapter = false;
@@ -790,7 +796,7 @@ void s_DocBook_Listener :: _openSectionTitle(void)
 
 void s_DocBook_Listener :: _openSpan(PT_AttrPropIndex api)
 {
-	if ((!m_bInParagraph && !m_bInTitle) || (m_iNestedTable == 1))
+	if ((!m_bInParagraph && !m_bInTitle))
 		return;
 
 	if(m_bInSpan)
@@ -992,7 +998,9 @@ s_DocBook_Listener::s_DocBook_Listener(PD_Document * pDocument,
 	m_bInFrame = false;
 	m_bInHdrFtr = false;
 	m_bInNote = false;
+	m_iBlockType = 0;
 	m_iNestedTable = -1;
+	m_iTableDepth = 0;
 	m_iListDepth = 0;
 	m_iPreviousListDepth = 0;
 	m_iSectionDepth = 0;
@@ -1004,7 +1012,7 @@ s_DocBook_Listener::s_DocBook_Listener(PD_Document * pDocument,
 
 s_DocBook_Listener::~s_DocBook_Listener()
 {
-
+	UT_VECTOR_FREEALL(char *, m_utvDataIDs);
 }
 
 bool s_DocBook_Listener::populate(PL_StruxFmtHandle /*sfh*/,
@@ -1300,12 +1308,12 @@ void s_DocBook_Listener::_handleMetaData(void)
 	}
 	if (m_pDocument->getMetaDataProp (PD_META_KEY_CREATOR, metaProp) && metaProp.size())
 	{
-/*
 		escaped = metaProp.escapeXML();
 		_tagOpen(TT_AUTHOR,"author",false,true,false);
+		_tagOpen(TT_OTHERNAME,"othername role=\"full\"",false,false,false);
 		m_pie->write(metaProp.utf8_str());
+		_tagClose(TT_OTHERNAME,"othername",false,false,false);
 		_tagClose(TT_AUTHOR,"author",true,false,false);
-*/;
 	}
 	if (m_pDocument->getMetaDataProp (PD_META_KEY_SUBJECT, metaProp) && metaProp.size())
 	{
@@ -1397,7 +1405,6 @@ void s_DocBook_Listener::_handleMetaData(void)
 			{
 				if(buf.empty())  //only blank space encountered
 				{
-					buf.clear();
 					continue;
 				}
 				if(_tagTop() == TT_BOOKINFO)
@@ -1513,7 +1520,7 @@ void s_DocBook_Listener::_handleImage(PT_AttrPropIndex api)
 		_tagOpen(TT_FIGURE,"figure",false,false,false);
 		_tagOpen(TT_TITLE,"title",false,false,false);
 
-		if(pAP->getAttribute("title", szValue))  //use the image's title instead of it file name, if it exists
+		if(pAP->getAttribute("title", szValue))  //use the image's title instead of its file name, if it exists
 		{
 			escaped = szValue;
 			escaped.escapeXML();
@@ -1590,7 +1597,7 @@ void s_DocBook_Listener::_handlePositionedImage(PT_AttrPropIndex api)
 		_tagOpen(TT_FIGURE,"figure",false,true,false);
 		_tagOpen(TT_TITLE,"title",false,false,false);
 
-		if(pAP->getAttribute("title", szValue))  //use the image's title instead of it file name, if it exists
+		if(pAP->getAttribute("title", szValue))  //use the image's title instead of its file name, if it exists
 		{
 			escaped = szValue;
 			escaped.escapeXML();
@@ -2123,19 +2130,29 @@ bool s_DocBook_Listener::populateStrux(PL_StruxDocHandle sdh,
 
 		case PTX_SectionTable:
 		{
-			_closeParagraph();
-			mTableHelper.OpenTable(sdh,pcr->getIndexAP()) ;
-			_openTable(pcr->getIndexAP());
+			m_iTableDepth++;
+			if(m_iTableDepth <= 2)
+			{
+				_closeParagraph();
+				mTableHelper.OpenTable(sdh,pcr->getIndexAP()) ;
+				_openTable(pcr->getIndexAP());
+			}
 			return true;
 		}
 
 		case PTX_SectionCell:
 		{
+			if(m_iTableDepth > 2)
+				return true;
+
+			if((m_iNestedTable == 2) && (m_iTableDepth == 1)) //the last cell had a nested table; reset the value
+				m_iNestedTable = -1;
+
 			PL_StruxDocHandle nextTable = NULL, nextCell = NULL;
 			bool bNextTable = m_pDocument->getNextStruxOfType(sdh, PTX_SectionTable, &nextTable);
 			bool bEndCell = m_pDocument->getNextStruxOfType(sdh, PTX_EndCell, &nextCell);
 
-			if(bNextTable && bEndCell)
+			if(bNextTable && bEndCell && (m_iNestedTable == -1))
 			{
 				if(m_pDocument->getStruxPosition(nextTable) < m_pDocument->getStruxPosition(nextCell)) //nested table
 				{
@@ -2146,25 +2163,39 @@ bool s_DocBook_Listener::populateStrux(PL_StruxDocHandle sdh,
 				}
 			}
 
-			// regular cell
-			_closeParagraph();
-			mTableHelper.OpenCell(pcr->getIndexAP());
-			_openCell(pcr->getIndexAP());
+			if((m_iNestedTable == -1) || (m_iNestedTable == 1))
+			{
+				// regular cell
+				_closeParagraph();
+				mTableHelper.OpenCell(pcr->getIndexAP());
+				_openCell(pcr->getIndexAP());
+			}
 			return true;
 		}
 
 		case PTX_EndTable:
 		{	    
+			// m_iTableDepth will be 1 if a nested table was closed
+			// or 0 if a regular table was closed
+			m_iTableDepth--;
+			if(m_iTableDepth > 1)
+				return true;
+
 			_closeParagraph();
 			_closeRow();
 			_closeTable();
 			mTableHelper.CloseTable();
-			m_iNestedTable = -1;
+
+			if(m_iNestedTable != 2) //don't allow any content after an </entrytbl>
+				m_iNestedTable = -1;
 			return true;
 		}
 
 		case PTX_EndCell:
 		{
+			if(m_iTableDepth > 2)
+				return true;
+
 			_closeParagraph();
 			_closeCell();
 			mTableHelper.CloseCell();
