@@ -17,7 +17,6 @@
  * 02111-1307, USA.
  */
 
-#include <locale.h>
 #include <ctype.h>
 
 #include "ut_string.h"
@@ -43,6 +42,7 @@
 #include "ut_string_class.h"
 
 #include "ie_exp_KWord_1.h"
+#include "ut_locale.h"
 
 /*****************************************************************/
 /*****************************************************************/
@@ -498,12 +498,11 @@ void s_KWord_1_Listener::_handlePageSize(PT_AttrPropIndex api)
   //
   // Code to write out the PageSize Definitions to disk
   // 
-	char *old_locale;
 
 	//const PP_AttrProp * pAP = NULL;
 	//bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
-	old_locale = setlocale (LC_NUMERIC, "C");
+	UT_LocaleTransactor lt (LC_NUMERIC, "C");
 
 	m_pie->write("<PAPER");
 
@@ -550,8 +549,6 @@ void s_KWord_1_Listener::_handlePageSize(PT_AttrPropIndex api)
 
 	m_pie->write("</PAPER>\n");
 
-	setlocale (LC_NUMERIC, old_locale);
-
 	m_bFirstWrite = false;
 	return;
 }
