@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiWord
  * Copyright (C) 1998 AbiSource, Inc.
  * Copyright (C) 2004 Marc Maurer (j.m.maurer@student.utwente.nl)
@@ -26,7 +28,6 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <locale.h>
 
 #include "ut_debugmsg.h"
 #include "ut_string.h"
@@ -45,6 +46,7 @@
 #include "xap_EncodingManager.h"
 #include "fd_Field.h"
 #include "ie_Table.h"
+#include "ut_locale.h"
 
 #include "ut_string_class.h"
 
@@ -610,12 +612,11 @@ void s_LaTeX_Listener::_convertColor(UT_String& szDest, const char* pszColor)
 		strncpy (colors[i],&pszColor[2*i],2);
 		colors[i][2]=0;
 	}
-	setlocale (LC_NUMERIC, "C");
+	UT_LocaleTransactor lt (LC_NUMERIC, "C");
 	UT_String_sprintf (szDest, "%.3f,%.3f,%.3f",
 			   strtol (&colors[0][0],NULL,16)/255.,
 			   strtol (&colors[1][0],NULL,16)/255.,
 			   strtol (&colors[2][0],NULL,16)/255.);
-	setlocale (LC_NUMERIC, "");
 }
 
 void s_LaTeX_Listener::_convertFontSize(UT_String& szDest, const char* pszFontSize)
@@ -695,7 +696,7 @@ void s_LaTeX_Listener::_openSpan(PT_AttrPropIndex api)
 			&& !UT_strcmp(szValue, "italic")
 			)
 		{
-			m_pie->write("\\textit{");
+			m_pie->write("\\emph{");
 		}
 		
 		if (pAP->getProperty("text-decoration", szValue))
@@ -1103,7 +1104,7 @@ void s_LaTeX_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 			break;
 
 		case 34:
-			(m_bBetweenQuotes = !m_bBetweenQuotes)? sBuf += "``" : sBuf += "''";
+			(m_bBetweenQuotes = !m_bBetweenQuotes)? sBuf += "{``}" : sBuf += "''";
 			pData++;
 			break;
 
@@ -1199,19 +1200,19 @@ static bool _convertLettersToSymbols(char c, char *& subst)
 	}
 }
 
-s_LaTeX_Listener::s_LaTeX_Listener(PD_Document * pDocument,
-										 IE_Exp_LaTeX * pie)
+s_LaTeX_Listener::s_LaTeX_Listener(PD_Document * pDocument, IE_Exp_LaTeX * pie)
+  : m_pDocument(pDocument),
+	m_pie(pie),
+	m_bInSection(false),
+	m_bInBlock(false),
+	m_bInCell(false),
+	m_bInSpan(false),
+	m_bInFootnote(false),
+	m_bFirstSection(true),
+	m_bInSymbol(0),
+	m_bInCourier(0),
+	m_bInSansSerif(0)
 {
-	m_pDocument = pDocument;
-	m_pie = pie;
-	m_bInSection = false;
-	m_bInBlock = false;
-	m_bInSpan = false;
-	m_bFirstSection = true;
-	m_bInSymbol = 0;
-	m_bInCourier = 0;
-	m_bInSansSerif = 0;
-
 	m_pie->write("%% ================================================================================\n");
 	m_pie->write("%% This LaTeX file was created by AbiWord.                                         \n");
 	m_pie->write("%% AbiWord is a free, Open Source word processor.                                  \n");
@@ -1263,7 +1264,8 @@ s_LaTeX_Listener::~s_LaTeX_Listener()
 	_closeBlock();
 	_closeSection();
 	_handleDataItems();
-	
+	DELETEP(m_pTableHelper);
+
 	m_pie->write("\n\\end{document}\n");
 }
 
@@ -1517,8 +1519,8 @@ bool s_LaTeX_Listener::populateStrux(PL_StruxDocHandle sdh,
 
 	case PTX_EndEndnote:
 	default:
-		UT_ASSERT_NOT_REACHED();
-		return false;
+		UT_ASSERT(UT_TODO);
+		return true;
 	}
 }
 
@@ -1597,14 +1599,9 @@ static int wvConvertUnicodeToLaTeX(U16 char16,char*& out)
 	
 	Best way to do it until LaTeX is Unicode enabled 
 	(Omega project).
-	-- MV 4.7.2000
- 
-	We use a separate if-statement here ... the 'case range'
-	construct is gcc specific :-(  -- MV 13/07/2000
+	-- MV 4.7.2000 
 	*/
-
-	//if ( (char16 >= 0xa0) && (char16 <= 0xff) ) 
-	{
+	
 	switch(char16)
 		{
 		/* Fix up these as math characters: */
@@ -1626,11 +1623,6 @@ static int wvConvertUnicodeToLaTeX(U16 char16,char*& out)
 		case 0xdc:
 			printf("\\ldots{}");
 			return(1);
-		}
-		return(0);
-	}
-	switch(char16)
-		{
 		case 37:
 			printf("\\%%");
 			return(1);
@@ -2446,6 +2438,112 @@ static int wvConvertUnicodeToLaTeX(U16 char16,char*& out)
 		case 0x017E:
 			printf("\\v{z}");  /* z with caron */
 			return(1);
+	/* German and Spanish characters as well as some specials */
+                case 0x00EB:
+                        printf("\\\"{e}");
+                        return(1);
+                case 0x00CB:
+                        printf("\\\"{E}");
+                        return(1);
+                case 0x00F6:
+                        printf("\\\"{o}");
+                        return(1);
+                case 0x00E4:
+                        printf("\\\"{a}");
+                        return(1);
+                case 0x00FC:
+                        printf("\\\"{u}");
+                        return(1);
+                case 0x00C4:
+                        printf("\\\"{A}");
+                        return(1);
+                case 0x00D6:
+                        printf("\\\"{O}");
+                        return(1);
+#if 0
+                case 0x00DC:
+                        printf("\\\"{U}");
+                        return(1);
+#endif
+                case 0x00DF:
+                        printf("\\ss{}");
+                        return(1);
+                case 0x00E9: /* e with acute */
+                        printf("\\\'{e}");
+                        return(1);
+                case 0x00C9: /* E with acute */
+                        printf("\\\'{E}");
+                        return(1);
+                case 0x00E8: /* e with grave */
+                        printf("\\`{e}");
+                        return(1);
+                case 0x00C8: /* E with grave */
+                        printf("\\`{E}");
+                        return(1);
+                case 0x00FD: /* y with acute */
+                        printf("\\\'{y}");
+                        return(1);
+                case 0x00DD: /* Y with acute */
+                        printf("\\\'{Y}");
+                        return(1);
+                case 0x00F8: /* o with stroke */
+                        printf("{\\o}");
+                        return(1);
+                case 0x00D8: /* O with stroke */
+                        printf("{\\O}");
+                        return(1);
+                case 0x00E0: /* a with grave */
+                        printf("\\`{a}");
+                        return(1);
+                case 0x00C0: /* A with grave */
+                        printf("\\`{A}");
+                        return(1);
+                case 0x00E1: /* a with acute */
+                        printf("\\\'{a}");
+                        return(1);
+                case 0x00ED: /* i with acute */
+                        printf("\\\'{i}");
+                        return(1);
+                case 0x00FA: /* u with acute */
+                        printf("\\\'{u}");
+                        return(1);
+                case 0x00F2: /* o with grave */
+                        printf("\\`{o}");
+                        return(1);
+                case 0x00F3: /* o with acute */
+                        printf("\\\'{o}");
+                        return(1);
+                case 0x00F1: /* n with tilde */
+                        printf("\\~{n}");
+                        return(1);
+                case 0x00C1: /* A with acute */
+                        printf("\\\'{A}");
+                        return(1);
+                case 0x00CD: /* I with acute */
+                        printf("\\\'{I}");
+                        return(1);
+                case 0x00DA: /* U with acute */
+                        printf("\\\'{U}");
+                        return(1);
+                case 0x00D3: /* O with acute */
+                        printf("\\\'{O}");
+                        return(1);
+                case 0x00E7: /* c with cedilla */
+                        printf("\\c{c}");
+                        return(1);
+                case 0x00C7: /* C with cedilla */
+                        printf("\\c{C}");
+                        return(1);
+                case 0x00D1: /* N with tilde */
+                        printf("\\~{N}");
+                        return(1);
+                case 0x00A1: /* inverted exclamation mark */
+                        printf("!`");
+                        return(1);
+                case 0x00BF: /* inverted question mark */
+                        printf("?`");
+                        return(1);
+
 
 	/* Windows specials (MV 4.7.2000). More could be added. 
 	See http://www.hut.fi/u/jkorpela/www/windows-chars.html
@@ -2455,7 +2553,7 @@ static int wvConvertUnicodeToLaTeX(U16 char16,char*& out)
 			printf("---"); /* em-dash */
 			return(1);
 		case 0x2018:
-			printf("`");  /* left single quote, Win */
+			printf("{`}");  /* left single quote, Win */
 			return(1);
 		case 0x2019:
 			printf("'");  /* Right single quote, Win */
@@ -2464,7 +2562,7 @@ static int wvConvertUnicodeToLaTeX(U16 char16,char*& out)
 			printf("\\quotesinglbase{}");  /* single low 99 quotation mark */
 			return(1);
 		case 0x201C:
-			printf("``");  /* inverted double quotation mark */
+			printf("{``}");  /* inverted double quotation mark */
 			return(1);
 		case 0x201D:
 			printf("''");  /* double q.m. */
