@@ -20,22 +20,20 @@
 
 #include <stdlib.h>
 #include <string.h>
-#include <locale.h>
 
 #include "ut_string.h"
 #include "ut_bytebuf.h"
 #include "ut_base64.h"
+#include "ut_locale.h"
 #include "ut_units.h"
 #include "ut_debugmsg.h"
 #include "pt_Types.h"
 #include "ie_exp_WML.h"
 #include "pd_Document.h"
-#include "pp_AttrProp.h"
 #include "px_ChangeRecord.h"
 #include "px_CR_Object.h"
 #include "px_CR_Span.h"
 #include "px_CR_Strux.h"
-#include "ie_Table.h"
 #include "fd_Field.h"
 #include "xap_App.h"
 
@@ -44,67 +42,6 @@
 
 #include "ut_string_class.h"
 
-// first declare the listener
-
-/*****************************************************************/
-/*****************************************************************/
-
-class s_WML_Listener : public PL_Listener
-{
-public:
-	s_WML_Listener(PD_Document * pDocument,
-		       IE_Exp_WML * pie);
-	virtual ~s_WML_Listener();
-
-	virtual bool		populate(PL_StruxFmtHandle sfh,
-					 const PX_ChangeRecord * pcr);
-
-	virtual bool		populateStrux(PL_StruxDocHandle sdh,
-					      const PX_ChangeRecord * pcr,
-					      PL_StruxFmtHandle * psfh);
-
-	virtual bool		change(PL_StruxFmtHandle sfh,
-				       const PX_ChangeRecord * pcr);
-
-	virtual bool		insertStrux(PL_StruxFmtHandle sfh,
-					    const PX_ChangeRecord * pcr,
-					    PL_StruxDocHandle sdh,
-					    PL_ListenerId lid,
-					    void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
-								    PL_ListenerId lid,
-								    PL_StruxFmtHandle sfhNew));
-  
-        virtual bool		signal(UT_uint32 iSignal);
-
-protected:
-	void				_closeSection(void);
-	void				_closeBlock(void);
-	void				_closeSpan(void);
-	void				_openParagraph(PT_AttrPropIndex api);
-	void				_openSection(PT_AttrPropIndex api);
-	void				_openSpan(PT_AttrPropIndex api);
-	char *				_stripSuffix(const char* from, char delimiter);
-
-  void _openTable(PT_AttrPropIndex api) ;
-  void _closeTable();
-  void _openCell(PT_AttrPropIndex api);
-  void _closeCell();
-
-	void				_outputData(const UT_UCSChar * p, UT_uint32 length);
-	void				_handleDataItems();
-	
-	PD_Document *		m_pDocument;
-	IE_Exp_WML *		m_pie;
-	bool			m_bInBlock;
-	bool			m_bInSpan;
-        bool                 m_bWasSpace;
-
-	const PP_AttrProp*	m_pAP_Span;
-
-  UT_Vector		m_utvDataIDs;	// list of data ids for image enumeration
-  ie_Table mTableHelper;
-};
-
 /*****************************************************************/
 /*****************************************************************/
 
@@ -161,10 +98,8 @@ UT_Error IE_Exp_WML::_writeDocument(void)
 		return UT_IE_NOMEMORY;
 	if (!getDoc()->tellListener(static_cast<PL_Listener *>(m_pListener)))
 		return UT_ERROR;
-	delete m_pListener;
+	DELETEP(m_pListener);
 
-	m_pListener = NULL;
-	
 	return ((m_error) ? UT_IE_COULDNOTWRITE : UT_OK);
 }
 
@@ -173,36 +108,43 @@ UT_Error IE_Exp_WML::_writeDocument(void)
 
 s_WML_Listener::s_WML_Listener(PD_Document * pDocument,
 			       IE_Exp_WML * pie)
-  : mTableHelper(pDocument)
+	: mTableHelper(pDocument),
+	m_pDocument(pDocument),
+	m_pie(pie),
+	m_bInBlock(false),
+	m_bInSpan(false),
+	m_bInAnchor(false),
+	m_bInHyperlink(false),
+	m_bInCell(false),
+	m_bInRow(false),
+	m_bInSection(false),
+	m_bInTable(false),
+	m_bPendingClose(false),
+	m_bWasSpace(false),
+	m_iCards(1),
+	m_iTableDepth(0)
 {
-	m_pDocument = pDocument;
-	m_pie = pie;
-	m_bInBlock = false;
-	m_bInSpan = false;
-	m_bWasSpace = false;
-
 	m_pie->write("<!DOCTYPE wml PUBLIC \"-//PHONE.COM//DTD WML 1.1//EN\"\n");
-	m_pie->write("\t\"http://www.phone.com/dtd/wml11.dtd\" >\n");
+	m_pie->write("\t\"http://www.openwave.com/dtd/wml11.dtd\" >\n");
 
 	/* keep ads to a minimum. size is at a premium */
 	m_pie->write("<!-- This WML file was created by AbiWord -->\n");
 	m_pie->write("<!-- See http://www.abisource.com/ -->\n\n");
-
 	m_pie->write("<wml>\n");
-	m_pie->write("<card>\n");
+	_handleMetaData();
 }
 
 s_WML_Listener::~s_WML_Listener()
 {
-	_closeSpan();
-	_closeBlock();
 	_closeSection();
+
+	if(m_bPendingClose)
+		m_pie->write("</card>\n");
+
+	m_pie->write("</wml>\n");
 	_handleDataItems();
 
 	UT_VECTOR_FREEALL(char*, m_utvDataIDs);
-
-	m_pie->write("</card>\n");
-	m_pie->write("</wml>\n");
 }
 
 /*****************************************************************/
@@ -223,54 +165,114 @@ char *s_WML_Listener::_stripSuffix(const char* from, char delimiter)
     return fremove_s;
 }
 
+void s_WML_Listener::_closeAnchor()
+{
+	if(!m_bInAnchor)
+	{
+		return;
+	}
+
+	m_pie->write("</anchor>");
+	m_bInAnchor = false;
+}
+
+void s_WML_Listener::_closeHyperlink()
+{
+	if(!m_bInHyperlink)
+	{
+		return;
+	}
+
+	m_pie->write("</a>");
+	m_bInHyperlink = false;
+}
+
 void s_WML_Listener::_closeTable()
 {
-  m_pie->write("</table>\n");
+	if(!m_bInTable)
+	{
+		return;
+	}
+
+	_closeCell();
+	_closeRow();
+	m_pie->write("</table>\n</p>\n");
+	m_bInTable = false;
+}
+
+void s_WML_Listener::_closeRow()
+{
+	if(!m_bInRow || !m_bInTable)
+	{
+		return;
+	}
+
+	m_pie->write("</tr>\n");
+	m_bInRow = false;
 }
 
 void s_WML_Listener::_closeCell()
 {
-  m_pie->write("</td>\n");
+	if(!m_bInCell || !m_bInTable)
+	{
+		return;
+	}
 
-  if ( mTableHelper.getNumCols () == mTableHelper.getRight () )
-    {
-      // logical end of a row
-      m_pie->write("</tr>\n");
-    } 
+	m_pie->write("</td>\n");
+	m_bInCell = false;
+
+	if ( mTableHelper.getNumCols () == mTableHelper.getRight () )
+	{
+		// logical end of a row
+		_closeRow();
+	} 
 }
 
 void s_WML_Listener::_openTable(PT_AttrPropIndex api)
 {
-  UT_sint32 nCols = mTableHelper.getNumCols();
+	if(!m_bInSection)
+		_openSection(api);
+
+	if(m_bInTable)
+	{
+		return; //no nested tables
+	}
+
+	UT_sint32 nCols = mTableHelper.getNumCols();
   
-  UT_String tableSpec = UT_String_sprintf("<table wml:columns=\"%d\">\n",nCols);
-  m_pie->write(tableSpec.c_str(), tableSpec.size());
+	UT_UTF8String tableSpec = UT_UTF8String_sprintf("<p>\n<table columns=\"%d\">\n",nCols);
+	m_pie->write(tableSpec.utf8_str(), tableSpec.size());
+	m_bInTable = true;
 }
 
-void s_WML_Listener::_openCell(PT_AttrPropIndex api)
+void s_WML_Listener::_openCell(void)
 {
-  UT_sint32 rowspan = 1, colspan = 1;
-  
-  rowspan = mTableHelper.getBot() - mTableHelper.getTop();
-  colspan = mTableHelper.getRight() - mTableHelper.getLeft();
-  
-  if (mTableHelper.getLeft() == 0)
-    {
-      // beginning of a new row
-      m_pie->write("<tr>\n");
-    }
+	if(!m_bInTable)
+	{
+		return;
+	}
 
-  UT_String td ("<td");
-  
-  if (rowspan > 1)
-    td += UT_String_sprintf(" rowspan=\"%d\"", rowspan);
-  
-  if (colspan > 1)
-    td += UT_String_sprintf(" colspan=\"%d\"", colspan);
-  
-  td += ">\n";
-  
-  m_pie->write(td.c_str());
+	//note: wml 1.1 doesn't support colspan or rowspan
+
+	if(mTableHelper.isNewRow())
+	{
+		// beginning of a new row
+		_closeCell();
+		_closeRow();
+		_openRow();
+	}
+
+	m_pie->write("<td>");
+	m_bInCell = true;
+}
+
+void s_WML_Listener::_openRow(void)
+{
+	if(!m_bInRow)
+	{
+		m_pie->write("<tr>\n");
+		m_bInRow = true;
+	}
 }
 
 /*****************************************************************/
@@ -278,36 +280,79 @@ void s_WML_Listener::_openCell(PT_AttrPropIndex api)
 
 void s_WML_Listener::_closeSection(void)
 {
-  // wml is simple: 1 section per document
-  // we could get fancy with cards later
-  return;
+	_closeSpan();
+	_closeBlock();
+
+	if(m_bInSection)
+		m_bPendingClose = true;
 }
 
-void s_WML_Listener::_openSection(PT_AttrPropIndex /* api*/)
+void s_WML_Listener::_openSection(PT_AttrPropIndex api)
 {
-  // wml is simple: 1 section per document
-  // we could get fancy with cards later
-  return;
+	const PP_AttrProp * pAP = NULL;
+	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+	const XML_Char * szValue = NULL;
+
+	if(pAP && bHaveProp && (pAP->getAttribute("strux-image-dataid", szValue)))
+	{
+		if(!UT_strcmp("",szValue))  // occurs with word documents
+			return;
+
+		_openSpan(api);
+		_handlePositionedImage(api);
+		_closeSpan();
+		return; //don't open a new section
+	}
+
+	if(m_bPendingClose)
+	{
+		m_iCards++;
+		m_pie->write(UT_UTF8String_sprintf("<do type=\"accept\" label=\"Next\"><go href=\"#card%d\"/></do>\n", m_iCards).utf8_str());
+		m_pie->write("</card>\n");
+		m_bInSection = false;
+		m_bPendingClose = false;
+	}
+
+	if(!m_bInSection)
+	{
+		m_pie->write(UT_UTF8String_sprintf("<card id=\"card%d\" ordered=\"true\">\n", m_iCards).utf8_str());
+		m_bInSection = true;
+	}
 }
 
 void s_WML_Listener::_closeBlock(void)
 {
-	if (!m_bInBlock)
+	_closeAnchor();
+	_closeHyperlink();
+
+	if(!m_bInBlock)
 	{
 		return;
 	}
+	if(!m_bInTable)
+	{
+		m_pie->write("</p>\n");
+	}
 
-	m_pie->write("</p>\n");
 	m_bInBlock = false;
 	return;
 }
 
 void s_WML_Listener::_openParagraph(PT_AttrPropIndex api)
 {
-        UT_DEBUGMSG(("OpenParagraph called!\n"));
+	xxx_UT_DEBUGMSG(("WML Export: OpenParagraph called!\n"));
 
 	const PP_AttrProp * pAP = NULL;
 	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+
+	if(!m_bInSection)
+		_openSection(api);
+
+	if (m_bInTable) //<p> tags aren't allowed inside of table-related tags
+	{
+		m_bInBlock = true;
+		return;
+	}
 	
 	if (bHaveProp && pAP)
 	{
@@ -363,113 +408,50 @@ bool s_WML_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 
 	case PX_ChangeRecord::PXT_InsertObject:
 		{
-			const PX_ChangeRecord_Object * pcro =
-				static_cast<const PX_ChangeRecord_Object *> (pcr);
-			const XML_Char* szValue;
-			UT_String buf;
-
-			fd_Field* field;
+			const PX_ChangeRecord_Object * pcro = static_cast<const PX_ChangeRecord_Object *> (pcr);
 			PT_AttrPropIndex api = pcr->getIndexAP();
-			const PP_AttrProp * pAP = NULL;
-			bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
 
 			switch (pcro->getObjectType())
 			{
 			case PTO_Image:
-
-				if(bHaveProp && pAP && pAP->getAttribute("dataid", szValue))
-				{
-					char* dataid = strdup(static_cast<const char*>(szValue));
-
-					m_utvDataIDs.push_back(dataid);
-
-					char * temp = _stripSuffix(UT_basename(szValue), '_');
-					char * fstripped = _stripSuffix(temp, '.');
-					FREEP(temp);
-					UT_String_sprintf(buf, "%s.png", fstripped);
-					FREEP(fstripped);
-					m_pie->write("<img alt=\"AbiWord Image ");
-					m_pie->write(buf);
-					m_pie->write("\" src=\"");
-					m_pie->write(UT_basename(m_pie->getFileName()));
-					m_pie->write("_data/");
-					m_pie->write(buf);
-					m_pie->write("\" ");
-					const XML_Char * szWidth = 0;
-					const XML_Char * szHeight = 0;
-
-					const char * old_locale = setlocale(LC_NUMERIC, "C");
-
-					if(pAP->getProperty("width", szWidth) &&
-					   pAP->getProperty("height", szHeight))
-					  {
-					    if(szWidth)
-					      {
-						UT_String_sprintf(buf, "%f", UT_convertToDimension(szWidth, DIM_PX));
-						m_pie->write (" width=\"");
-						m_pie->write (buf);
-						m_pie->write ("\" ");
-					      }
-					    if(szHeight)
-					      {
-						UT_String_sprintf(buf, "%f", UT_convertToDimension(szHeight, DIM_PX));
-						m_pie->write (" height=\"");
-						m_pie->write (buf);
-						m_pie->write ("\" ");
-					      }
-					  }
-					
-					setlocale(LC_NUMERIC, old_locale);
-
-					m_pie->write(" />\n");
-				}
+			{
+				_handleImage(api);
 				return true;
+			}
 
 			case PTO_Field:
-				if(bHaveProp && pAP && pAP->getAttribute("type", szValue))
-				{
-					field = pcro->getField();
-
-					if(UT_strcmp(szValue, "list_label") != 0)
-					{
-						m_pie->write(szValue);
-					}
-				}
+			{
+				_handleField(pcro, api);
 				return true;
+			}
 
 			case PTO_Hyperlink:
-				if(bHaveProp && pAP && pAP->getAttribute("xlink:href", szValue))
-				{
-					m_pie->write("<a href=\"");
-					m_pie->write(szValue);
-					m_pie->write("\">");
-				}
-				else
-				{
-					m_pie->write("</a>");
-				}
+			{
+				_handleHyperlink(api);
 				return true;
+			}
 
 			case PTO_Bookmark:
-				if(bHaveProp && pAP && pAP->getAttribute("type", szValue))
-				{
-					if( UT_XML_stricmp(szValue, "start") == 0 )
-					{
-						pAP->getAttribute("name", szValue);
-						m_pie->write("<anchor id=\"");
-						m_pie->write(szValue);
-						m_pie->write("\">");
-					}
-					else
-					{
-						m_pie->write("</anchor>");
-					}
-				}
+			{
+				_handleBookmark(api);
 				return true;
+			}
+
+			case PTO_Embed:
+			{
+				_handleEmbedded(api);
+				return true;
+			}
+
+			case PTO_Math:
+			{
+				_handleMath(api);
+				return true;
+			}
 
 			default:
-				UT_ASSERT(0);
-				return false;
+				UT_ASSERT(UT_TODO);
+				return true;
 			}
 		}
 
@@ -477,8 +459,8 @@ bool s_WML_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 		return true;
 		
 	default:
-		UT_ASSERT(0);
-		return false;
+		UT_ASSERT(UT_TODO);
+		return true;
 	}
 }
 
@@ -492,55 +474,95 @@ bool s_WML_Listener::populateStrux(PL_StruxDocHandle sdh,
 
 	switch (pcrx->getStruxType())
 	{
+	case PTX_Section:
+	case PTX_SectionFrame:
+	case PTX_SectionHdrFtr:
+	{
+		if(!m_bInTable)
+		{
+			_closeSection();
+			_openSection(pcr->getIndexAP());
+		}
+		return true;
+	}
+
 	case PTX_Block:
-	  {
+	{
 		_closeSpan();
 		_closeBlock();
 		_openParagraph(pcr->getIndexAP());
 		return true;
-	  }
+	}
 
 	case PTX_SectionTable:
-	  {
-	    mTableHelper.OpenTable(sdh,pcr->getIndexAP()) ;
-	    _closeSpan();
-	    _closeBlock();
-	    _openTable(pcr->getIndexAP());
-	    return true ;
-	  }
-	case PTX_EndTable:
-	  {
-	    _closeBlock();
-	    _closeTable();
-	    mTableHelper.CloseTable();
-	    return true ;
-	  }
-	case PTX_SectionCell:
-	  {
-	    mTableHelper.OpenCell(pcr->getIndexAP()) ;
-	    _closeSpan();
-	    _closeBlock();
-	    _openCell(pcr->getIndexAP());
-	    return true ;
-	  }
-	case PTX_EndCell:
-	  {
-	    _closeBlock();
-	    _closeCell();
-	    mTableHelper.CloseCell();
+	{
+		m_iTableDepth++;
+		if(m_iTableDepth == 1)
+		{
+		    _closeSpan();
+		    _closeBlock();
+		    mTableHelper.OpenTable(sdh,pcr->getIndexAP());
+		    _openTable(pcr->getIndexAP());
+		}
 	    return true;
-	  }
+	}
+
+	case PTX_EndTable:
+	{
+		m_iTableDepth--;
+		if(m_iTableDepth == 0)
+		{
+		    _closeBlock();
+		    _closeTable();
+	    	mTableHelper.CloseTable();
+		}
+	    return true;
+	}
+
+	case PTX_SectionCell:
+	{
+		if(m_iTableDepth == 1)
+		{
+		    mTableHelper.OpenCell(pcr->getIndexAP()) ;
+		    _closeSpan();
+	    	_closeBlock();
+		    _openCell();
+		}
+	    return true;
+	}
+
+	case PTX_EndCell:
+	{
+		if(m_iTableDepth == 1)
+		{
+		    _closeBlock();
+		    _closeCell();
+	    	mTableHelper.CloseCell();
+		}
+	    return true;
+	}
 
-	case PTX_SectionEndnote:
-	case PTX_SectionHdrFtr:
-	case PTX_Section:
 	case PTX_EndFrame:
-	case PTX_EndMarginnote:
-	case PTX_EndFootnote:
-	case PTX_SectionFrame:
-	case PTX_SectionMarginnote:
-	case PTX_SectionFootnote:
+	{
+		if(!m_bInTable)
+		{
+			_closeSection();
+		}
+		return true;
+	}
+
 	case PTX_EndEndnote:
+	case PTX_EndFootnote:
+	case PTX_EndTOC:
+	case PTX_SectionEndnote:
+	case PTX_SectionFootnote:
+	case PTX_SectionTOC:
+	{
+		return true;
+	}
+
+	case PTX_EndMarginnote:
+	case PTX_SectionMarginnote:
 	default:
 		UT_ASSERT(UT_TODO);
 		return true;
@@ -572,7 +594,451 @@ bool s_WML_Listener::signal(UT_uint32 /* iSignal */)
 	return false;
 }
 
-void s_WML_Listener::_handleDataItems()
+/*****************************************************************/
+/*****************************************************************/
+
+void s_WML_Listener::_handleMetaData()
+{
+	UT_UTF8String metaProp("");
+	m_pie->write("<head>\n");
+
+	#define WRITEMETA(name)\
+		metaProp.escapeXML();\
+		if(metaProp.length())\
+			m_pie->write(UT_UTF8String_sprintf("<meta name=\"%s\" content=\"%s\"/>\n", name, metaProp.utf8_str()).utf8_str());\
+
+	if (m_pDocument->getMetaDataProp (PD_META_KEY_TITLE, metaProp) && metaProp.size())
+	{
+		WRITEMETA("title");
+	}
+	if (m_pDocument->getMetaDataProp (PD_META_KEY_CREATOR, metaProp) && metaProp.size())
+	{
+		WRITEMETA("author");
+	}
+	if (m_pDocument->getMetaDataProp (PD_META_KEY_SUBJECT, metaProp) && metaProp.size())
+	{
+		WRITEMETA("subject");
+	}
+	if (m_pDocument->getMetaDataProp (PD_META_KEY_DESCRIPTION, metaProp) && metaProp.size())
+	{
+		WRITEMETA("description");
+	}
+	if (m_pDocument->getMetaDataProp (PD_META_KEY_PUBLISHER, metaProp) && metaProp.size())
+	{
+		WRITEMETA("publisher");
+	}
+	if (m_pDocument->getMetaDataProp (PD_META_KEY_CONTRIBUTOR, metaProp) && metaProp.size())
+	{
+		WRITEMETA("contributor");
+	}
+	if (m_pDocument->getMetaDataProp (PD_META_KEY_DATE, metaProp) && metaProp.size())
+	{
+		WRITEMETA("date");
+	}
+	if (m_pDocument->getMetaDataProp (PD_META_KEY_SOURCE, metaProp) && metaProp.size())
+	{
+		WRITEMETA("source");
+	}
+	if (m_pDocument->getMetaDataProp (PD_META_KEY_RELATION, metaProp) && metaProp.size())
+	{
+		WRITEMETA("relation");
+	}
+	if (m_pDocument->getMetaDataProp (PD_META_KEY_COVERAGE, metaProp) && metaProp.size())
+	{
+		WRITEMETA("coverage");
+	}
+	if (m_pDocument->getMetaDataProp (PD_META_KEY_RIGHTS, metaProp) && metaProp.size())
+	{
+		WRITEMETA("rights");
+	}
+	if (m_pDocument->getMetaDataProp (PD_META_KEY_KEYWORDS, metaProp) && metaProp.size())
+	{
+		WRITEMETA("keywords");
+	}
+
+	#undef WRITEMETA
+
+	m_pie->write("</head>\n");
+}
+
+/*****************************************************************/
+/*****************************************************************/
+
+void s_WML_Listener::_handleEmbedded(PT_AttrPropIndex api)
+{
+	//this should never happen, but it doesn't hurt to make sure
+	if((m_bInTable && (!m_bInRow || !m_bInCell)))
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		return;
+	}
+
+	const XML_Char* szValue = 0;
+	const PP_AttrProp * pAP = NULL;
+	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+
+	UT_return_if_fail(bHaveProp && pAP && pAP->getAttribute("dataid", szValue))
+
+	UT_UTF8String buf = "snapshot-png-";
+	buf += szValue;
+
+	char* dataid = UT_strdup(buf.utf8_str());
+	m_utvDataIDs.push_back(dataid);
+	buf += ".png";
+
+	m_pie->write("<img alt=\"AbiWord Chart\" src=\""); //an alt attribute is required in WML
+	m_pie->write(UT_basename(m_pie->getFileName()));
+	m_pie->write("_data/");
+	m_pie->write(buf.utf8_str());
+	m_pie->write("\"");
+
+	UT_LocaleTransactor t(LC_NUMERIC, "C");
+
+	if(pAP->getProperty("height", szValue))
+	{
+		buf.clear();
+		UT_UTF8String_sprintf(buf, "%f", UT_convertToDimension(szValue, DIM_PX));
+		m_pie->write(" height=\"");
+		m_pie->write(buf.utf8_str());
+		m_pie->write("\"");
+	}
+	if(pAP->getProperty("width", szValue))
+	{
+		buf.clear();
+		UT_UTF8String_sprintf(buf, "%f", UT_convertToDimension(szValue, DIM_PX));
+		m_pie->write(" width=\"");
+		m_pie->write(buf.utf8_str());
+		m_pie->write("\"");
+	}
+	if(pAP->getProperty("lang", szValue))
+	{
+		m_pie->write(" xml:lang=\"");
+		m_pie->write(szValue);
+		m_pie->write("\"");
+	}
+
+	m_pie->write("/>");
+}
+
+/*****************************************************************/
+/*****************************************************************/
+
+void s_WML_Listener::_handleField(const PX_ChangeRecord_Object * pcro, PT_AttrPropIndex api)
+{
+	//this can happen with certain .doc files
+	if((m_bInTable && (!m_bInRow || !m_bInCell)))
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		return;
+	}
+
+	const PP_AttrProp * pAP = NULL;
+	const XML_Char* szValue = NULL;
+	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+
+	UT_return_if_fail(bHaveProp && pAP && pAP->getAttribute("type", szValue));
+
+	UT_UTF8String buf;
+	fd_Field * field = pcro->getField();
+
+	if(UT_strcmp(szValue, "list_label") != 0)
+	{
+		buf = field->getValue();  //retrieve the field text
+		buf.escapeXML();
+
+		if(buf.length())
+		{
+			m_pie->write(buf.utf8_str());
+		}
+	}
+}
+
+/*****************************************************************/
+/*****************************************************************/
+
+void s_WML_Listener::_handleBookmark(PT_AttrPropIndex api)
+{
+	//this can happen with certain .doc files
+	if((m_bInTable && (!m_bInRow || !m_bInCell)))
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		return;
+	}
+
+	const PP_AttrProp * pAP = NULL;
+	const XML_Char* szValue = NULL;
+	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+	UT_UTF8String buf;
+
+	if(bHaveProp && pAP && pAP->getAttribute("type", szValue))
+	{
+		_closeAnchor(); //nested anchors are not allowed
+
+		if(UT_strcmp(szValue, "start") == 0 && (m_bInHyperlink == 0) && pAP->getAttribute("name", szValue))
+		{
+			buf = szValue;
+			buf.escapeXML();
+
+			if(buf.length())
+			{
+				m_pie->write("<anchor id=\"");
+				m_pie->write(buf.utf8_str());
+				m_pie->write("\">");
+				m_bInAnchor = true;
+			}
+		}
+	}
+}
+
+/*****************************************************************/
+/*****************************************************************/
+
+void s_WML_Listener::_handleHyperlink(PT_AttrPropIndex api)
+{
+	//this can happen with certain .doc files
+	if((m_bInTable && (!m_bInRow || !m_bInCell)))
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		return;
+	}
+
+	const PP_AttrProp * pAP = NULL;
+	const XML_Char* szValue = NULL;
+	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+	UT_UTF8String buf;
+
+	if(bHaveProp && pAP && pAP->getAttribute("xlink:href", szValue))
+	{
+		buf = szValue;
+		buf.escapeURL();
+		_closeAnchor();
+		_closeHyperlink();
+
+		if(buf.length())
+		{
+			m_pie->write("<a href=\"");
+			m_pie->write(buf.utf8_str());
+			m_pie->write("\">");
+			m_bInHyperlink = true;
+		}
+	}
+	else
+	{
+		_closeHyperlink();
+	}
+}
+
+/*****************************************************************/
+/*****************************************************************/
+
+void s_WML_Listener::_handleMath(PT_AttrPropIndex api)
+{
+	//this should never happen, but it doesn't hurt to make sure
+	if((m_bInTable && (!m_bInRow || !m_bInCell)))
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		return;
+	}
+
+	const XML_Char* szValue = 0;
+	const PP_AttrProp * pAP = NULL;
+	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+
+	UT_return_if_fail(bHaveProp && pAP && pAP->getAttribute("dataid", szValue));
+
+	UT_UTF8String buf = "snapshot-png-";
+	buf += szValue;
+
+	char *dataid = UT_strdup(buf.utf8_str());
+	m_utvDataIDs.push_back(dataid);
+	buf += ".png";
+
+	m_pie->write("<img alt=\"AbiWord Equation\" src=\""); //an alt attribute is required in WML
+	m_pie->write(UT_basename(m_pie->getFileName()));
+	m_pie->write("_data/");
+	m_pie->write(buf.utf8_str());
+	m_pie->write("\"");
+
+	if(pAP->getProperty("lang", szValue))
+	{
+		m_pie->write(" xml:lang=\"");
+		m_pie->write(szValue);
+		m_pie->write("\"");
+	}
+
+	m_pie->write("/>");
+}
+
+/*****************************************************************/
+/*****************************************************************/
+
+void s_WML_Listener::_handleImage(PT_AttrPropIndex api)
+{
+	//this should never happen, but it doesn't hurt to make sure
+	if((m_bInTable && (!m_bInRow || !m_bInCell)))
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		return;
+	}
+
+	const PP_AttrProp * pAP = NULL;
+	const XML_Char* szValue = NULL;
+	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+
+	UT_return_if_fail(bHaveProp && pAP && pAP->getAttribute("dataid", szValue));
+
+	UT_UTF8String buf;
+	char * dataid = UT_strdup(static_cast<const char*>(szValue));
+	m_utvDataIDs.push_back(dataid);
+
+	char * temp = _stripSuffix(UT_basename(szValue), '_');
+	char * fstripped = _stripSuffix(temp, '.');
+	UT_UTF8String_sprintf(buf, "%s.png", fstripped);
+
+	FREEP(temp);
+	FREEP(fstripped);
+
+	m_pie->write("<img alt=\""); //an alt attribute is required in WML
+
+	if(pAP->getAttribute("alt", szValue))  // check for existing alt text
+	{
+		UT_UTF8String alt = szValue;
+		alt.escapeXML();
+		m_pie->write(alt.utf8_str());
+	}
+	else  // fall back to the file name
+	{
+		m_pie->write("AbiWord Image ");
+		m_pie->write(buf.utf8_str());
+	}
+
+	m_pie->write("\" src=\"");
+	m_pie->write(UT_basename(m_pie->getFileName()));
+	m_pie->write("_data/");
+	m_pie->write(buf.utf8_str());
+	m_pie->write("\"");
+
+	const XML_Char * szWidth = 0;
+	const XML_Char * szHeight = 0;
+	UT_LocaleTransactor t(LC_NUMERIC, "C");
+
+	if(pAP->getProperty("width", szWidth))
+	{
+		if(szWidth)
+		{
+			UT_UTF8String_sprintf(buf, "%f", UT_convertToDimension(szWidth, DIM_PX));
+			m_pie->write (" width=\"");
+			m_pie->write (buf.utf8_str());
+			m_pie->write ("\"");
+		}
+	}
+	if(pAP->getProperty("height", szHeight))
+	{
+		if(szHeight)
+		{
+			UT_UTF8String_sprintf(buf, "%f", UT_convertToDimension(szHeight, DIM_PX));
+			m_pie->write (" height=\"");
+			m_pie->write (buf.utf8_str());
+			m_pie->write ("\"");
+		}
+	}
+	if(pAP->getProperty("lang", szValue))
+	{
+		m_pie->write(" xml:lang=\"");
+		m_pie->write(szValue);
+		m_pie->write("\"");
+	}
+				
+	m_pie->write("/>");
+}
+
+/*****************************************************************/
+/*****************************************************************/
+
+void s_WML_Listener::_handlePositionedImage(PT_AttrPropIndex api)
+{
+	//this should never happen, but it doesn't hurt to make sure
+	if((m_bInTable && (!m_bInRow || !m_bInCell)))
+	{
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		return;
+	}
+
+	const PP_AttrProp * pAP = NULL;
+	const XML_Char* szValue = NULL;
+	bool bHaveProp = m_pDocument->getAttrProp(api,&pAP);
+
+	UT_return_if_fail(bHaveProp && pAP && pAP->getAttribute("strux-image-dataid", szValue));
+
+	UT_UTF8String buf;
+	char * dataid = UT_strdup(static_cast<const char*>(szValue));
+	m_utvDataIDs.push_back(dataid);
+
+	char * temp = _stripSuffix(UT_basename(szValue), '_');
+	char * fstripped = _stripSuffix(temp, '.');
+	UT_UTF8String_sprintf(buf, "%s.png", fstripped);
+
+	FREEP(temp);
+	FREEP(fstripped);
+
+	m_pie->write("<img alt=\""); //an alt attribute is required in WML
+
+	if(pAP->getAttribute("alt", szValue))  // check for existing alt text
+	{
+		UT_UTF8String alt = szValue;
+		alt.escapeXML();
+		m_pie->write(alt.utf8_str());
+	}
+	else  // fall back to the file name
+	{
+		m_pie->write("AbiWord Image ");
+		m_pie->write(buf.utf8_str());
+	}
+
+	m_pie->write("\" src=\"");
+	m_pie->write(UT_basename(m_pie->getFileName()));
+	m_pie->write("_data/");
+	m_pie->write(buf.utf8_str());
+	m_pie->write("\"");
+
+	const XML_Char * szWidth = 0;
+	const XML_Char * szHeight = 0;
+	UT_LocaleTransactor t(LC_NUMERIC, "C");
+
+	if(pAP->getProperty("width", szWidth))
+	{
+		if(szWidth)
+		{
+			UT_UTF8String_sprintf(buf, "%f", UT_convertToDimension(szWidth, DIM_PX));
+			m_pie->write (" width=\"");
+			m_pie->write (buf.utf8_str());
+			m_pie->write ("\"");
+		}
+	}
+	if(pAP->getProperty("height", szHeight))
+	{
+		if(szHeight)
+		{
+			UT_UTF8String_sprintf(buf, "%f", UT_convertToDimension(szHeight, DIM_PX));
+			m_pie->write (" height=\"");
+			m_pie->write (buf.utf8_str());
+			m_pie->write ("\"");
+		}
+	}
+	if(pAP->getProperty("lang", szValue))
+	{
+		m_pie->write(" xml:lang=\"");
+		m_pie->write(szValue);
+		m_pie->write("\"");
+	}
+				
+	m_pie->write("/>");
+}
+
+/*****************************************************************/
+/*****************************************************************/
+
+void s_WML_Listener::_handleDataItems(void)
 {
  	const char * szName;
 	const char * szMimeType;
@@ -602,7 +1068,7 @@ void s_WML_Listener::_handleDataItems()
 			UT_sint32 result = m_pDocument->getApp()->makeDirectory(fname.c_str(), 0750);
 			if (result < 0)
 			{
-				UT_DEBUGMSG(("Failed to create directory\n"));
+				UT_DEBUGMSG(("WML Export: Failed to create directory\n"));
 				/* we might want to return an error here, 
 				 * but I don't think so. */
 			}
@@ -646,7 +1112,7 @@ void s_WML_Listener::_handleDataItems()
 
 void s_WML_Listener::_openSpan(PT_AttrPropIndex api)
 {
-	if (!m_bInBlock)
+	if (!m_bInBlock || m_bInHyperlink || m_bInAnchor || (m_bInTable && (!m_bInRow || !m_bInCell)))
 	{
 		return;
 	}
@@ -656,50 +1122,24 @@ void s_WML_Listener::_openSpan(PT_AttrPropIndex api)
 
 	if (bHaveProp && pAP)
 	{
-		const XML_Char * szValue;
+		const XML_Char * szValue = NULL;
 
-		if (
-			(pAP->getProperty("font-weight", szValue))
-			&& !UT_strcmp(szValue, "bold")
-			)
+		if ((pAP->getProperty("font-weight", szValue))
+			&& !UT_strcmp(szValue, "bold"))
 		{
-		  m_pie->write("<b>");
+			m_pie->write("<b>");
 		}
 		
-		if (
-			(pAP->getProperty("font-style", szValue))
-			&& !UT_strcmp(szValue, "italic")
-			)
+		if ((pAP->getProperty("font-style", szValue))
+			&& !UT_strcmp(szValue, "italic"))
 		{
-		  m_pie->write("<i>");
+			m_pie->write("<i>");
 		}
-
 		
-		if (
-			(pAP->getProperty("text-decoration", szValue))
-			)
+		if ((pAP->getProperty("text-decoration", szValue)))
 		{
-			const XML_Char* pszDecor = szValue;
-
-			XML_Char* p;
-			if (!UT_cloneString((char *&)p, pszDecor))
-			{
-				// TODO outofmem
-			}
-			
-			UT_ASSERT(p || !pszDecor);
-			XML_Char*	q = strtok(p, " ");
-
-			while (q)
-			{
-				if (0 == UT_strcmp(q, "underline"))
-				{
-				      m_pie->write("<u>");
-				}
-
-				q = strtok(NULL, " ");
-			}
-			free(p);
+			if(strstr(szValue,"underline"))
+				m_pie->write("<u>");
 		}
 
 		// In my WML world...
@@ -724,7 +1164,7 @@ void s_WML_Listener::_openSpan(PT_AttrPropIndex api)
 
 void s_WML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 {
-	if (!m_bInBlock)
+	if (!m_bInBlock || (m_bInTable && (!m_bInRow || !m_bInCell)))
 	{
 		return;
 	}
@@ -734,6 +1174,7 @@ void s_WML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 	
 	UT_UTF8String sBuf;
 	const UT_UCSChar * pData;
+	m_bWasSpace = false;
 
 	for (pData=data; (pData<data+length); /**/)
 	{
@@ -762,7 +1203,7 @@ void s_WML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 			break;
 
 		case ' ':
-		case '\t':
+		case UCS_TAB:
 		  // try to honor multiple spaces
 		  // tabs get treated as a single space
 		  //
@@ -781,8 +1222,18 @@ void s_WML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 		  break;
 
 		default:
-		  sBuf.appendUCS4(pData, 1);
-		  pData++;
+		{
+			if(*pData < 0x20)  //invalid xml chars
+			{
+				pData++;
+			}
+			else
+			{
+				sBuf.appendUCS4(pData, 1);
+				pData++;
+			}
+			m_bWasSpace = false;
+		}
 		}
 	}
 
@@ -791,15 +1242,14 @@ void s_WML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 
 void s_WML_Listener::_closeSpan(void)
 {
-	if (!m_bInSpan)
+	if (!m_bInSpan || m_bInHyperlink || m_bInAnchor)
 		return;
 
 	const PP_AttrProp * pAP = m_pAP_Span;
 	
 	if (pAP)
 	{
-
-		const XML_Char * szValue;
+		const XML_Char * szValue = NULL;
 		
 		if (pAP->getProperty("text-position", szValue))
 		{
@@ -815,41 +1265,18 @@ void s_WML_Listener::_closeSpan(void)
 
 		if ((pAP->getProperty("text-decoration", szValue)))
 		{
-			const XML_Char* pszDecor = szValue;
-
-			XML_Char* p;
-			if (!UT_cloneString((char *&)p, pszDecor))
-			{
-				// TODO outofmem
-			}
-			
-			UT_ASSERT(p || !pszDecor);
-			XML_Char*	q = strtok(p, " ");
-
-			while (q)
-			{
-				if (0 == UT_strcmp(q, "underline"))
-				{
-				      m_pie->write("</u>");
-				}
-
-				q = strtok(NULL, " ");
-			}
-			free(p);
+			if(strstr(szValue,"underline"))
+				m_pie->write("</u>");				
 		}
 
-		if (
-			(pAP->getProperty("font-style", szValue))
-			&& !UT_strcmp(szValue, "italic")
-			)
+		if ((pAP->getProperty("font-style", szValue))
+			&& !UT_strcmp(szValue, "italic"))
 		{
 		  m_pie->write("</i>");
 		}
 		
-		if (
-			(pAP->getProperty("font-weight", szValue))
-			&& !UT_strcmp(szValue, "bold")
-			)
+		if ((pAP->getProperty("font-weight", szValue))
+			&& !UT_strcmp(szValue, "bold"))
 		{
 		  m_pie->write("</b>");
 		}
