@@ -30,7 +30,7 @@ wvAddCHPXFromBucket (CHP * achp, UPXF * upxf, STSH * stsh)
     fprintf (stderr, "\n");
     i = 0;
 #endif
-    while (i < upxf->cbUPX - 2)
+    while (i + 2 < upxf->cbUPX) /* is this check sufficient ?? */
       {
 	  sprm = bread_16ubit (upxf->upx.chpx.grpprl + i, &i);
 #ifdef SPRMTEST
@@ -112,8 +112,14 @@ void
 wvInitCHPFromIstd (CHP * achp, U16 istdBase, STSH * stsh)
 {
     wvTrace (("initing from %d\n", istdBase));
-    if (istdBase == istdNil)
-	wvInitCHP (achp);
+    if (istdBase == istdNil) {
+ 	wvInitCHP (achp);
+
+	/* Set the Nil style's fonts from the defaults. */
+	achp->ftcAscii = stsh->Stshi.rgftcStandardChpStsh[0];
+	achp->ftcFE = stsh->Stshi.rgftcStandardChpStsh[1];
+	achp->ftcOther = stsh->Stshi.rgftcStandardChpStsh[2];
+      }
     else
       {
 	  if (istdBase >= stsh->Stshi.cstd)
@@ -145,6 +151,7 @@ wvInitCHPFromIstd (CHP * achp, U16 istdBase, STSH * stsh)
 			    wvApplyCHPXFromBucket (achp,
 						   &(stsh->std[istdBase].
 						     grupe[0].chpx), stsh);
+				strncpy(achp->stylename,stsh->std[istdBase].xstzName, sizeof(achp->stylename));
 			    break;
 			}
 		  }
@@ -272,6 +279,8 @@ wvInitCHP (CHP * item)
     item->hpsBidi = 0;
     item->icoBidi = 0;
     item->lidBidi = 0;
+
+	item->stylename[0] = 0;
 }
 
 void
@@ -379,7 +388,9 @@ wvCopyCHP (CHP * dest, CHP * src)
     dest->ftcBidi = src->ftcBidi;
     dest->hpsBidi = src->hpsBidi;
     dest->icoBidi = src->icoBidi;
-    dest->lidBidi = src->lidBidi; 
+    dest->lidBidi = src->lidBidi;
+
+	strcpy(dest->stylename,src->stylename);
 }
 
 /*
@@ -428,7 +439,7 @@ wvCopyCHPX (CHPX * dest, CHPX * src)
 	dest->grpprl = (U8 *) wvMalloc (dest->cbGrpprl);
     else
 	dest->grpprl = NULL;
-    if (dest->grpprl == NULL)
+    if (dest->grpprl == NULL || src->grpprl == NULL)
 	return;
     for (i = 0; i < dest->cbGrpprl; i++)
 	dest->grpprl[i] = src->grpprl[i];
@@ -560,7 +571,7 @@ wvMergeCHPXFromBucket (CHPX * dest, UPXF * src)
 	      *dpointer++ = *pointer++;
 
 	  testp = NextNode (&tree, testn);
-	  DeleteNode (&tree, testn);
+	  wvDeleteNode (&tree, testn);
 	  testn = testp;
       }
     wvFree (dest->grpprl);
@@ -656,47 +667,72 @@ wvUpdateCHPXBucket (UPXF * src)
  */
 
 int
-wvAssembleSimpleCHP (wvVersion ver, CHP * achp, U32 fc, CHPX_FKP * fkp,
+wvAssembleSimpleCHP (wvVersion ver, CHP * achp, const PAP * apap, U32 fc, CHPX_FKP * fkp,
 		     STSH * stsh)
 {
     CHPX *chpx;
-    int index, i;
+    int index;
     UPXF upxf;
     int ret = 0;
     U16 tistd;
 
-    /* initialize CHP to para's stylesheet character properties
-       * this should have resolved all the other stylesheet dependencies
+	
+    /* initialize CHP to para's stylesheet character properties this
+       * should have resolved all the other stylesheet dependencies
        * for us, when the stsh's were initialized. */
 
-    /* before this function was called, achp->istd should have
-       * been set to the current paragraph properties' stylesheet */
-    tistd = achp->istd;
-    wvInitCHPFromIstd (achp, achp->istd, stsh);
-    achp->istd = tistd;
+	wvInitCHPFromIstd (achp, apap->istd, stsh);
+	/* having done this, we want to set the achp->istd to nil, since
+    any char istd value stored in the para style is not applicable
+    (its a para style */
+	achp->istd = istdNil;
+	tistd = istdNil;
+	
+ apply_chpx:
+	/* get CHPX */
+	if(fkp)
+	{
+		/* the PAPX version of the function only looks at rgfc's, which are
+		 * the same for CHPX and PAPX FKPs, so we'll reuse the function */
+		index = wvGetIndexFCInFKP_PAPX ((PAPX_FKP *) fkp, fc);
 
-    /*index is the i in the text above */
-    /* the PAPX version of the function only looks at rgfc's, which are
-       * the same for CHPX and PAPX FKPs, so we'll reuse the function */
-    index = wvGetIndexFCInFKP_PAPX ((PAPX_FKP *) fkp, fc);
+		wvTrace (("index is %d, using %d\n", index, index - 1));
 
-    wvTrace (("index is %d, using %d\n", index, index - 1));
+		chpx = &(fkp->grpchpx[index - 1]);
 
-    /* get CHPX */
-    chpx = &(fkp->grpchpx[index - 1]);
+		/* apply CHPX from FKP */
+		if ((chpx) && (chpx->cbGrpprl > 0))
+		{
+			ret = 1;
+			/* for (i = 0; i < chpx->cbGrpprl; i++) */
+			upxf.cbUPX = chpx->cbGrpprl;
+			upxf.upx.chpx.grpprl = chpx->grpprl;
+			if (ver == WORD8)
+				wvAddCHPXFromBucket (achp, &upxf, stsh);
+			else
+				wvAddCHPXFromBucket6 (achp, &upxf, stsh);
+		}
 
-    /* apply CHPX from FKP */
-    if ((chpx) && (chpx->cbGrpprl > 0))
-      {
-	  ret = 1;
-	  for (i = 0; i < chpx->cbGrpprl; i++)
-	      upxf.cbUPX = chpx->cbGrpprl;
-	  upxf.upx.chpx.grpprl = chpx->grpprl;
-	  if (ver == WORD8)
-	      wvAddCHPXFromBucket (achp, &upxf, stsh);
-	  else
-	      wvAddCHPXFromBucket6 (achp, &upxf, stsh);
-      }
+		if(achp->istd < stsh->Stshi.cstd) {
+		  if (0 != stsh->std[achp->istd].xstzName) {
+		    strncpy(achp->stylename,stsh->std[achp->istd].xstzName, sizeof(achp->stylename));
+		  }
+		  else {
+		    wvError (("trying to copy null string\n"));
+		  }
+		}
+
+		if(achp->istd != tistd)
+		{
+			/* the chpx contained instruction to apply character
+			   style: we have to start all over again, since we init
+			   the chp from the wrong (paragraph) style */
+			tistd = achp->istd;
+			wvInitCHPFromIstd (achp, achp->istd, stsh);
+			goto apply_chpx;
+		}
+	}
+	
     return (ret);
 }
 
@@ -714,7 +750,8 @@ wvGetCHPX (wvVersion ver, CHPX * item, U8 * page, U16 * pos)
     else
 	item->grpprl = NULL;
 
-    item->istd = 0;		/* I have no idea what to set this to... */
+    item->istd = 0;		/* I have no idea what to set this to... --
+						   nothing; the istd is contained in the grpprl*/
 
     for (i = 0; i < item->cbGrpprl; i++)
 	wvTrace (("chpx byte is %x\n", item->grpprl[i]));
