@@ -482,7 +482,7 @@ get_block_ptr (MsOle *f, BLP b, gboolean forwrite)
 
 	offset = (b+1)*BB_BLOCK_SIZE;
 	f->syswrap->lseek (f->file_des, offset, SEEK_SET, f->syswrap->closure);
-	f->syswrap->read (f->file_des, attr->data, BB_BLOCK_SIZE, f->syswrap->closure);
+	(f->syswrap->read) (f->file_des, attr->data, BB_BLOCK_SIZE, f->syswrap->closure);
 	attr->usage = 1;
 	attr->dirty = forwrite;
 
@@ -1791,7 +1791,7 @@ ms_ole_open_vfs (MsOle **fs, const char *name,
 		f->mem = g_new (guint8, BB_BLOCK_SIZE);
 
 		if (!f->mem ||
-		    f->syswrap->read (file, f->mem, BB_BLOCK_SIZE,
+		    (f->syswrap->read) (file, f->mem, BB_BLOCK_SIZE,
 				      f->syswrap->closure) == -1) {
 			g_warning ("Error reading header\n");
 			f->syswrap->close (file, f->syswrap->closure) ;
